#!/usr/bin/perl
# 
# Script to offer some vdr-functions to other network-clients.
# File-access is read only and any real changes are done by the vdr himself!
#
# Script is written by geronimo a-t vdrportal d-o-t de
# 
# COPYRIGHT (c) 2006 Creative Commons by-nc-sa some rights reserved.
# 		For details on copyright see:
# 				http://creativecommons.org/licenses/by-nc-sa
#
# There's no warranty at all! Use at your own responsability.
#
# INSTALL (i.e. at LinVDR):
# 1.	copy this script to /usr/bin
#
# 2.	add START_VDRPROXY="yes" to /etc/sysconfig
#
# 3.	add these (3) lines to /etc/init.d/rcStart.delayed
#  		if [ x$START_VDRPROXY = x"yes" ]; then
#  			execute vdrproxy.pl
#  		fi	
#
# 4. 	Enjoy!	
#
use strict;
use IO::Socket;
use Net::hostent;
use POSIX ":sys_wait_h";

my @allowedClients = qw(django17 localhost);	# place your client names here
my @epgClients = qw(django17);            # clients that like to receive a copy from epg
my $epgFN   = '/ramdisk/epg.data';			# direct RO-access to epg data
my $chnFN   = '/etc/vdr/channels.conf';	# direct RO-access to channels
my $timFN   = '/etc/vdr/timers.conf';		# direct RO-access to timers
my $port    = 7936;								# public port of this proxy
my $vdrPort = 2001;								# port of the vdr svdrp-interface
my $vdrHost = 'localhost';						# hostname of the vdr (should be localhost)
my $commands = {
  HELO => \&cHello		  
, GEPG => \&cGetEpg
, CGEP => \&cContGetEpg
, CHAN => \&cSetChannel
, GRAB => \&cGrab
, LSTC => \&cListChannel
, LSTT => \&cListTimer
, UPDT => \&cChgTimer
, NEWT => \&cNewTimer
, STOP => \&cStopSrv
};
my($client, $server, $hostinfo, $pid, $lastCheck, $channels, $debug, %knownHosts, $clientHost);

sub REAPER {
   my $child;
	
   while((my $waitedpid = waitpid(-1,WNOHANG)) > 0){
      logmsg( "reaped $waitedpid" . ($? ? " with exit $?" : ''));
      }
   $SIG{CHLD} = \&REAPER;
}

sub cHello      { 
	my $args = shift; 
	my $client = shift;
	
	logmsg('cHello( '.$args.' ) ...'."\n"); 
	print($client 'OLEH '.$args);
}

sub cGetEpg { 
	my $args = shift; 
	my $client = shift;
	my $fp;
	my @st = stat($epgFN);

	logmsg('cGetEpg() - time-check: ( '.$args.' / '.$st[9].' )');
	if($args >= $st[9]){
		logmsg('cGetEpg() - seems as client already got this one ...'."\n");
		print($client "\n\n");

		return;
		}
	logmsg('cGetEpg() - ok, ok, gonna do some smooth shift...'."\n");
	my $size = -s $epgFN;
	if(open($fp, $epgFN)){
		while(my $line = <$fp>){
			print($client $line);	  
			}
		print($client "\n\n");
		close($fp);	  
		}
	my $response = <$client>;
	logmsg('cGetEpg() - from client <'.$response.'>');	  
	if($response =~ /^(\d+)/){
		my $clientSize = $1;
	
		logmsg('cGetEpg(): size-match ('.$clientSize.' / '.$size.')');
		if($size eq $clientSize){ print($client 'OK'."\n\n");  }
		else							{ print($client 'NOK'."\n\n"); }
		}
	else{ print($client 'NOK'."\n\n"); }
}

sub cListChannel { 
	my $args = shift; 
	my $client = shift;
	my $fp;
	
	logmsg('cListChannel( '.$args.' ) ...'."\n"); 
	open($fp, $chnFN) or die 'could not access channel information!';
	while(my $line = <$fp>){
		print($client $line);	  
		}
	print($client "\n\n");
	close($fp);
}

sub cListTimer   { 
	my $args = shift; 
	my $client = shift;
	my $fp;
	
	logmsg('cListTimer( '.$args.' ) ...'."\n"); 
	open($fp, $timFN) or die 'could not access timer information!';
	while(my $line = <$fp>){
		print($client $line);	  
		}
	print($client "\n\n");
	close($fp);
}

sub doVdrCommand {
	my $client = shift;	  
	my $cmd    = shift;
	my $args   = shift;
	my $vdr    = IO::Socket::INET->new(Proto => 'tcp'
						 					   , PeerAddr => $vdrHost
											   , PeerPort => $vdrPort)
			  or die(print($client 'Oups, could not connect to vdr!'."\n"));

   $vdr->autoflush(1);
	my $line = <$vdr>;

	logmsg($line);
	print($vdr $cmd.' '.$args."\n");
	while($line = <$vdr>){
		print($client $line);	  
		last if not substr($line, 3,1) eq '-';
		}
	print($client "\n\n");
	print($vdr 'quit'."\n");
}

sub cSetChannel  { 
	my $args = shift; 
	my $client = shift;
	
	logmsg('cSetChannel( '.$args.' ) ...'."\n"); 
	doVdrCommand($client, 'chan', $args);
}

sub cGrab        { 
	my $args = shift; 
	my $client = shift;
	
	logmsg('cGrab( '.$args.' ) ...'."\n"); 
	doVdrCommand($client, 'grab', $args);
}

sub cChgTimer    { 
	my $args = shift; 
	my $client = shift;
	
	logmsg('cChgTimer( '.$args.' ) ...'."\n"); 
	doVdrCommand($client, 'modt', $args);
}

sub cNewTimer    { 
	my $args = shift; 
	my $client = shift;
	
	logmsg('cNewTimer( '.$args.' ) ...'."\n"); 
	doVdrCommand($client, 'newt', $args);
}

sub cStopSrv { 
	my $args   = shift; 
	my $client = shift;
	my $parent = shift; 
	
	logmsg('cStopSrv( '.$args.' ) ...'."\n");  

	kill("TERM", $parent); 
}

sub doClientCommand {
	my $pid = shift;	  
	my $client = shift;
   $client->autoflush(1);
	my $cMsg;

	logmsg('doClientCommand()... parent has pid #'.$pid) if $debug;
   while(not ($cMsg = <$client>)){};
	if($cMsg =~ /^([\S]+)\s*(.+)?$/){
		my $cmd  = uc($1);
		my $args = $2;

		logmsg('client sent command<'.$cmd.'> with args['.$args.']') if $debug;	  
		if(exists($commands->{$cmd})){
			$commands->{$cmd}($args, $client, $pid);
			}
		else{ logmsg('Oups, unknown command!'); }
		}
   close $client;
}

sub logmsg { my $msg = "@_"; $msg =~ s/\n$//; system('logger \''.$0.' '.$$.': '.$msg.'\''); }

#
# start of main
#
my $mainPid = $$;
foreach (@allowedClients){ $knownHosts{$_} = 1; }
while(1){
	$server = IO::Socket::INET->new(Proto     => 'tcp'
   	                           , LocalPort => $port
	                              , Listen    => SOMAXCONN
											, ReuseAddr => 1);
	if(not $server){
		logmsg('could NOT start server: '.$!);
		sleep(60);
		next;
		}
	$SIG{CHLD} = \&REAPER;

	logmsg('Server accepting clients at port #'.$port);
	while($client = $server->accept()){
		$hostinfo = gethostbyaddr($client->peeraddr);
		$clientHost = $hostinfo ? lc($hostinfo->name) : $client->peerhost;
	   logmsg('Connect from '.$clientHost);
		if(!defined($pid = fork)){ logmsg('cannot fork: '.$!); return; }
		elsif($pid){ close $client; }
		else { 
			if(exists($knownHosts{$clientHost}) and $knownHosts{$clientHost}){
				exit(doClientCommand($mainPid, $client)); 
				}
			logmsg($clientHost.' tried to connect, but is not allowed by "knownHosts"');
			exit(print($client 'Access denied!'."\n"));
			}
		}
	logmsg('Oups, server closed without intention. Gonna restart... ('.$!.')');
	sleep(60);					 
	}
