#!/usr/bin/perl

###############################################################################
# THIS SOFTWARE IS FREE SOFTWARE! IT IS LICENCED UNDER THE GNU PUBLIC LICENCE
#  (c) Dr. Peter Sebbel alias doc (peter@vdr-portal.de) 
# 
#
#  create_indexvdr.pl (c) 2003-2004 by  Dr. Peter Sebbel, a perl script for the generation of an index.vdr as
#  used by the VideoDiskRecorder (VDR)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; version 2 of the License
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##############################################################################

use strict;
use warnings;


my $EOF = 0; 		# All input processed ?
my $total_input = 0;
my $max_size = 2000000000; # Standard file size 
my $out_dir;
my $seqstart = pack ("H8", "000001B8");
my $picstart = pack ("H8", "00000100");
my %pictypes = (
    "001" =>  "I",
	"010" =>  "P",
	"011" =>  "B",
	);
my $FROM;
my @packet_buffer; 
my @packet_infos; 
my @video_payload_buffer;

my $index_file = "./index.vdr";
my $file_no      = 1;
my $file_offset  = 0;
my $current_file_size = 0;
my $index_only   = 0;
my $OFH;


#read_index_file();
parse_parameters();

open INDEXFH, ">$out_dir/$index_file" or die "Can not open outfile\n";    


if (! $index_only) {
    open $OFH, ">$out_dir/001.vdr" or die "could not open file for writing output\n";
    binmode $OFH;
}
    
process_PES_packets(); 

close INDEXFH;
if (! $index_only) {
    close $OFH;
}


sub readNextChunk {
        my $ret_buf = "";
        my $helpbuffer;
        my $bytes_to_read = 10000000;
        while ((length($ret_buf) < 10000000) && (sysread STDIN, $helpbuffer, (10000000 - length($ret_buf)), 0)) {
            $ret_buf .= $helpbuffer;
            $helpbuffer = "";
            #print "CHUNK: Got til now:" . length($ret_buf) . "\n";
        }
        $total_input += length($ret_buf);
        if (length($ret_buf) < 10000000) {
            print STDERR "\nRead all input, total is $total_input\n";
            $EOF = 1;
        }
        print STDERR "\nRead next chunk, total is $total_input\n";
        my $deb = unpack ("H8", substr($ret_buf,0,4));
        #print STDERR  "CHILD debug: $deb\n";
        return  \$ret_buf;
}

sub process_PES_packets	{
	
	my $offset = 0;  		# stores the position we are at in the current buffer
	my $packet;
    my $payload;
    my $lengthcounter = 0;
    my $packetcounter = 0;
	

	my $gop_offset = 0;     # Where does the Current GOP start?
    my $packet_offset = 0;  # Where does the Current PES_Packet start?
    my @index_buffer;       # Here are the entries for the index_vdr stored, until they get written to the file (i.e. at the end of a GOP)

    my %pes_header;
	my $head_tmpl = 'H6 H2 n1 B8 B8 C1 B40';
	my $dts_tmpl = 'B40';
    
	
	my $buffer = ${ readNextChunk() }; 	# First we read the first chunk of data
		
	while (($EOF eq 0) || ($offset  < length($buffer))) {
		
		
		($pes_header{startcode}, $pes_header{stream_id}, 
	    $pes_header{PES_packet_length}, $pes_header{eflags1}, 
	    $pes_header{eflags2}, $pes_header{header_length},
	    $pes_header{PTS_raw_bits}) 
	    = unpack ( $head_tmpl, substr($buffer, $offset, 14) );
	
		if ($pes_header{startcode} ne "000001")	{
			die "No 0x000001 found at current packet, found $pes_header{startcode} instead\noffset is $offset\n"; 
		}
		my $decimal_code = hex($pes_header{stream_id});
		
        
        
        if ((!(191 < $decimal_code ) &&  (240 > $decimal_code)) &&  (! $decimal_code == 189)) {
			print "unknown Streamtype $pes_header{stream_id}, decimal $decimal_code ignoring\n";
            $offset +=  $pes_header{PES_packet_length} + 6;
			next;
		}
        
		# MPEG2 Audio and Video must have an extended Header as well as AC3
		$pes_header{payload_start} = (9 + $pes_header{header_length});
		
		if ((($offset + $pes_header{PES_packet_length} + 150) > (length($buffer))) && ($EOF == 0)) {
			my $helpbuffer = substr($buffer, $offset);
			$buffer = $helpbuffer . ${ readNextChunk() };
			$offset = 0;
		}

		$packet = substr($buffer, $offset, $pes_header{PES_packet_length} + 6);
        $payload = substr($packet, $pes_header{payload_start});
		$offset +=  $pes_header{PES_packet_length} + 6;
        
        push @packet_infos, $pes_header{stream_id};
        push @packet_infos, $lengthcounter;
        push @packet_buffer, $packet;
        push @video_payload_buffer, $payload;
        
        if (($payload =~ m/$seqstart/g) && $packetcounter){
            #print "jumping for flush with " . scalar(@packet_infos) . "in buffer\n";
            flush_buffer();
        }
  
        $packetcounter++;
		$lengthcounter += ($pes_header{PES_packet_length} + 6);
    }
    flush_buffer();
	print "\nstopping cause offset + pes_header{PES_packet_length}) < (length(buffer)) " . ($offset +  $pes_header{PES_packet_length}) . " < " . (length($buffer)) . "\n";
	print "\n$packetcounter PES packets with a total length of $lengthcounter processed\n";
}





sub flush_buffer {
  
    my $GOP = join "", @packet_buffer;
    @packet_buffer = ();
    
    if (! $index_only) { # write the data to the target dir, but only if we are not called with the --index-only flag
        if ($current_file_size + length($GOP) > $max_size) {
            close $OFH;
            #my $final_size = -s (sprintf( "%03d.vdr" , ($file_no)));
            $file_offset += $current_file_size;
            my $new_file = sprintf( "%03d.vdr" , (++$file_no));
            open $OFH, ">$out_dir/$new_file" or die "Can not open next file for writing\n";
            binmode $OFH;
            $current_file_size = 0;
        }
        $current_file_size += length($GOP);
        print $OFH $GOP;
    }
    
    while (@packet_infos) {
        my $id      = shift @packet_infos;
        my $offset  = shift @packet_infos;
        
        #my $packet  = shift @packet_buffer;
        my $payload = shift @video_payload_buffer;
        if ($video_payload_buffer[0]) {
            $payload .= substr ($video_payload_buffer[0],0,2);
        }
        if (($id =~ /^e/) && ($payload =~ m/$picstart(.{4})/)   ) {
				my $pictype = "0b0" . substr(unpack("B16", $1),10,3);
                #print OFH "$pictypes{$pictype} $offset\n";
                #print "file_offset is now $file_offset offset is $offset diff is " . ($offset - $file_offset) ."\n";
                my $bin_code = pack("ICCS", ($offset - $file_offset), oct($pictype), $file_no , 0);
                #print oct($pictype) . " $pictype\n";
                print INDEXFH $bin_code;
                #my ($c, $Ptype, $number, $reserved) = unpack ("I C C S", substr($buffer, $offset, 8));
        } 
    }
}

sub usage {
    print <<END;
USAGE: cat vdrfile1 ... vdrfileN | create_indexvdr.pl -o TARGET_DIR [-s chunk_size]
    This skript accepts vdr files from STDIN and tries
    to create a proper VDR recording structure and an index.vdr.
    By default files are no bigger than 2 GB, which can
    be overidden by the -s option. The only required parameter
    is the destination directory.
END
    
}


sub parse_parameters {
    while (my $param = shift(@ARGV)) {
        if ($param =~ /^-s$/) {
            $max_size = shift(@ARGV) || 0;
            if (($max_size > 2147483648) || ($max_size < 1000000)) {
                die "Size must be between 1000000 and 2147483648\n";
            }
        } elsif ($param =~ /^-o$/) {
            
            $out_dir = shift(@ARGV) || 0;
            print "Got param for output  $out_dir\n";
            if (! -d $out_dir) {
                if (! (mkdir $out_dir) ) {
                    die "Output Directory did not exist an could not be created!\n";
                }
                
            }
        } elsif ($param =~ /^--index-only$/) {
            $index_only = 1;
        }
    }
    if (! $out_dir) {
        usage();
    }
}

