#!/bin/sh

# Configuration script for the VDR EXTENSION BOARD
# Version 0.26

# turn off stderr
exec 2>&-

# sig 2 and 15 trap & clean up
trap 'rm /tmp/tmp{1,2,3}.$$; exit 0' 2 15

#Defaults
SCRIPT_PATH="/usr/bin/"
LINPIC_PATH="/usr/bin/"
HEX_FILES_PATH=/usr/share/extb/"
SETSERIAL_CMD="setserial"
COM_PORT="/dev/ttyS1"

# -----------------------------------------------------------------------------

_MAIN () {
FLAG="0"
dialog --clear --title " MAIN MENU " --cancel-label "EXIT" \
       --backtitle "VDR EXTENSION BOARD" \
       --menu '' 15 72 9 \
	 1 " Status LEDs " \
	 2 " LC-Display & Backlight " \
	 3 " Audio/Video/SCART " \
	 4 " External outputs " \
	 5 " Macros " \
	 6 " Options (set & read from PIC, save to file) " \
	 7 " Load options from config file and send them to PIC " \
	 8 " Download HEX file to PIC " \
	 9 " Download HEX file to PIC and send options from config " 2> /tmp/tmp1.$$

SEL=$(cat /tmp/tmp1.$$)

case $SEL in
 1)
  _LED
 ;;
 2)
 _LCD
 ;;
 3)
 _AVS
 ;;
 4)
 _EXT
 ;;
 5)
 _MAC
 ;;
 6)
 _OPT
 ;;
 7)
 _OPT_1
 ;;
 8)
 _PIC
 ;;
 9)
 _PIC_OPT
 ;;
esac
}

# -----------------------------------------------------------------------------

_LED () {
dialog --no-shadow --cancel-label "Main Menu" --title " Status LEDs " \
--backtitle "VDR EXTENSION BOARD" --menu \
"Use the following dialog to change the status \n\
of the LEDs on the fly (mainly for testing)." \
21 50 13 \
	"S_ALL" " >>> All LEDs on " \
	"R_ALL" " >>> All LEDs off " \
	"S_LD1" " LED1  * On " \
	"R_LD1" "       * Off " \
	"T_LD1" "       * Toggle " \
 	"S_LD2" " LED2  * On " \
	"R_LD2" "       * Off " \
	"T_LD2" "       * Toggle " \
	"S_LD3" " LED3  * On " \
	"R_LD3" "       * Off " \
	"T_LD3" "       * Toggle " \
	"S_LD4" " LED4  * On " \
	"R_LD4" "       * Off " \
	"T_LD4" "       * Toggle " \
	"S_LD5" " LED5  * On " \
	"R_LD5" "       * Off " \
	"T_LD5" "       * Toggle " \
	"S_LD6" " LED6  * On " \
	"R_LD6" "       * Off " \
	"T_LD6" "       * Toggle " 2> /tmp/tmp1.$$
NAVI=$?
SEL=$(cat /tmp/tmp1.$$)

case $NAVI in

0)
 if [ $SEL == "S_ALL" ]; then SEL="S_LD1 S_LD2 S_LD3 S_LD4 S_LD5 S_LD6"; fi
 if [ $SEL == "R_ALL" ]; then SEL="R_LD1 R_LD2 R_LD3 R_LD4 R_LD5 R_LD6"; fi
 
 for i in $SEL; do tx.sh $i; done

 _LED
;;

1)
 _MAIN
;;
esac 
}

# -----------------------------------------------------------------------------

_LCD () {
dialog --no-shadow --cancel-label "Main Menu" --title " LC-Display & Backlight " \
       --backtitle "VDR EXTENSION BOARD" --menu \
"Use the following dialog to turn on and off \n\
the LCD contrast voltage and the LCD backlight \n\
on the fly (mainly for testing)." \
13 50 4 \
	"S_LCD"    "LCD contrast voltage  * On" \
	"R_LCD"    "                      * Off" \
 	"S_LCD_BL" "LCD backlight         * On" \
	"R_LCD_BL" "                      * Off" 2> /tmp/tmp1.$$
NAVI=$?
SEL=$(cat /tmp/tmp1.$$)

case $NAVI in

0)
 tx.sh $SEL
 _LCD
;;

1)
 _MAIN
;;
esac 
}

# -----------------------------------------------------------------------------

_AVS () {
dialog --no-shadow --cancel-label "Main Menu" \
       --title " Audio/Video/SCART " \
       --backtitle "VDR EXTENSION BOARD" \
       --menu \
"Use the following dialog to change the audio, video, and SCART \n\
settings on the fly (mainly for testing)." \
21 66 13 \
	"S_DVB"     "Video source    * DVB" \
	"S_VGA"     "                * VGA" \
	"S_AUX"     "                * AUX" \
	"R_VID_ALL" "                * NONE" \
	"T_DVB_VGA" "Toggle source   * DVB/VGA" \
	"T_DVB_AUX" "                * DVB/AUX" \
	"T_VGA_AUX" "                * VGA/AUX" \
	"S_SC8_6V"  "SCART #8        * 6V" \
	"R_SC8_6V"  "                * 12V" \
	"S_ANA"     "Analog audio    * On" \
	"R_ANA"     "                * Off" \
	"T_ANA"     "                * Toggle" \
	"S_DIA"     "Digital audio   * On" \
	"R_DIA"     "                * Off" \
	"T_DIA"     "                * Toggle" \
	"S_ANDIA"   "Ana/Dig audio   * On" \
	"R_ANDIA"   "                * Off" \
	"T_ANDIA"   "                * Toggle" 2> /tmp/tmp1.$$
NAVI=$?
SEL=$(cat /tmp/tmp1.$$)

case $NAVI in

0)
 tx.sh $SEL
 _AVS
;;

1)
 _MAIN
;;
esac 
}

# -----------------------------------------------------------------------------

_EXT () {
dialog --no-shadow --cancel-label "Main Menu" --title " External Outputs " \
       --backtitle "VDR EXTENSION BOARD" --menu \
"Use the following dialog to change the status \n\
of the outputs on the fly (mainly for testing)." \
21 52 13 \
	"S_ALL"  ">>> ALL Outputs on" \
	"R_ALL"  ">>> ALL Outputs off" \
	"S_OUT1" "Output 1  * On" \
	"R_OUT1" "          * Off" \
	"T_OUT1" "          * Toggle" \
 	"S_OUT2" "Output 2  * On" \
	"R_OUT2" "          * Off" \
	"T_OUT2" "          * Toggle" \
	"S_OUT3" "Output 3  * On" \
	"R_OUT3" "          * Off" \
	"T_OUT3" "          * Toggle" \
	"S_OUT4" "Output 4  * On" \
	"R_OUT4" "          * Off" \
	"T_OUT4" "          * Toggle" 2> /tmp/tmp1.$$
NAVI=$?
SEL=$(cat /tmp/tmp1.$$)

case $NAVI in

0)
 if [ $SEL == "S_ALL" ]; then SEL="S_OUT1 S_OUT2 S_OUT3 S_OUT4"; fi
 if [ $SEL == "R_ALL" ]; then SEL="R_OUT1 R_OUT2 R_OUT3 R_OUT4"; fi

 for i in $SEL; do tx.sh $i; done

 _EXT
;;

1)
 _MAIN
;;
esac 
}

# ------------------------------------------------------------------------------

_OPT () {
F1="";F2="";F3="";F4="";F5="";F6="";F7="";F8="";F9="";F10="";F11="";F12=""
rm /tmp/PICflags
tx.sh Status
dialog --no-shadow --backtitle "VDR EXTENSION BOARD" \
       --infobox "    Please wait ..." 3 30

sleep 2

 grep "V0" /tmp/PICflags
  if [ $? = 0 ]; then F2="(X)"; F4="(X)"; F6="(X)"; fi

 grep "V1" /tmp/PICflags
  if [ $? = 0 ]; then F1="(X)"; F4="(X)"; F6="(X)"; fi
        
 grep "V2" /tmp/PICflags
  if [ $? = 0 ]; then F2="(X)"; F3="(X)"; F6="(X)"; fi

 grep "V3" /tmp/PICflags
  if [ $? = 0 ]; then F1="(X)"; F3="(X)"; F6="(X)"; fi

 grep "V4" /tmp/PICflags
  if [ $? = 0 ]; then F2="(X)"; F4="(X)"; F5="(X)"; fi

 grep "V5" /tmp/PICflags
  if [ $? = 0 ]; then F1="(X)"; F4="(X)"; F5="(X)"; fi

 grep "V6" /tmp/PICflags
  if [ $? = 0 ]; then F2="(X)"; F3="(X)"; F5="(X)"; fi

 grep "V7" /tmp/PICflags
  if [ $? = 0 ]; then F1="(X)"; F3="(X)"; F5="(X)"; fi

 grep "SB0" /tmp/PICflags
  if [ $? = 0 ]; then F9="(X)"; F12="(X)"; fi

 grep "SB1" /tmp/PICflags
  if [ $? = 0 ]; then F7="(X)"; F12="(X)"; fi

 grep "SB2" /tmp/PICflags
  if [ $? = 0 ]; then F8="(X)"; F12="(X)"; fi

 grep "SB4" /tmp/PICflags
  if [ $? = 0 ]; then F9="(X)"; F10="(X)"; fi

 grep "SB5" /tmp/PICflags
  if [ $? = 0 ]; then F7="(X)"; F10="(X)"; fi

 grep "SB6" /tmp/PICflags
  if [ $? = 0 ]; then F8="(X)"; F10="(X)"; fi

 grep "SB8" /tmp/PICflags
  if [ $? = 0 ]; then F9="(X)"; F11="(X)"; fi

 grep "SB9" /tmp/PICflags
  if [ $? = 0 ]; then F7="(X)"; F11="(X)"; fi

 grep "SBa" /tmp/PICflags
  if [ $? = 0 ]; then F8="(X)"; F11="(X)"; fi

dialog --no-shadow --cancel-label "Save options" \
       --title " Options " \
       --backtitle "VDR EXTENSION BOARD" \
       --extra-button --extra-label "Main Menu" \
       --menu \
"Current options are marked with (X). Changes will be sent to the \n\
PIC and reread from there immediately (takes 2 seconds). \n\
Use <Save options> to save the current options to a file." \
21 72 12 \
       	"F_DVB_RGB"    " DVB mode   * RGB                   $F1 " \
	"F_DVB_CS"     "            * Composite or S-Video  $F2 " \
	"F_VGA_RGB"    " VGA mode   * RGB                   $F3 " \
	"F_VGA_CS"     "            * Composite or S-Video  $F4 " \
	"F_AUX_RGB"    " AUX mode   * RGB                   $F5 " \
	"F_AUX_CS"     "            * Composite or S-Video  $F6 " \
	"F_SC8_ON_PWR" " SCART #8   * On powering up        $F7 " \
	"F_SC8_ON_VDR" "            * On VDR start          $F8 " \
	"F_SC8_OFF"    "            * Deactivated           $F9 " \
	"F_BOOT_VGA"   " Boot mode  * VGA                   $F10 " \
	"F_BOOT_AUX"   "            * AUX                   $F11 " \
	"F_BOOT_OFF"   "            * None                  $F12 " 2> /tmp/tmp1.$$
NAVI=$?
SEL=$(cat /tmp/tmp1.$$)

case $NAVI in

0)
 tx.sh $SEL
 _OPT
;;

1)
 cp /tmp/PICflags /etc/extb/PICflags.conf
 _MAIN
;;

3)
 _MAIN
;; 
esac 
}

# -----------------------------------------------------------------------------

_MAC () {
dialog --no-shadow --cancel-label "Main Menu" --title " Macros " \
       --backtitle "VDR EXTENSION BOARD" --menu \
"Use the following dialog to execute a macro \n\
in the PIC on the fly (mainly for testing)." \
18 50 10 \
	"M1"  "Macro M1" \
	"M2"  "Macro M2" \
	"M3"  "Macro M3" \
	"M4"  "Macro M4" \
	"M5"  "Macro M5" \
	"M6"  "Macro M6" \
	"M7"  "Macro M7" \
	"M8"  "Macro M8" \
	"M9"  "Macro M9" \
	"M10" "Macro M10" 2> /tmp/tmp1.$$
NAVI=$?
SEL=$(cat /tmp/tmp1.$$)

case $NAVI in

0)
 tx.sh $SEL
 _MAC
;;

1)
 _MAIN
;;
esac 
}

# -----------------------------------------------------------------------------

_OPT_1 () {
if test -e "PICflags.conf"
then
    dialog --no-shadow --backtitle "VDR EXTENSION BOARD" \
	   --yesno "Overwrite PIC settings?" 6 28

case $? in

1)
 _MAIN
;;

0)
 grep "V0" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_DVB_CS;tx.sh F_VGA_CS;tx.sh F_AUX_CS; fi

 grep "V1" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_DVB_RGB;tx.sh F_VGA_CS;tx.sh F_AUX_CS; fi
        
 grep "V2" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_DVB_CS;tx.sh F_VGA_RGB;tx.sh F_AUX_CS; fi

 grep "V3" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_DVB_RGB;tx.sh F_VGA_RGB;tx.sh F_AUX_CS; fi

 grep "V4" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_DVB_CS;tx.sh F_VGA_CS;tx.sh F_AUX_RGB; fi

 grep "V5" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_DVB_RGB;tx.sh F_VGA_CS;tx.sh F_AUX_RGB; fi

 grep "V6" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_DVB_CS;tx.sh F_VGA_RGB;tx.sh F_AUX_RGB; fi

 grep "V7" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_DVB_RGB;tx.sh F_VGA_RGB;tx.sh F_AUX_RGB; fi

 grep "SB0" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_SC8_OFF;tx.sh F_BOOT_OFF; fi

 grep "SB1" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_SC8_ON_PWR;tx.sh F_BOOT_OFF; fi

 grep "SB2" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_SC8_ON_VDR;tx.sh F_BOOT_OFF; fi

 grep "SB4" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_SC8_OFF;tx.sh F_BOOT_VGA; fi

 grep "SB5" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_SC8_ON_PWR;tx.sh F_BOOT_VGA; fi

 grep "SB6" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_SC8_ON_VDR;tx.sh F_BOOT_VGA; fi

 grep "SB8" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_SC8_OFF;tx.sh F_BOOT_AUX; fi

 grep "SB9" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_SC8_ON_PWR;tx.sh F_BOOT_AUX; fi

 grep "SBa" PICflags.conf
  if [ $? = 0 ]; then tx.sh F_SC8_ON_VDR;tx.sh F_BOOT_AUX; fi
;;
esac

else
    dialog --no-shadow --backtitle "VDR EXTENSION BOARD" \
	   --msgbox "There is no config file!" 6 30
fi
_MAIN
}

# -----------------------------------------------------------------------------

_PIC_OPT () {
FLAG="1"    
_PIC
}

# -----------------------------------------------------------------------------

_PIC () {
ls $HEX_FILES_PATH > /tmp/tmp2.$$
LIST=$(cat /tmp/tmp2.$$)
echo "dialog --no-shadow --cancel-label \"Main Menu\" --title \" PIC HEX file download\" \\" > /tmp/tmp3.$$
echo "--backtitle \"VDR EXTENSION BOARD\" --menu \"Please select a HEX-file, confirm with <OK> and wait" >> /tmp/tmp3.$$
echo "until the message 'Looking for bootloader' appears." >> /tmp/tmp3.$$
echo "Then press the reset switch on the Extension Board.\" 13 60 4 \\" >> /tmp/tmp3.$$
for i in $LIST; do
    echo "\"$i\" \"\" \\" >> /tmp/tmp3.$$
done    
echo "\"\" \"\" 2> /tmp/tmp1.$$" >> /tmp/tmp3.$$
bash /tmp/tmp3.$$
NAVI=$?
FILE=$(cat /tmp/tmp1.$$)

case $NAVI in

0)
 tx.sh M2 # sending M2 to Extension Board to disable video/audio
 clear
 /etc/init.d/irexec stop
 /etc/init.d/lircd stop
 sleep 1
 $SETSERIAL_CMD $COM_PORT uart 16550
 sleep 1
 picdl -P$COM_PORT $HEX_FILES_PATH$FILE
 echo ""
 echo "Restarting LIRC ..."
 echo ""
 $SETSERIAL_CMD $COM_PORT uart none
 sleep 1
 /etc/init.d/lircd start
 /etc/init.d/irexc start
 echo ""
 echo "Stopping VDR ..."
 /etc/init.d/vdr stop
 sleep 1
 echo ""
 echo "Restarting VDR ..."
 rcvdr start
 cd $SCRIPT_PATH
 echo ""
 # just in case the Extension Board plugin is not installed ...
 echo "Sending 'M1' to Extension Board ..."
 tx.sh M1
 tx.sh M1
 sleep 4
 if [ "$FLAG" = "1" ];
    then _OPT_1;
    else _MAIN;
 fi    
;;

1)
 _MAIN
;;
esac
}

# -----------------------------------------------------------------------------

# ***** PROGRAM START *****
_MAIN

# force clean up
kill -INT $$