#!/usr/bin/perl
#
# perl script to convert VDR cutting marks (marks.vdr) to byte positions in the stream
#
# intended to use for cutting with Project X (set CollectionPanel.CutMode=0)
#
# usage: marks2bytepos.pl /path/to/recording/with/timestamp >cut.px
# output: byte positions (one per line) are printed on STDOUT

use strict;

sub add
{
   my ($pint, $add) = @_;
   
   for (my $i=3; $i>0; $i--)
   {
      $pint->[$i] += $add%1000;
      if ($pint->[$i] >=1000) 
      { 
         $pint->[$i-1] += int($pint->[$i]/1000); 
         $pint->[$i] = $pint->[$i]%1000; 
      }
      $add = int($add/1000);
   }
   $pint->[0] += $add;
}

if (1 != @ARGV)
{
   print "\nmarks2bytepos - convert vdr cutting marks to byte positions in the stream\n\n";
   print "Usage: $0 /path/to/recording\n";
   print "byte positions (one per line) are printed on STDOUT\n";
   
   exit (1);
}

open (MARKS, "<$ARGV[0]/marks.vdr") or die ("Couldn't open $ARGV[0]/marks.vdr");
my @marks=<MARKS>;
close(MARKS);

open (INDEX, "<$ARGV[0]/index.vdr") or die ("Couldn't open $ARGV[0]/index.vdr");

foreach (@marks)
{
   my $buffer;
   my @bytepos=(0, 0, 0, 0);
   my ($h,$m,$s,$f) = split /[:.]/,$_;
   ($f,undef)=split / /, $f;
   my $frame = ($h * 3600 + $m * 60 + $s)* 25 + $f-1;

   seek (INDEX, 8*$frame, 'SEEK_SET');
   read(INDEX, $buffer, 8);
   my ($offset, $type, $filenumber, $reserved) = unpack("LCCS", $buffer);

   add(\@bytepos, $offset);
   
   for (my $i=1; $i<$filenumber; $i++)
   {
      add(\@bytepos, (-s sprintf ("%s/%03d.vdr", $ARGV[0], $i)));
   }
        
   printf("%d%03d%03d%03d\n", @bytepos);
};

close (INDEX);

exit (0);
