#!/usr/bin/perl
###############################################################################
# THIS SOFTWARE IS FREE SOFTWARE! IT IS LICENCED UNDER THE GNU PUBLIC LICENCE
# 
#
#  VDRsync Studio (c) 2003-2004 by  Dr. Peter Sebbel, a perl script supposed to 
#  become a GUI for VDRsync, and (a little bit) dvdauthor
#  
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; version 2 of the License
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##############################################################################

use strict;
use warnings;

use Cwd;
use Tk;
use Tk::JPEG::Lite;
use Tk::widgets qw/Balloon ProgressBar/;
use Time::HiRes qw(gettimeofday tv_interval);
use Tk::DirTree;
use Tk::Tree;
use Tk::ErrorDialog

require Tk::DialogBox;
require Tk::LabEntry;
require Tk::NoteBook;
require Tk::Dialog;
#require "/usr/local/lib/perl/vdrsync/VDRRECORDING.pm";


our $version = 0.0.12;


my %config;     # A hash that contains many different Configuration Options

my @mark_bars;
my @mark_ticks;
my @chapter_ticks;


my $split = 0;

my %jpegs;            # Hash that contains the Framenumber as the key, and the file name as a value if the jpeg of this frame has already been decoded
my $video_dir = "/video";
my @recording_list;
my %selected_rec;
my $first_init = 1;

# Some vars for the jog-dial and the status of the keyboard
my $dial_pos     = 0;
my $dial_status  = 0;
my $shift_status = 0;
my $ctrl_status  = 0;
my $snap_mark    = 0;
my $single_step_mode = 0;
my $marks_changed = 0;
my $chapter_marks_changed = 0;


###########################################################3
# The vars hold the frames that contain some controls, used for toggeling on and of
my $misc_ctr_frame;
my $misc_but_frame;
my $movie_nav_frame;
my $mark_mani_frame;

my %dial_hash = (
    1    =>   0.5,
    2    =>   2.0,
    3    =>   5.0,
    4    =>   10.0,
    5    =>   20.0,
    6    =>   50.0,
    7    =>   100.0,
    8    =>   200.0,
    9    =>   500.0,
    10    =>   1000.0,
);
# Step sizes for the navigation
$config{small_step}  = 0.5;
$config{middle_step} = 60;
$config{big_step}    = 300;
$config{wheel_step}  = 0.5;

$config{queue_manager} = 1;


my $cut_length;
my $control_window;


my $indir = shift @ARGV;

my %streams;
####################
# Globals for the Screen Setupe etc
####################
my $click_pos;
my $percent_done = 0;
my $current_picture;
my $canvas_image;
my $movie_length;
my $current_gop;
my $frameno;
my $current_pos;
my $mark_no;
my $mplayer_pid;
my $chapter_mode = 0;
my $tmp_dir; 
####################
# Globals for the DVD Setup
####################
$config{dvd_chapter_interval} = 300; # DVD chapter Intervall in seconds
$config{dvd_output_dir}       = "/tmp";
$config{dvd_basename}         = "VDRsyncDVD";
$config{dvd_root_menu_title}  = "Created by VDRsync";
$config{dvd_root_label}       = "Start Movie";
$config{dvd_device}           = "/dev/dvd";
$config{dvd_chapter_menu_len} = 20;
$config{dvd_use_pipe}         = 1;
$config{dvd_chapter_menu}     = 1;
$config{dvd_menu}             = 1;
$config{dvd_fit}              = 1;
$config{dvd_cut}              = 1;
$config{dvd_movie_title}      = "Created by VDRsync";
$config{preview_width}        = 360;
$config{preview_height}       = 288;

$config{control_window}       = 0;
$config{movie_nav_frame}      = 0;
$config{mark_mani_frame}      = 0;
$config{misc_but_frame}       = 0;
$config{misc_ctr_frame}       = 0;


my $dvd_length      = "NA";
my $dvd_size        = "NA";
my $dvd_vsize       = "NA";
my $dvd_asize       = "NA";

my $dvd_title_no    = 0;
my $dvd_rfactor     = 1;
my $dvd_movie_title;
my $dvd_basename;
my $dvd_root_menu_title;
my $dvd_root_label;
my $dvd_output_dir;
my $dvd_burn_cmd; 
my $chapter_strg;
my $dvd_video_opt;
my $dvd_audio_opt;
my $dvd_stream_frame;
my $dvd_movie_menu_text = "Here we will place the summary.txt if available";
my $dvd_main_menu_background;
my $dvd_indir          = "";
my $dvd_menu_text;
my $dvd_menu_text_changed = 0;
my $dvd_vdrsync_cmd;
my $dvd_user_cmd_add;
my @titlesets;
####################
# Globals for helper locations and options
####################


#########################################################################
# Hash that lists potential helper Programs
#########################################################################
my %progs = (
    mplex               => "",
    gop2jpeg            => "",
    dvdauthor           => "",
    ffmpeg              => "",
    transcode           => "",
    growisofs           => "",
    "vdrsync.pl"        => "",
    "queue_manager.pl"  => "",
    "dvd-menu.pl"       => "",
    "create_indexvdr.pl"=> "",
    "vdrsync_buffer.pl" => "",
    tcrequant           => "",
    requant             => "",
    jpeg2yuv            => "",
    marks2pts           => "",
    pts2marks           => "",

);

$config{capture_prog} = "gop2jpeg";


$config{master_tmp_dir} = "/tmp";

####################
# Globals for DIVX
####################

my $divx_bitrate;
my $divx_cd_number;
my $divx_video_opt; 
my $divx_audio_opt; 
my $divx_pass_number; 
my $divx_audio_format;
my $divx_vdrsync_cmd = "";

####################
# Globals for SVCD
####################

my $svcd_bitrate;
my $svcd_cd_number;
my $svcd_video_opt; 
my $svcd_audio_opt; 
my $svcd_pass_number; 
my $svcd_vdrsync_cmd = "";

####################
# Globals for VDRsync
####################
my $vs_cut_recording = 1;
my $vs_video_filter  = "";
my $vs_audio_filter  = ""; 
my $vs_ac3_filter    = ""; 
my $vs_basename      = "vdrsync";
my $vs_out_dir       = "./"; 
my $vs_mkiso         = 0;
my $vs_master_dvd    = 0;
my $vs_cut_filesize  = 2000000000;
my $vs_cut_filedir   = "/tv/cut/";
my $vs_other;
my $vs_fifos;
my $vs_multiplex;
my $vs_ignore;
my $vs_post_exec;
my $vs_mpeg2;
#my $vdrsync_output_dir = "./";
#my $vdrsync_basename = "vdrsync";
#my $vdrsync_options = "";
   
####################
# Globals that I forgot about  ;), Ahh no, it is for benchmarking
####################
my $get_gop_time = 0;
my $decode_gop_time = 0;
my $pic_counter = 0;
my $load_pic_time = 0;
    
##############
# Vars needed as globals
# Recording page
##############
my $dial;
my $dial_line;
my $can;
my $total_length_label;
my $prog_can;
my $prog_rect;


my $nav_busy = 0; # here we state that the program is busy fetching an image, so some subs refuse to get called again...
my $nav_queue;    # instead, queue up the requests and switch on a "after" function, that will take care of the problem.
##############
# DVDauthor Page
##############


my $dvd_can;
my $recording;

##################################################################################
##################################################################################
##################################################################################
##################################################################################

my $rmw = MainWindow->new;
$rmw->title("VDRsync GUI Version XY");
$rmw->configure(-menu => my $menubar = $rmw->Menu);

my $file    = $menubar->cascade(-label => '~File');
my $options = $menubar->cascade(-label => '~Options');
my $help    = $menubar->cascade(-label => '~Help');
my $file_menu = $file->command(
    -label       => 'Open',
    -underline   => 0,
    -command     => \&open_recording,
);
 $file->command(
    -label       => 'Exit',
    -underline   => 1,
    -command     => \&clean_up,
);

foreach (['Show Movie Navigation' , \$config{movie_nav_frame}], 
         ['Show Mark Manipulation', \$config{mark_mani_frame}],
         ['Show Misc Controls'    , \$config{misc_ctr_frame}],
         ['Show Misc Buttons'     , \$config{misc_but_frame}],
         ['Controls in extra Window', \$config{control_window}],
         ) {
   $options->checkbutton(
        -label    => $_->[0],
        -variable => $_->[1],
        -command => [ \&paint_controls ],
    );
}

    

my $nb = $rmw->NoteBook( )->pack(-expand => 1, -fill => 'both');
my $mw = $nb->add('Recording_page', -label => 'Recording',  -raisecmd => \&recording_notebook_raised);
my $vs_page;                        # = $nb->add('vs_page', -label => 'VDRsync Only');
my $dvd_page = $nb->add('dvdauthor_page', -label => 'Create DVD', -raisecmd => \&update_dvd_page);
my $divx_page;                      # = $nb->add('divx_page', -label => 'DIVX');
my $svcd_page;                      # = $nb->add('svcd_page', -label => 'SVCD');
my $config_page = $nb->add('config_page', -label => 'Configuration');


my $five_back_button;
my $one_back_button;
my $prev_gop_button;
my $next_gop_button;
my $one_for_button;
my $five_for_button;


################################################
sub paint_vdrsync_notebook {
   
   my $prg_location_frame = $vs_page->Frame(-relief => "groove", -bd => 4)->pack(-side => "top", -anchor => "nw" );
   my $loc_label = $prg_location_frame->Label(  -text => "VDRsync Options for manual run" )->grid(-columnspan => 2 );
   
   
  # my $dummy_frame = $vs_page->Frame(-relief => "groove", -bd => 4)->grid(-column => 1, -row => 0, -sticky => "ne");#, -columnspan => , -rowspan => 8);
  # $dummy_frame->Label(  -text => "Dummy" )->grid(-column => 0, -row => 0);
  # $dummy_frame->Label(-text => "cut recording?")->grid(-column => 0, -row => 1);
   
   $prg_location_frame->Label(-text => "cut recording?")->grid( 
   $prg_location_frame->Checkbutton(   -variable => \$vs_cut_recording )); 
   
   $prg_location_frame->Label(-text => "Master DVD")->grid(
   $prg_location_frame->Checkbutton(   -variable => \$vs_master_dvd ));
   
   $prg_location_frame->Label(-text => "Make ISO Image")->grid(
   $prg_location_frame->Checkbutton(   -variable => \$vs_mkiso ));
   
   $prg_location_frame->Label(-text => "Use FIFOs")->grid(
   $prg_location_frame->Checkbutton(   -variable => \$vs_fifos ));
      
   $prg_location_frame->Label(-text => "Multiplex")->grid(
   $prg_location_frame->Entry(   -textvariable => \$vs_multiplex ));
   
   $prg_location_frame->Label(-text => "Ignore Streams")->grid(
   $prg_location_frame->Entry(   -textvariable => \$vs_ignore ));
   
   $prg_location_frame->Label(-text => "PostExec")->grid(
   $prg_location_frame->Entry(   -textvariable => \$vs_post_exec ));
   
   $prg_location_frame->Label(-text => "Generic MPEG2")->grid(
   $prg_location_frame->Entry(   -textvariable => \$vs_mpeg2 ));
   
      
   $prg_location_frame->Label(-text => "Video Filter")->grid(
   $prg_location_frame->Entry(   -textvariable => \$vs_video_filter ));#->grid(-column => 1, -row => 2);
   
   $prg_location_frame->Label(-text => "Audio Filter MP2")->grid(
   $prg_location_frame->Entry(   -textvariable => \$vs_audio_filter ));
   
   $prg_location_frame->Label(-text => "Audio Filter AC3")->grid(
   $prg_location_frame->Entry(   -textvariable => \$vs_ac3_filter ));#->grid(-column => 1, -row => 4); 
   
   $prg_location_frame->Label(-text => "basename")->grid(
   $prg_location_frame->Entry(   -textvariable => \$vs_basename ));
   
   $prg_location_frame->Label(-text => "Output Dir")->grid(
   $prg_location_frame->Entry(   -textvariable => \$vs_out_dir ));
   
   
   $prg_location_frame->Label(-text => "Other Options")->grid(
   $prg_location_frame->Entry(   -textvariable => \$vs_other ));
   
   my $do_button_frame = $vs_page->Frame(-relief => "groove", -bd => 4)->pack(-side => "top", -anchor => "nw" );#grid(-column => 0, -row => 1);
   $do_button_frame->Button(-text => "Start Conversion", -command =>  \&start_vdrsync_run)->grid(-column => 0, -row => 0);
   
}


################################################
sub paint_divx_notebook {
   $divx_page->Label(-text => "Bitrate")->pack(-anchor => "nw");
   $divx_page->Entry(   -textvariable => \$divx_bitrate )->pack(-anchor => "nw"); 
   $divx_page->Label(-text => "Number Of CDs")->pack(-anchor => "nw");
   $divx_page->Entry(   -textvariable => \$divx_cd_number )->pack(-anchor => "nw"); 
   $divx_page->Label(-text => "Video Options")->pack(-anchor => "nw");
   $divx_page->Entry(   -textvariable => \$divx_video_opt )->pack(-anchor => "nw"); 
   $divx_page->Label(-text => "Audio Options")->pack(-anchor => "nw");
   $divx_page->Entry(   -textvariable => \$divx_audio_opt )->pack(-anchor => "nw"); 
   $divx_page->Label(-text => "1 or 2 Pass")->pack(-anchor => "nw");
   $divx_page->Entry(   -textvariable => \$divx_pass_number )->pack(-anchor => "nw"); 
   $divx_page->Label(-text => "DIVX Audio Format")->pack(-anchor => "nw");
   $divx_page->Entry(   -textvariable => \$divx_audio_format )->pack(-anchor => "nw"); 
   $divx_page->Label(-text => "VDRsync command")->pack(-anchor => "nw");
   $divx_page->Entry(   -textvariable => \$divx_vdrsync_cmd )->pack(-anchor => "nw"); 
   
   
   
   
}

################################################
sub paint_svcd_notebook {
    $svcd_page->Label(-text => "Bitrate")->pack( );
    $svcd_page->Entry(   -textvariable => \$svcd_bitrate )->pack; 
    $svcd_page->Label(-text => "Number Of CDs")->pack( );
    $svcd_page->Entry(   -textvariable => \$svcd_cd_number )->pack; 
    $svcd_page->Label(-text => "Video Options")->pack( );
    $svcd_page->Entry(   -textvariable => \$svcd_video_opt )->pack; 
    $svcd_page->Label(-text => "Audio Options")->pack( );
    $svcd_page->Entry(   -textvariable => \$svcd_audio_opt )->pack; 
    $svcd_page->Label(-text => "1 or 2 Pass")->pack( );
    $svcd_page->Entry(   -textvariable => \$svcd_pass_number )->pack; 
    $svcd_page->Label(-text => "VDRsync command")->pack(-anchor => "nw");
    $svcd_page->Entry(   -textvariable => \$svcd_vdrsync_cmd )->pack(-anchor => "nw"); 
   
    
}

################################################
sub paint_dvd_notebook {
    my $grouping_frame = $dvd_page->Frame(-relief => "groove", -bd => 4)->grid(-column => "0", -row => "0", -sticky => "nw");
    $grouping_frame->Label(-text  => "Chapter Interval")->grid(
    my $chapter_e = $grouping_frame->Entry(-width => 30, -textvariable => \$config{dvd_chapter_interval} ), -sticky => "e"); 
    $grouping_frame->Balloon(-state => "balloon")->attach($chapter_e, -msg => "Length of chapters on the DVD in sec");
    
    
    $grouping_frame->Label(-text  => "Output Dir")->grid( 
    my $output_e = $grouping_frame->Entry(-width => 30, -textvariable => \$dvd_output_dir), -sticky => "e"); #-validate => "focusout", -validatecommand => \&build_dvd_vdrsync_cmd,
    $grouping_frame->Balloon(-state => "balloon")->attach($output_e, -msg => "The Directory in which the DVD will be created");
    #$grouping_frame->Label(-text => "Video Options")->grid( 
    #$grouping_frame->Entry(-width => 50, -textvariable => \$dvd_video_opt )); 
    #$grouping_frame->Label(-text => "Audio Options")->grid(
    #$grouping_frame->Entry(-width => 50, -textvariable => \$dvd_audio_opt )); 
    $grouping_frame->Label(-text  => "DVD Basename")->grid(
    my $basename_e = $grouping_frame->Entry(-width => 30, -textvariable => \$dvd_basename), -sticky => "e"); #-validate => "focusout", -validatecommand => \&build_dvd_vdrsync_cmd,
    $grouping_frame->Balloon(-state => "balloon")->attach($basename_e, -msg => "The name of the Directory that will be created");
    
    $grouping_frame->Label(-text  => "DVD Main Menu Title")->grid(
    my $maintitle_e = $grouping_frame->Entry(-width => 30, -textvariable => \$dvd_root_menu_title), -sticky => "e"); #-validate => "focusout", -validatecommand => \&build_dvd_vdrsync_cmd,
    $grouping_frame->Balloon(-state => "balloon")->attach($maintitle_e, -msg => "The Title for the first Menu that is played after inserting the DVD");
    
    $grouping_frame->Label(-text  => "Main Menu Label Text")->grid(
    my $maintext_e = $grouping_frame->Entry(-width => 30, -validate => "focusout", -textvariable => \$dvd_root_label), -sticky => "e"); 
    $grouping_frame->Balloon(-state => "balloon")->attach($maintext_e, -msg => "Text that is displayed next to the Buttons that starts the movie");
    
    $grouping_frame->Label(-text  => "DVD Movie Menu Title")->grid(
    my $movietitle_e =  $grouping_frame->Entry(-width => 30, -textvariable => \$dvd_movie_title), -sticky => "e"); #-validate => "focusout", -validatecommand => \&build_dvd_vdrsync_cmd,
    $grouping_frame->Balloon(-state => "balloon")->attach($movietitle_e, -msg => "Title of the Movie Menu (the 2nd Menu you see) ");
    
    
    
    #$grouping_frame->Label(-text => "DVD Device")->grid(
    #$grouping_frame->Entry(-width => 50, -validate => "focusout", -validatecommand => \&update_dvd_page, -textvariable => \$config{dvd_device})); 
    ##$grouping_frame->Label(-text => "Burn Command")->grid(
    #$grouping_frame->Entry(-width => 50, -textvariable => \$dvd_burn_cmd ));
    $grouping_frame->Label(-text  => "Movie Menu Text")->grid(-column => "0", -row => 6, -sticky => "n", -columnspan => 2);
    
    $dvd_menu_text = $grouping_frame->Text(-height => 7, -width => 60, -wrap => 'word'  )->grid(-row=> "7",-column => "0", -rowspan => "1", -columnspan => 2,  -sticky => "e");# -textvariable => \$dvd_movie_menu_text )); 
    $grouping_frame->Balloon(-state => "balloon")->attach($dvd_menu_text, -msg => "Text on the 2nd DVD-Menu you will see, normaly taken from summary.vdr");
    $grouping_frame->Button(-text => "Save Text", -command =>  \&save_dvd_settings)->grid( #-column => "0", -sticky => "ne");
    $grouping_frame->Button(-text => "Add to DVD", -command =>  \&add_titleset));

    my $gframe2 = $dvd_page->Frame(-relief => "groove", -bd => 4)->grid(-column => "0", -row => "1", -sticky => "nwe");
    #$grouping_frame->Label(-text => "Main Menu Background")->grid(
    #$grouping_frame->Entry(-width => 50, -textvariable => \$dvd_main_menu_background )); 
    
    $gframe2->Label(-text  => "VDRsync Options")->grid( -columnspan => "3");
    my $usep_cb = $gframe2->Checkbutton(-text => "--use-pipe",
                       -variable  => \$config{dvd_use_pipe},
                       #-command   => [ \&build_dvd_vdrsync_cmd ]
                       )->grid(-sticky => "nw", -row => "1", -column => "0");
    $gframe2->Balloon(-state => "balloon")->attach($usep_cb, -msg => "This Option will speed up the DVD creation run a lot, with the cost of higher memory consumption");

    
    my $chapterm_cb = $gframe2->Checkbutton(-text => "--dvd-chapter-menus",
                       -variable  => \$config{dvd_chapter_menu},
                       #-command   => [ \&build_dvd_vdrsync_cmd ]
                       )->grid(-sticky => "w", -row => "1", -column => "1");
    $gframe2->Balloon(-state => "balloon")->attach($chapterm_cb, -msg => "If you want animated chapter menus, enable this one");
                    
    
    
    my $dvdm_cb = $gframe2->Checkbutton(-text => "--dvd-menu",
                       -variable  => \$config{dvd_menu},
                       #-command   => [ \&build_dvd_vdrsync_cmd ]
                       )->grid(-sticky => "w", -row => "1", -column => "2");
    $gframe2->Balloon(-state => "balloon")->attach($dvdm_cb, -msg => "If you want DVD menus, enable this one");
    
    
    
    my $fit_cb = $gframe2->Checkbutton(-text => "--fit-to-dvd",
                       -variable  => \$config{dvd_fit},
                       #-command   => [ \&build_dvd_vdrsync_cmd ]
                       )->grid(-sticky => "w", -row => "2", -column => "0");
    $gframe2->Balloon(-state => "balloon")->attach($fit_cb, -msg => "If --fit-to-dvd is enabled, VDRsync will try to shrink your recording on the fly. This needs tcrequant or requant.");
    
    
    my $cut_cb = $gframe2->Checkbutton(-text => "--cut",
                       -variable  => \$config{dvd_cut},
                       #-command   => [ \&build_dvd_vdrsync_cmd ]
                       )->grid(-sticky => "w", -row => "2", -column => "1");
    $gframe2->Balloon(-state => "balloon")->attach($cut_cb, -msg => "The recording will be cut during processing if you enable this option");                   
                       
                       
    $gframe2->Label(-text  => "User Options:")->grid(-column => "0", -row => "3", -columnspan => 1);
    my $cmd_e = $gframe2->Entry(-width => 40, -textvariable => \$dvd_user_cmd_add )->grid( -sticky => "w", -column => "1", -row => "3", -columnspan => 2); 
    $gframe2->Balloon(-state => "balloon")->attach($cmd_e, -msg => "If you want to add some other Options to the vdrsync-command,\n you can do it here");


    $gframe2->Button(-text => "Show Contents", -command =>  \&show_titlesets)->grid(-column => "0", -row => "4", -columnspan => 1);
    $gframe2->Button(-text => "Start DVD Creation", -command =>  \&start_dvd_creation_run)->grid(-column => "1", -row => "4", -columnspan => 2);


    #
    # Finally we create a frame that will display some summary stats for the DVD
    #
    my $gframe3 = $dvd_page->Frame(-relief => "groove", -bd => 4)->grid
            (-row => "1", -column => "1", -sticky => "nw");
    
    $gframe3->Label(-text  => "DVD Summary")->grid( -columnspan => "2");
    $gframe3->Label(-text  => "No. of Titlesets:")->grid(($gframe3->Label( -textvariable => \$dvd_title_no)) ,-sticky => "w");
    $gframe3->Label(-text  => "Total Length:")->grid(($gframe3->Label( -textvariable => \$dvd_length)) ,-sticky => "w");
    $gframe3->Label(-text  => "Total Size:")->grid(($gframe3->Label( -textvariable => \$dvd_size)) ,-sticky => "w");
    $gframe3->Label(-text  => "Total Video (MB):")->grid(($gframe3->Label( -textvariable => \$dvd_vsize)) ,-sticky => "w");
    $gframe3->Label(-text  => "Requant Factor:")->grid(($gframe3->Label( -textvariable => \$dvd_rfactor)) ,-sticky => "w");
    
    
    
    $dvd_page->gridColumnconfigure(0, -weight => 0);
    $dvd_page->gridRowconfigure(0, -weight => 0);
    $dvd_page->gridColumnconfigure(1, -weight => 1);
    $dvd_page->gridRowconfigure(1, -weight => 1);
    #$dvd_page->gridColumnconfigure(2, -weight => 1);
 
    

}

################################################
sub paint_config_notebook {    

    my $left_frame   = $config_page->Frame(-relief => "groove", -bd => 4)->grid(-rowspan => "3", -sticky => "nw");
    #$left_frame->Label(  -text => "General Program Options" )->grid(-columnspan => 2 );
   
#    $left_frame->Label(   -text => "vdrsync")->grid( 
#    $left_frame->Entry(   -textvariable => \$progs{"vdrsync.pl"})); 
#    $left_frame->Label(   -text => "dvdauthor")->grid(
#    $left_frame->Entry(   -textvariable => \$progs{dvdauthor} ));
#    $left_frame->Label(-text => "growisofs")->grid(
#    $left_frame->Entry(   -textvariable => \$progs{growisofs} ));
#    $left_frame->Label(-text => "vdrsync Output Dir")->grid(
#    $left_frame->Entry(   -textvariable => \$vdrsync_output_dir )); 
#    $left_frame->Label(-text => "vdrsync Basename")->grid(
#    $left_frame->Entry(   -textvariable => \$vdrsync_basename ));
#    $left_frame->Label(-text => "vdrsync Options")->grid(
#    $left_frame->Entry(   -textvariable => \$vdrsync_options )); 
#    $left_frame->Label(-text => "Capture Program")->grid(
#    $left_frame->Entry(   -textvariable => \$config{capture_prog} )); 
#    $left_frame->Label(  -text => " " )->grid(-columnspan => 2 );
    $left_frame->Label(  -text => "Directories" )->grid(-columnspan => 2 );
    
    $left_frame->Label(   -text => "Video Directory")->grid(
    $left_frame->Entry(   -textvariable => \$video_dir ), -sticky => "e"); 
    #$left_frame->Label(-text => "Default Directory for DVDs")->grid(
    #$left_frame->Entry(   -textvariable => \$config{dvd_output_dir} )); 
    $left_frame->Label(   -text => "Directory for temp files")->grid(
    $left_frame->Entry(   -textvariable => \$config{master_tmp_dir}), -sticky => "e");#, -validate => "focusout",  -validatecommand => sub { print "Calling from Entry\n";  &create_tmp_dir } )); 

    $left_frame->Label(  -text => " " )->grid(-columnspan => 2 );    
    $left_frame->Label(  -text => "DVD Options" )->grid(-columnspan => 2 );    
    $left_frame->Label(   -text => "Chapter Interval")->grid(
    $left_frame->Entry(   -textvariable => \$config{dvd_chapter_interval} ), -sticky => "e"); 
    $left_frame->Label(   -text => "Default Basename")->grid(
    $left_frame->Entry(   -textvariable => \$config{dvd_basename} ), -sticky => "e");
    $left_frame->Label(   -text => "Default Root Menu Title")->grid(
    $left_frame->Entry(   -textvariable => \$config{dvd_root_menu_title} ), -sticky => "e");
    $left_frame->Label(   -text  => "Default DVD Main Text")->grid(
    $left_frame->Entry(   -textvariable => \$config{dvd_root_label}), -sticky => "e"); 
    $left_frame->Label(   -text  => "Default DVD Movie Menu Title")->grid(
    $left_frame->Entry(   -textvariable => \$config{dvd_movie_title}), -sticky => "e"); 
    $left_frame->Label(   -text => "Chapter Menu Length (s)")->grid(
    $left_frame->Entry(   -textvariable => \$config{dvd_chapter_menu_len} ), -sticky => "e");
    $left_frame->Label(   -text => "Default Output Dir")->grid(
    $left_frame->Entry(   -textvariable => \$config{dvd_output_dir} ), -sticky => "e"); 
    
    $left_frame->Label(  -text => " " )->grid(-columnspan => 2 );
    
    $left_frame->Label(  -text => "Misc. Options" )->grid(-columnspan => 2 );    
    $left_frame->Label(   -text => "File size for cut recordings")->grid(
    $left_frame->Entry(   -textvariable => \$vs_cut_filesize ), -sticky => "e"); 
    $left_frame->Label(   -text => "Path (only if not already set!)")->grid(
    $left_frame->Entry(   -textvariable => \$config{path_str} ), -sticky => "e"); 
    $left_frame->Label(   -text => "DVD Device")->grid(
    $left_frame->Entry(   -textvariable => \$config{dvd_device} ), -sticky => "e"); 
    
     my $qm_cb = $left_frame->Checkbutton(-text => "Use Queue Manager",
                       -variable  => \$config{queue_manager},
                       #-command   => [ \&build_dvd_vdrsync_cmd ]
                       )->grid( -columnspan => "2");                       
    $left_frame->Balloon(-state => "balloon")->attach($qm_cb, -msg => "Enable or disable the experimental Queue Manager");
    
    #$left_frame->Label(-text => "")->grid( -columnspan => "2");
    
 
#    $left_frame->Label(-text => "Default directory for cut recordings")->grid(
#    $left_frame->Entry(   -textvariable => \$vs_cut_filedir )); 
    
#    $left_frame->Label(-text => "DVD Device")->grid(
#    $left_frame->Entry(   -textvariable => \$config{dvd_device} )); 
#    $left_frame->Label(-text => "Dummy2")->grid(
#    $left_frame->Entry(   -textvariable => \$Dummy2 )); 
    
    
    my $right_frame   = $config_page->Frame(-relief => "groove", -bd => 4)->grid( -column => 1, -row => 0, -sticky => "nw");
    #my $capconf_frame = $config_page->Frame(    -relief => "groove", -bd => 4)->grid(-column => "1", -row => "0", -sticky => "nw" );    
    my $capconf_label = $right_frame->Label(  -text => "Please choose capture program" )->grid(-columnspan => 2 ); 
    
    #~ # create the Radiobuttons that will let us change it
    
    foreach (qw(gop2jpeg ffmpeg transcode)) {
        $right_frame->Radiobutton(
            -text     => $_,
            -value    =>  $_,
            -variable => \$config{capture_prog},
            -command  => \&set_capture_prog)->grid(-sticky => "nw", -columnspan => 2);
    }

    
    my $mconf_label = $right_frame->Label(  -text => "Step Size" )->grid(-columnspan => 2 );
    $right_frame->Label( -text => "Small Step")->grid( 
    $right_frame->Entry(  -textvariable => \$config{small_step} )); #-validate => "focusout", -validatecommand => \&update_step_size,
    $right_frame->Label( -text => "Middle Step")->grid( 
    $right_frame->Entry( -textvariable => \$config{middle_step} )); 
    $right_frame->Label( -text => "Big Step")->grid( 
    $right_frame->Entry( -textvariable => \$config{big_step} )); 
    $right_frame->Label( -text => "Mouse Wheel")->grid( 
    $right_frame->Entry( -textvariable => \$config{wheel_step})); 
    
    $right_frame->Label( -text => "Preview Window Size" )->grid(-columnspan => 2 );
    $right_frame->Label( -text => "Width")->grid( 
    $right_frame->Entry( -textvariable => \$config{preview_width} )); #-validate => "focusout", -validatecommand => \&update_preview_size,
    $right_frame->Label( -text => "Height")->grid( 
    $right_frame->Entry( -textvariable => \$config{preview_height} )); #-validate => "focusout", -validatecommand => \&update_preview_size,
    
    
    my $cw_cb = $left_frame->Checkbutton(-text => "Use Control Window",
                       -variable  => \$config{control_window},
                       #-command   => [ \&build_dvd_vdrsync_cmd ]
                       )->grid();                       
    $right_frame->Balloon(-state => "balloon")->attach($cw_cb, -msg => "Show controls in extra Window");
    
    $config_page->Button(-text => "Save Settings", -command =>  \&save_config)->grid(-column => 1, -row => 1);
    
    
    $config_page->gridColumnconfigure(0, -weight => 0);
    $config_page->gridRowconfigure(0, -weight => 0);
    $config_page->gridColumnconfigure(1, -weight => 1);
    $config_page->gridRowconfigure(1, -weight => 1);
    $config_page->gridRowconfigure(2, -weight => 2);
}



###############################################
sub paint_movie_navigation_bar {
    my $parent = shift;
    my $x_pos  = shift;
    my $y_pos  = shift;
   
 
    $movie_nav_frame = $parent->Frame(-relief => "groove", -bd => 4)->grid(-column => $x_pos, -row => $y_pos, -columnspan => 1);
    $movie_nav_frame->Label(  -text => "Movie Navigation" )->grid(-column => 0, -row => 0, -columnspan => 6);

    $five_back_button  = $movie_nav_frame->Button(-text => "<<<", -command =>  [ \&show_pic_at_diff, (-1 * $config{big_step}) ] )->grid(-column => 0, -row => 1);#pack(
    $mw->Balloon(-state => "balloon")->attach($five_back_button, -msg => "rewind $config{big_step} sec");

    $one_back_button  = $movie_nav_frame->Button(-text => "<<", -command =>  [ \&show_pic_at_diff, (-1 * $config{middle_step}) ] )->grid(-column => 1, -row => 1);#pack(
    $mw->Balloon(-state => "balloon")->attach($one_back_button, -msg => "rewind $config{middle_step} sec");

    $prev_gop_button = $movie_nav_frame->Button(-text => "<", -command =>  [ \&show_pic_at_diff, (-1 * $config{small_step}) ] )->grid(-column => 2, -row => 1);#pack(
    $mw->Balloon(-state => "balloon")->attach($prev_gop_button, -msg => "back $config{small_step} sec");

    $next_gop_button = $movie_nav_frame->Button(-text => ">", -command =>  [ \&show_pic_at_diff, $config{small_step} ] )->grid(-column => 3, -row => 1);#pack(
    $mw->Balloon(-state => "balloon")->attach($next_gop_button, -msg => "forward $config{small_step} sec");

    $one_for_button  = $movie_nav_frame->Button(-text => ">>", -command =>  [ \&show_pic_at_diff, $config{middle_step} ] )->grid(-column => 4, -row => 1);#pack(
    $mw->Balloon(-state => "balloon")->attach($one_for_button, -msg => "forward $config{middle_step} sec");

    $five_for_button  = $movie_nav_frame->Button(-text => ">>>", -command =>  [ \&show_pic_at_diff, $config{big_step} ] )->grid(-column => 5, -row => 1);#pack(
    $mw->Balloon(-state => "balloon")->attach($five_for_button, -msg => "forward $config{big_step} sec");
}
    
###############################################
sub paint_mark_manipulation_bar {
    my $parent = shift;
    my $x_pos  = shift;
    my $y_pos  = shift;

    $mark_mani_frame = $parent->Frame(-relief => "groove", -bd => 4)->grid(-column => $x_pos, -row => $y_pos, -columnspan => 1);
    my $mark_label = $mark_mani_frame->Label(  -text => "Mark Manipulation" )->grid(-column => 0, -row => 0, -columnspan => 6 );

    #~ my $set_mark_button  = $mark_mani_frame->Button(-text => "Set Mark", -width => "10", -command =>  [ \&setmark] )->grid(-column => 0, -row => 1, -columnspan => 1);#pack(
    #~ $mw->Balloon(-state => "balloon")->attach($set_mark_button, -msg => "Set a mark at current position");

    #~ my $del_mark_button  = $mark_mani_frame->Button(-text => "Del Mark", -width => "10", -command =>  [ \&delmark] )->grid(-column => 3, -row => 1, -columnspan => 1);#pack(
    #~ $mw->Balloon(-state => "balloon")->attach($del_mark_button, -msg => "Delete the mark at the current position");

    my $prev_mark_button  = $mark_mani_frame->Button(-text => "< Jump ", -width => "8", -command =>  [ \&jumpmark, -1] )->grid(-column => 0, -row => 1, -columnspan => 1);#pack(
    $mw->Balloon(-state => "balloon")->attach($prev_mark_button, -msg => "Jump to the previous mark");

    my $toggle_mark_button  = $mark_mani_frame->Button(-text => "Toggle Mark", -width => "8", -command =>  [ \&togglemark] )->grid(-column => 1, -row => 1, -columnspan => 1);#pack(
    $mw->Balloon(-state => "balloon")->attach($toggle_mark_button, -msg => "Set or delete a mark at the current position");

    my $next_mark_button  = $mark_mani_frame->Button(-text => "Jump >", -width => "8", -command =>  [ \&jumpmark, 1] )->grid(-column => 2, -row => 1, -columnspan => 1);#pack(
    $mw->Balloon(-state => "balloon")->attach($next_mark_button, -msg => "Jump to the next mark");

    my $upload_button = $mark_mani_frame->Button(-text => 'Upload', -command =>  \&upload_marks)->grid(-column => 0, -row => 2, -columnspan => 1);
    $mw->Balloon(-state => "balloon")->attach($upload_button, -msg => "Upload marks via marks2pts");

    my $download_button = $mark_mani_frame->Button(-text => 'Download', -command =>  \&download_marks)->grid(-column => 1, -row => 2, -columnspan => 1);
    $mw->Balloon(-state => "balloon")->attach($download_button, -msg => "download marks via pts2marks");

    my $write_button  = $mark_mani_frame->Button(-text => "Save", -background => "red", -command =>  \&write_marks_file )->grid(-column => 2, -row => 2, -columnspan => 1);
    $mw->Balloon(-state => "balloon")->attach($write_button, -msg => "Write the Marks to marks.vdr");
 }


###############################################
sub paint_misc_control_bar {
    my $parent = shift;
    my $x_pos  = shift;
    my $y_pos  = shift;
    
    $misc_ctr_frame = $parent->Frame(-relief => "groove", -bd => 4)->grid(-column => $x_pos, -row => $y_pos, -columnspan => 1);
    $dial = $misc_ctr_frame->Canvas(qw/-width 120 -height 60 -bd 2 -relief sunken/)->grid(
                                -column     => 0, 
                                -row        => 0, 
                                -rowspan    => 2,
                                -columnspan => 1);
    $dial_line = $dial->createLine(qw/ 60 60 60 0   -arrow last -width 5/);
    my $dial_balloon = $mw->Balloon(-state => "balloon")->attach($dial, -msg => "The mouse wheel changes ffw/rew status.\nAlternativley hold down the left button and move the mouse\nA right click resets it.");

    my $chapter_mode_box = $misc_ctr_frame->Checkbutton(-text => 'Chapter Mode', -variable => \$chapter_mode)->grid(-column => 1, -row => 0, -columnspan => 1);
    my $gen_chap_button = $misc_ctr_frame->Button(-text => 'Generate Chapters', -command =>  \&generate_chapters)->grid(-column => 1, -row => 1, -columnspan => 1);
    $mw->Balloon(-state => "balloon")->attach($gen_chap_button, -msg => "Generate a chapter mark every 5 minutes");
    
    $chapter_mode_box->bind("<Button-1>", [ \&chapter_mode_click]);
    
    $dial->CanvasBind("<Button-4>", [ \&dial_wheelevent, -1 ]);
    $dial->CanvasBind("<Button-5>", [ \&dial_wheelevent, 1 ]);
    $dial->CanvasBind("<Button-3>", [ \&dial_canvas_right]);
    $dial->CanvasBind("<Button-1>", [ \&dial_canvas_Button_Press]);
    $dial->CanvasBind("<ButtonRelease-1>", [ \&dial_canvas_Button_Release]);
}
###############################################
sub paint_misc_button_bar {
    my $parent = shift;
    my $x_pos  = shift;
    my $y_pos  = shift;

    $misc_but_frame = $parent->Frame(-relief => "groove", -bd => 4)->grid(-column => $x_pos, -row => $y_pos, -columnspan => 1);

    my $player_button = $misc_but_frame->Button(-text => 'View Movie', -command => [ \&preview_movie, undef ])->grid(-column => 0, -row => 0, -columnspan => 1);
    $mw->Balloon(-state => "balloon")->attach($player_button, -msg => "Preview the movie from here");

    my $cplayer_button = $misc_but_frame->Button(-text => 'View Cut Movie', -command => [ \&preview_movie, "cut" ])->grid(-column => 1, -row => 0, -columnspan => 1);
    $mw->Balloon(-state => "balloon")->attach($cplayer_button, -msg => "Preview the cut movie from here");

    my $snapshot_button = $misc_but_frame->Button(-text => 'Take Snapshot', -command =>  \&take_snapshot)->grid(-column => 0, -row => 1 , -columnspan => 1);
    $mw->Balloon(-state => "balloon")->attach($snapshot_button, -msg => "Save current Image");

    my $cut_button  = $misc_but_frame->Button(-text => "Cut File", -background => "red", -command => [ \&movie_cut, $split ])->grid(-column => 1, -row => 1, -columnspan => 1);
    $mw->Balloon(-state => "balloon")->attach($cut_button, -msg => "Cut the movie");

    #############################################

    my $done_button = $misc_but_frame->Button(-text => "Done", -width => "30", -command => [\&clean_up])->grid(-columnspan => 2);#pack(
    $mw->Balloon(-state => "balloon")->attach($done_button, -msg => "Exit the Program");
}
#my $split_box = $mw->Checkbutton(-text => 'Split Output into single files', -variable => \$split)->grid(-column => 2, -row => 16, -columnspan => 3);
###############################################
sub paint_recording_notebook {

    my $canvas_frame = $mw->Frame(-relief => "groove", -bd => 4)->grid(-column => 0, -row => 0, -rowspan => 5);
    $can = $canvas_frame->Canvas(-width => $config{preview_width}, -height => $config{preview_height}, -background => "black")->grid(-column => 0, -row => 0);
    $prog_can = $canvas_frame->Canvas(-width => $config{preview_width}, -height => 30)->grid(-column => 0, -row => 5);
    my $total_rect = $prog_can->createRectangle(0, 0, $config{preview_width}, 30, -fill => 'white');
    $prog_rect = $prog_can->createRectangle(0,20, 0, 30, -fill => 'green');
    
    
    #~ my $mark_move_label = $mark_mani_frame->Label(  -text => "Move Marks" )->grid(-column => 0, -row => 3, -columnspan => 6 );
    #~ my $move_one_for_button  = $mark_mani_frame->Button(-text => ">", -command =>  [ \&movemark, 1] )->grid(-column => 3, -row => 4);
    #~ $mw->Balloon(-state => "balloon")->attach($move_one_for_button, -msg => "Move the mark to the next GOP");

    #~ my $move_five_for_button  = $mark_mani_frame->Button(-text => ">>", -command =>  [ \&movemark, 5] )->grid(-column => 4, -row => 4);
    #~ $mw->Balloon(-state => "balloon")->attach($move_five_for_button, -msg => "Move the Mark 5 GOPs forward");

    #~ my $move_twenty_for_button  = $mark_mani_frame->Button(-text => ">>>",  -command =>  [ \&movemark, 20] )->grid(-column => 5, -row => 4);
    #~ $mw->Balloon(-state => "balloon")->attach($move_twenty_for_button, -msg => "Move the Mark 20 GOPs forward");

    #~ my $move_one_back_button = $mark_mani_frame->Button(-text => "<",  -command =>  [ \&movemark, -1] )->grid(-column => 2, -row => 4);
    #~ $mw->Balloon(-state => "balloon")->attach($move_one_back_button, -msg => "Move the mark to the previous GOP");;

    #~ my $move_five_back_button = $mark_mani_frame->Button(-text => "<<",  -command =>  [ \&movemark, -5] )->grid(-column => 1, -row => 4);
    #~ $mw->Balloon(-state => "balloon")->attach($move_five_back_button, -msg => "Move the mark 5 GOPs backwards");

    #~ my $move_twenty_back_button = $mark_mani_frame->Button(-text => "<<<",  -command =>  [ \&movemark, -20] )->grid(-column => 0, -row => 4);
    #~ $mw->Balloon(-state => "balloon")->attach($move_twenty_back_button, -msg => "Move the mark 20 GOPs backwards");
    #############################################

    my $info_frame = $mw->Frame(-relief => "groove", -bd => 4 )->grid(-column => 0, -row => 5, -columnspan => 2 , -sticky => "nsew");
    $total_length_label = $info_frame->Label( -text => "Length: 0 / 0", -anchor => "w", -justify => "left" )->grid(-column => 1, -row => 0, -columnspan => 1  );

    my $cut_length_value = $info_frame->Label( -textvariable => \$cut_length )->grid(-column => 2, -row => 0, -columnspan => 1);

    my $pos_info_value = $info_frame->Label( -textvariable => \$current_pos )->grid(-column => 0, -row => 0, -columnspan => 1);

    #my $mark_no_value = $info_frame->Label( -textvariable => \$mark_no )->grid(-column => 0, -row => 3, -columnspan => 5 );

    #my $show_pos_value = $info_frame->Label( -textvariable => \$click_pos )->grid(-column => 0, -row => 4, -columnspan => 5 );

    ##############################################                    

    my $progress = $mw->ProgressBar(-length => 400, -from => 0, -to => 100, -blocks => 50, 
                -colors => [0, 'green', 50, 'yellow' , 80, 'red'],    
                -variable => \$percent_done)->grid(
                                -column => 0, 
                                -row => 6, 
                                -columnspan => 2);




    #my $gen_chap_shots_button = $mw->Button(-text => 'Create Chapter Shots', -command =>  \&create_chapter_images)->grid(-column => 4, -row => 8, -columnspan => 3);
    #my $gen_chap_shots_balloon = $mw->Balloon(-state => "balloon");
    #$gen_chap_shots_balloon->attach($gen_chap_shots_button, -msg => "Generate a chapter mark every 5 minutes");

    ################################################
}


###############################################
sub paint_controls {
    #
    # Are the Controls currently in the right frame? If not, destroy all controls
    #
    if (($control_window && (! $config{control_window})) ||
       ((! $control_window) && $config{control_window})) {
        if ($movie_nav_frame) {
            $movie_nav_frame->destroy();
            $movie_nav_frame = "";
        }
        if ($mark_mani_frame) {
            $mark_mani_frame->destroy();
            $mark_mani_frame = "";
        }
        if ($misc_but_frame) {
            $misc_but_frame->destroy();
            $misc_but_frame = "";
        }
        if ($misc_ctr_frame) {
            $misc_ctr_frame->destroy();
            $misc_ctr_frame = "";
        }    
        if ($control_window) {
            $control_window->destroy;
            $control_window = "";
        }
    } 
    
    my $cparent;
    if ($config{control_window}) {
        print "Controls will be in the extra window\n";
        if (! Exists($control_window)) {
            $control_window = $rmw->Toplevel( );
            $control_window->title("Control Window");
            $control_window->protocol("WM_DELETE_WINDOW", \&ctr_win_close);
            #~ $control_window->Button(-text => "Close", 
                #~ -command => sub { $control_window->withdraw })->grid;
        } else {
            $control_window->deiconify( );
            $control_window->raise( );
        }
        $cparent = $control_window;  
        
    } else {
        print "Controls will be in the main window\n";
        $cparent = $mw;
    }
    
    #############################################
    if (($config{movie_nav_frame}) && (! $movie_nav_frame)) {
        print "paint movie nav\n";
        paint_movie_navigation_bar ($cparent, 1, 0);
    } elsif ((! $config{movie_nav_frame}) && ($movie_nav_frame)) {
        $movie_nav_frame->destroy;
        $movie_nav_frame = "";
    }
    #############################################
    if (($config{mark_mani_frame}) && (! $mark_mani_frame)) {
        print "paint mark mani\n";
        paint_mark_manipulation_bar($cparent, 1, 1);
    } elsif ((! $config{mark_mani_frame}) && ($mark_mani_frame)) {
        $mark_mani_frame->destroy;
        $mark_mani_frame = "";
    }
    #############################################
    if (($config{misc_ctr_frame}) && (! $misc_ctr_frame)) {
        print "paint ctr frame\n";
        paint_misc_control_bar($cparent, 1, 2);
    } elsif ((! $config{misc_ctr_frame}) && ($misc_ctr_frame)){
        $misc_ctr_frame->destroy;
        $misc_ctr_frame = "";
    }
    #############################################
    if (($config{misc_but_frame}) && (! $misc_but_frame)) {
        print "paint but frame\n";
        paint_misc_button_bar($cparent, 1, 3);
    } elsif ((! $config{misc_but_frame}) && ($misc_but_frame)){
        $misc_but_frame->destroy;
        $misc_but_frame = "";
    }
    #############################################
}    


###############################################
sub define_bindings {
    $prog_can->CanvasBind("<Button-1>", [ \&jump_xy, Ev('x'), Ev('y') ]);
    $prog_can->CanvasBind("<Button-3>", [ \&jump_xy, Ev('x'), Ev('y') ]);
    $prog_can->CanvasBind("<Double-Button-1>", [ \&jumpmark_click, -1 ]);
    $prog_can->CanvasBind("<Double-Button-3>", [ \&jumpmark_click, 1 ]);
    $prog_can->CanvasBind("<Button-2>", [ \&togglemark ]);
    $prog_can->CanvasBind("<Button-4>", [ \&wheelevent, -1 ]);
    $prog_can->CanvasBind("<Button-5>", [ \&wheelevent,  1 ]);
    $can->CanvasBind("<Button-1>", [ \&jump_xy, Ev('x'), Ev('y') ]);
    $can->CanvasBind("<Button-3>", [ \&jump_xy, Ev('x'), Ev('y') ]);
    $can->CanvasBind("<Double-Button-1>", [ \&jumpmark_click, -1 ]);
    $can->CanvasBind("<Double-Button-3>", [ \&jumpmark_click, 1 ]);
    $can->CanvasBind("<Button-2>", [ \&togglemark ]);
    $can->CanvasBind("<Button-4>", [ \&wheelevent, -1 ]);
    $can->CanvasBind("<Button-5>", [ \&wheelevent,  1 ]);

    
    $rmw->bind("<KeyPress>", [ \&Key_pressed ]);
    $rmw->bind("<KeyRelease>", [ \&Key_released ]);
    
}

#paint_divx_notebook();
#paint_svcd_notebook();
#paint_vdrsync_notebook();

paint_recording_notebook();
read_config();
paint_controls();
define_bindings;
paint_dvd_notebook();
paint_config_notebook();



init();
$rmw->protocol("WM_DELETE_WINDOW", \&clean_up);

print "finished init\n";
MainLoop;

sub ctr_win_close {
    if ($misc_ctr_frame) {
        $misc_ctr_frame->destroy;
        $misc_ctr_frame = "";
    }
    if ($misc_but_frame) {
        $misc_but_frame->destroy;
        $misc_but_frame = "";
    }
    if ($movie_nav_frame) {
        $movie_nav_frame->destroy;
        $movie_nav_frame = "";
    }
    if ($mark_mani_frame) {
        $mark_mani_frame->destroy;
        $mark_mani_frame = "";
    }
    $control_window->destroy;
    $control_window = "";
 
}
sub recording_notebook_raised {
    paint_controls();
    update_step_size();
    if (($config{preview_height} !=  $can->cget(-height)) ||
        ($config{preview_width}  !=  $can->cget(-width))) {
        update_preview_size();
    }
}

sub update_preview_size {
    
    print "Trying to Update Preview Size....\n";
    %jpegs= ();
    print "Deleted all jpegs: " . (join " ",(keys(%jpegs))) . "\n";
    $can->configure(-width => $config{preview_width}, -height => $config{preview_height}, -background => "black");
    
    $prog_can->configure(-width => $config{preview_width}, -height => 30);
    $prog_can->createRectangle(0, 0, $config{preview_width}, 30, -fill => 'white');
    $prog_rect = $prog_can->createRectangle(0,20, 0, 30, -fill => 'green');
    
    $current_picture = $can->createImage($config{preview_width} / 2, $config{preview_height} / 2, -image => $canvas_image);
}
sub update_dvd_stats {
    
    print "Updating DVD stats\n";
    $dvd_title_no = scalar(@titlesets);
    $dvd_length = 0;
    $dvd_size   = 0;
    $dvd_vsize  = 0;
    $dvd_asize  = 0;
    $dvd_rfactor = 1;
    
    if (!$dvd_title_no) {
        return;
    }

    if ($config{dvd_cut}) {
        foreach my $tset (@titlesets) {
            if ($config{dvd_cut}) {
                $dvd_length += $$tset{ts_len_c};
                $dvd_size   += $$tset{ts_size_c};
            } else {
                $dvd_length += $$tset{ts_len};
                $dvd_size   += $$tset{ts_size};
            }
            $dvd_vsize += $$tset{ts_vsize};
            $dvd_asize += $$tset{ts_asize};
            print "adding to vsize: $$tset{ts_vsize}\n";
        } 
    }
    
    $dvd_vsize = int($dvd_vsize);
    $dvd_size = int($dvd_size / 1000000);
    $dvd_length = $recording->frames2HMS($dvd_length*25);
    
    my $too_much = 0;
    
    if ($dvd_size >= 4440) {
        print "DVD too big,needs resizeing\n";
        $too_much = $dvd_size - 4440;
        $dvd_rfactor = int(100 * $dvd_vsize / ($dvd_vsize - $too_much)) / 100 ;
    }
    
}

sub show_titlesets {

    if ((!@titlesets) || (@titlesets && ($indir ne $titlesets[-1]{rec_dir}))) {
        if (yes_no_box("The current recording has not been added to the DVD. Do you want to add it now?") eq "Yes") {
            add_titleset();
        }
    }
    print "Showing current Titlesets\n";
    my $db;
    $db = $rmw->Toplevel(-title => 'Current Titlesets');
    draw_titleset_dialog($db);
}

sub draw_titleset_dialog {

    print "Drawing titleset dialog\n";
    my $db = shift;
    foreach my $kid ($db->children) {
        print "Trying to destroy $kid\n";
        $kid->destroy();
    }
    
    for (my $i = 0; $i < scalar(@titlesets); $i++) {
        print "Adding $titlesets[$i]{rec_dir}\n";
        $db->Label( -text=> "Directory:")->grid(
        $db->Label( -text=> "$titlesets[$i]{rec_dir}"));
        $db->Label( -text=> "Title:")->grid(
        $db->Label( -text=> "$titlesets[$i]{rec_title}"));
        $db->Label( -text=> "Main Menu Title:")->grid(
        $db->Label( -text=> "$titlesets[$i]{root_title}"));
        $db->Label( -text=> "Main Menu Label:")->grid(
        $db->Label( -text=> "$titlesets[$i]{root_label}"));
        $db->Label( -text=> "Ignored Audio Streams:")->grid(
        $db->Label( -text=> "$titlesets[$i]{ignore}"));
        $db->Button( -text=> "Remove from DVD", 
                -command => [ \&remove_titleset, $db, $i ] )
                ->grid(-columnspan => "2");
    }
    $db->Button(-text => "Close" =>,
                    -command => sub { $db->withdraw })->grid(-columnspan => "2");
}


sub add_titleset {
    if ($indir) {
        my $ignore;
        foreach my $sid (keys(%streams)) {
            if ($sid eq "video") {next}
            if (! $streams{$sid}{include}) {
                print "$sid will be skipped\n";
                $ignore .= "$sid,"
            }
        }
        
        if ($ignore) {
            chop $ignore;
        } else {
            $ignore = "none";
        }
        push @titlesets, {
            rec_dir     => $indir,
            rec_title   => $dvd_movie_title,
            root_title  => $dvd_root_menu_title,
            root_label  => $dvd_root_label,
            ts_len_c    => $selected_rec{len_cut},
            ts_len      => $selected_rec{len},
            ts_size_c   => $selected_rec{size_cut},
            ts_size     => $selected_rec{size},
            ts_asize    => $selected_rec{s_audio},
            ts_vsize    => $selected_rec{s_video},
            
            #menu_text   => $dvd_movie_menu_text,
            ignore      => $ignore,
        };
        print "Added the title set: $titlesets[-1]{rec_dir}\n";
        print "The following titlesets are now selected:\n";
        
        foreach my $tset (@titlesets) {
            foreach(keys(%$tset)) {
                print "$tset $_  $$tset{$_} \n";
            }
        }
        $rmw->Busy;
        update_dvd_stats();
        $rmw->Unbusy;
    }
    
}

sub remove_titleset {
    print "removing titleset\n";
    my $db = shift;
    my $titleset_no = shift;
    if (scalar(@titlesets) <= 1) {
        @titlesets = ();
    } else {
        my @harray;
        for (my $i = 0; $i < scalar(@titlesets); $i++) {
            if ($i == $titleset_no) {
                next;
            }
            push @harray, $titlesets[$i];
        }
        @titlesets = @harray;
    }
    $rmw->Busy;
    update_dvd_stats();
    draw_titleset_dialog($db);
    $rmw->Unbusy;
}

sub save_config {
    my $home_dir = (getpwuid($>))[7];
    if (! -s "$home_dir/.vdrsync/vdrsync-gui.conf") {
        if (yes_no_box("Config File does not exists, should I create it?\n") ne "Yes") {
            error_box("Setings not saved!\n");
            return;
        }
    }
    if (! -d "$home_dir/.vdrsync/") {
        if (! mkdir "$home_dir/.vdrsync/") {
            error_box("Could not create $home_dir/.vdrsync/: $!\n");
            return;
        }
    }
    open CFH, ">$home_dir/.vdrsync/vdrsync-gui.conf" or do {
            error_box("Could not open $home_dir/.vdrsync/vdrsync-gui.conf for writing\nSettings not saved!");
            return;
    };
  
    foreach (sort(keys(%config))) {
        printf CFH "%-20s : %-20s\n", $_, $config{$_} if defined $config{$_};
    }
    close CFH;
}

sub read_config {
    my $home_dir = (getpwuid($>))[7];
    if (! -s "$home_dir/.vdrsync/vdrsync-gui.conf") {
        return;
        if (yes_no_box("Config File does not exists, should I create it with defaults / current settings?\n") ne "Yes") {
            error_box("Settings not saved!\n");
            return;
        } else {
            save_config;
        }
    }
    open CFH, "$home_dir/.vdrsync/vdrsync-gui.conf" or do {
            error_box("Could not open $home_dir/.vdrsync/vdrsync-gui.conf for reading\nUsing default values!");
            return;
    };
    while (<CFH>) {
        my ($key, $val) = split/\s+:\s+/, $_;
        $key =~s/\s//gm;
        $val =~s/^\s*//gm;
        $val =~s/\s*$//gm;
        $config{$key} = $val if $val ;
    }
    print "Config File read\n";
    
}

sub set_capture_prog {
    print "redefining capture Program...\n";
    if (! $progs{$config{capture_prog}}) {
        error_box("$config{capture_prog} could not be found!\nThis program can not be used without an external capture program!\nRefer to the \"Requirements\" section in the README.\n");
    }
}

sub update_step_size {
    #print "Now redefine callbacks\n";
    if (! $prog_can) {
        return;
    }
    $prog_can->CanvasBind("<Button-4>", [ \&wheelevent, (-1 * $config{wheel_step}) ]);
    $prog_can->CanvasBind("<Button-5>", [ \&wheelevent, $config{wheel_step} ]);
    $can->CanvasBind("<Button-4>", [ \&wheelevent, (-1 * $config{wheel_step}) ]);
    $can->CanvasBind("<Button-5>", [ \&wheelevent, $config{wheel_step} ]);
    if ((! $five_back_button) || (! $config{movie_nav_frame})){
       $mw->focus; # Just to get the focus out of the Entry Widgets
       return 1;
    }
   
    $five_back_button->configure(-command =>  [ \&show_pic_at_diff, (-1 * $config{big_step})    ] );
    $one_back_button->configure (-command =>  [ \&show_pic_at_diff, (-1 * $config{middle_step}) ] );
    $prev_gop_button->configure (-command =>  [ \&show_pic_at_diff, (-1 * $config{small_step})  ] );
    $next_gop_button->configure (-command =>  [ \&show_pic_at_diff, (     $config{small_step})  ] );
    $one_for_button->configure  (-command =>  [ \&show_pic_at_diff, (     $config{middle_step}) ] );
    $five_for_button->configure (-command =>  [ \&show_pic_at_diff, (     $config{big_step})    ] );
    $mw->focus; # Just to get the focus out of the Entry Widgets
   
    return 1; # Returning 1 is crucial for the update of the values, otherwise the entry widget does not accpet the new values....
}


sub build_dvd_vdrsync_cmd  {
    
    if (! @titlesets) {
        return;
    }
    print "building VDRsync cmd \n";
    
    #
    # First some basic vars are checked an set if necessary
    #
    
    if (! $dvd_output_dir) {
        $dvd_output_dir = $config{dvd_output_dir} || "/tmp";
    } 
    if (! $dvd_basename) {
        $dvd_basename = $config{dvd_basename} || "VDRsyncDVD";
    }
    if (! $dvd_root_label) {
        $dvd_root_label = $config{dvd_root_label} || "Start Movie";
    }
    if (! $dvd_root_menu_title) {
        $dvd_root_menu_title = $config{dvd_root_menu_title} || "Created by VDRsync";
    }
    if (! $dvd_movie_title) {
        $dvd_movie_title = $config{dvd_movie_title} || "Created by VDRsync";
    }
    
    #
    # Start the construction of the VDRsync CL
    #
    $dvd_vdrsync_cmd = $progs{"vdrsync.pl"}  ;
   
    if ($config{dvd_chapter_menu}) {
        $dvd_vdrsync_cmd .=  " --dvd-chapter-menus " ;
        $config{dvd_menu} = 1;
    }

    if ($config{dvd_menu}) {
        if (! $config{dvd_chapter_menu}) {
            $dvd_vdrsync_cmd .=  " --dvd-menu " ;
        }
    }
    if ($config{dvd_use_pipe}) {
        $dvd_vdrsync_cmd .=  " --use-pipe " ;
    }
    if ($config{dvd_cut}) {
        $dvd_vdrsync_cmd .=  " --cut " ;
    }
    if ($config{dvd_fit}) {
        $dvd_vdrsync_cmd .=  " --fit-to-dvd " ;
    }

    
    
    print "Starting to work with the titlesets...\n";
    my $ignore; 
    foreach my $tset (@titlesets) {
        print "now analysing $$tset{rec_dir} for ignore settings\n";
        if ($$tset{ignore} ne "none") {
            print "$$tset{ignore} will be skipped\n";
            $ignore .= "$$tset{ignore}+"
        } else {
            $ignore .= "+"
        }
        
    }
    if ($ignore !~ /^\++$/) {
        chop $ignore;
        print "Adding ignore flag\n";
        $dvd_vdrsync_cmd .= " --ignore $ignore ";
        print "\nAdding  --ignore $ignore ";
    } else {
        print "nothing to ignore found\n";
    }

    #
    # Build the parameter list for the Titleset's Menus (=Movie's Menus) Texts
    #
    if ($config{dvd_menu}) {
        $dvd_vdrsync_cmd .= " --dvd-menu-texts ";
        foreach my $tset (@titlesets) {
            print "now analysing $tset for $$tset{rec_dir}, looking for summary.vdr\n";
            if (-s "$$tset{rec_dir}/summary.vdr" && $config{dvd_menu}) {
                $dvd_vdrsync_cmd .= "\"$$tset{rec_dir}/summary.vdr\",";
                #print "\nadding --dvd-menu-texts \"$indir/summary.vdr\" ";
            } else {
                $dvd_vdrsync_cmd .= "\"No Summary Available\",";
            }
        } 
        chop  $dvd_vdrsync_cmd;
    }
    


    #
    # Just ignore chaptermarks for a while
    #
    #if (-s "$indir/chapter_marks.vdr" && $config{dvd_chapter_menu}) {
    #    $dvd_vdrsync_cmd .= " --chapter \"$indir/chapter_marks.vdr\" ";
    #
    #}
    
    
    #
    # The DVD Root Menu title is either what has been specified for the last titleset, or what is currently displayed, or a default
    #
    # Also add the root menu labels, next to the Start Buttons...
    #
    
    if ($config{dvd_menu}) {
        $dvd_vdrsync_cmd .=  " --dvd-root-title "; 
        $dvd_vdrsync_cmd .= $titlesets[-1]{root_menu_title} || " \"$dvd_root_menu_title\" " || "DVD Menu Created by VDRsync";
        $dvd_vdrsync_cmd .= " --dvd-root-labels \"";
        foreach my $tset (@titlesets) {
            print "Extracting root menu label from $tset for $$tset{rec_dir}\n";
            $dvd_vdrsync_cmd .=  "$$tset{root_label}|" || "Start Movie|";
        }
        chop  $dvd_vdrsync_cmd;
        $dvd_vdrsync_cmd .= "\"";
    }
    
    
    #
    # Now we add the Titlesets menu titles
    #
    if ($config{dvd_menu}) {
        $dvd_vdrsync_cmd .=  " --dvd-menu-titles "; 
        foreach my $tset (@titlesets) {
            print "Extracting  menu title from $tset for $$tset{rec_dir}\n";
            $dvd_vdrsync_cmd .=  "\"$$tset{rec_title}\"," || "\"Created by VDRsync\",";
        }
        chop $dvd_vdrsync_cmd;
    }
    

    if ( $dvd_user_cmd_add) {
        print "Found user Addition...\n";
        $dvd_vdrsync_cmd .= " $dvd_user_cmd_add ";
    }
    $dvd_vdrsync_cmd .=  " -o \"$dvd_output_dir\" --basename \"$dvd_basename\" --master-dvd ";
    foreach my $tset (@titlesets) {
        print "Extracting  menu title from $tset for $$tset{rec_dir}\n";
        $dvd_vdrsync_cmd .=  " \"$$tset{rec_dir}\" ";
    }
    
}

sub fill_recording_list {
    
    my $old_dir = getcwd;
    print "old dir is $old_dir\n";
    my @raw_recs;

    while (! @raw_recs) {
        print "ok, no raw recs...\n";
        #$counter++;
        while ((! $video_dir) || (! -d $video_dir)) {
            my $db = $rmw->DialogBox(-title => 'Please select a Directory', -buttons => ['Ok', 'Cancel'], 
                    -default_button => 'Ok');
            my $dt = $db->Scrolled("DirTree", -directory => "/",
                            -scrollbars => "se",
                            -width => '40',
                            -height => '25',                            
                            )->pack(-fill => 'both', -expand => 1);
            my $answer = $db->Show();    
            if ($answer eq "Ok") {
                ($video_dir) = $dt->infoSelection if ($answer eq "Ok");
            } else {
                return ();
            }
        }
        
        if ( -d $video_dir )  {
            #print "searching for recordings under $video_dir $counter\n";
            
            chdir $video_dir or die "can not change into $video_dir\n";
            print "Changed to dir " . getcwd ."\n";
    
            @raw_recs = sort(`find ./ -name index.vdr -print`);
            chdir $old_dir or die "Can not change back to $old_dir\n";
            if (! @raw_recs) {
                $video_dir = "";
            }
        } else {
            print "Not searchin for recordings under $video_dir, cause is does not exist\n";
            print "what the f***... Could not find a indir\n";
            return ();
        }
        
    }
    return @raw_recs;
}
sub rec_selection_dialog {
    my @raw_recs = @_ ;
    if (!@raw_recs) {
        @raw_recs = fill_recording_list();
        if (!@raw_recs) {
            return -1;
        }
    }
    my %rec_hash = %{ generate_recording_listing( \@raw_recs) };
    
    #foreach my $ll (keys(%rec_hash)){
    #    print "$ll: $rec_hash{$ll}{rec_name}\n$rec_hash{$ll}{rec_date}\n";
    #}
    my $sort_order = "name";
    my $i = 0;
    my $sel_rec;
    my $db = $rmw->DialogBox(-title => 'Please select a recording', -buttons => ['Ok', 'Cancel', 'Change Directory'], 
                -default_button => 'Ok');
    
    
    
    # Create the vertical Scrollbar
    my $scrollbar = $db->Scrollbar( );
    
    my $lb1 = $db->Listbox( -selectmode => "single", -width => "0", -yscrollcommand => ['set' => $scrollbar]);
    my $lb2 = $db->Listbox( -selectmode => "single", -width => "0", -yscrollcommand => ['set' => $scrollbar]);
    my $lb3 = $db->Listbox( -selectmode => "single", -width => "0", -yscrollcommand => ['set' => $scrollbar]);

    #Pack the Scrollbar first so that it doesn't disappear when we resize
    $scrollbar->pack(-side => 'right', -fill => 'y');
    $lb1->pack(-side => 'left', -fill => 'both');            
    $lb2->pack(-side => 'left', -fill => 'both');            
    $lb3->pack(-side => 'left', -fill => 'both');            
    
   
    
    foreach (qw(name date group group_date  )) {
        $db->Radiobutton(-text => $_,
                   -value    => $_,
                   -variable => \$sort_order,
                   -command  => [ \&set_sort_order, \$sort_order, $lb1, $lb2, $lb3, \%rec_hash] )->pack(-side => 'top', -anchor => "nw");
    }
    #, $lb1, $lb2, $lb3 
    
    sort_recording_list (\%rec_hash, "name");

    foreach my $dummy  (@recording_list) {
        $lb1->insert('end', $rec_hash{$dummy}{rec_name});
        $lb2->insert('end', $rec_hash{$dummy}{rec_group});
        $lb3->insert('end', $rec_hash{$dummy}{rec_date});
    } 
    #Configure the Scrollbar to talk to the Listbox widgets
    
    $scrollbar->configure(-command =>  sub { $lb1->yview(@_); $lb2->yview(@_); $lb3->yview(@_) }  );
   
    
    $lb1->bind('<Button-4>' => [ \&wheel_selection_dialog, -1, $lb1 ,$lb2, $lb3 ]);   
    $lb1->bind('<Button-5>' => [ \&wheel_selection_dialog,  1, $lb1 ,$lb2, $lb3 ]);
    $lb2->bind('<Button-4>' => [ \&wheel_selection_dialog, -1, $lb1 ,$lb2, $lb3 ]);   
    $lb2->bind('<Button-5>' => [ \&wheel_selection_dialog,  1, $lb1 ,$lb2, $lb3 ]);
    $lb3->bind('<Button-4>' => [ \&wheel_selection_dialog, -1, $lb1 ,$lb2, $lb3 ]);   
    $lb3->bind('<Button-5>' => [ \&wheel_selection_dialog,  1, $lb1 ,$lb2, $lb3 ]);
    
               

    my $answer = $db->Show();    
    
    if ($answer eq "Ok") {
        $sel_rec =  ($lb1->curselection)[0];
        print "now selected: $recording_list[$sel_rec]\n";
        %selected_rec = %{$rec_hash{$recording_list[$sel_rec]}};
        $selected_rec{rec_dir} = $recording_list[$sel_rec];
        $selected_rec{rec_dir} =~ s/^.\///;
        $selected_rec{rec_dir} = "$video_dir/" . $selected_rec{rec_dir};
        
        print "retunrning $selected_rec{rec_dir}\n";
        return "$selected_rec{rec_dir}";
    } elsif ($answer eq "Change Directory") {
        $video_dir = "";
        return (rec_selection_dialog(fill_recording_list()));
    } else {
        return "";
    }
    
    #print "Got: $rec_paths[-1]\n at $rec_dates[-1]\n and $rec_names[-1]\n";
    #die "$sel_rec was choosen, that is $indir\n";
}


sub wheel_selection_dialog {
    my $dummy = shift;
    #print "Got  " . (join " ", @_) ."\n";
    my $dir = shift;
    my $lb1 = shift;
    my $lb2 = shift;
    my $lb3 = shift;
   
    my $sel = $lb1->curselection  || 0;
    $lb1->selectionClear($sel); 
    $lb2->selectionClear($sel); 
    $lb3->selectionClear($sel); 
    #print "sel is $sel\n";
    if (! $sel || $sel < 1) {
        $sel = 1;
    }
    my @helpl = $lb1->get(0, 'end' );
    my $max = $#helpl;
    if ( $sel >= $max) {
        $sel = $max - 1;
    }
    $lb2->selectionSet($sel + $dir); 
    $lb2->see($sel + $dir); 
    $lb3->selectionSet($sel + $dir); 
    $lb3->see($sel + $dir); 
    $lb1->selectionSet($sel + $dir); 
    $lb1->see($sel + $dir); 
}

sub set_sort_order {
    my $sort_order = shift;
    #print "Got order $$sort_order and " . (join " ", @_) ."\n";
    my ($lb1, $lb2, $lb3, $rec_hash_ref) = @_;
    $lb1->delete(0, 'end');
    sort_recording_list($rec_hash_ref, $$sort_order); # Here we update @recording_lsit
    
    $lb1->delete(0, 'end');
    $lb2->delete(0, 'end');
    $lb3->delete(0, 'end');
    foreach my $dummy  (@recording_list) {
        $lb1->insert('end', $$rec_hash_ref{$dummy}{rec_name});
        $lb2->insert('end', $$rec_hash_ref{$dummy}{rec_group});
        $lb3->insert('end', $$rec_hash_ref{$dummy}{rec_date});
    } 
    #$lb1->activate(0);
    $lb1->focus();
    
    
}

sub generate_recording_listing {
    
    my @raw_recs = @{ shift @_ };
    my %rec_hash;
    my $dummy;
 
    for (my $i = 0; $i < scalar(@raw_recs); $i++) {
        chomp($raw_recs[$i]);
        my @path_bits = (split /\//, $raw_recs[$i]);
        $raw_recs[$i] = (join "/", @path_bits[0 .. (scalar(@path_bits) - 2)]) . "/"; 
        #$raw_recs[$i] = substr(@raw_recs[$i],1);
        
        $dummy = (split /\//, $raw_recs[$i])[-2];
        #$dummy = @path_bits[-2];
        $dummy =~ s/_/ /gm;
        $dummy =~ s/^ $/unknown/gm;
    
        $rec_hash{$raw_recs[$i]}{rec_name} = $dummy;
        
        if ( scalar(@path_bits) > 4) {
            #print "morethan4 $raw_recs[$i]\n";
            #$dummy = (join "/",@path_bits[(scalar(@path_bits)-5) .. (scalar(@path_bits)-4)]); 
            
            $dummy = (split /\//, $raw_recs[$i])[-3];
            $dummy =~s/_/ /gm;
            #$rec_hash{$raw_recs[$i]}{rec_group} =  $rec_hash{$raw_recs[$i]}{rec_name};
            $rec_hash{$raw_recs[$i]}{rec_group} = $dummy
            #$rec_hash{$raw_recs[$i]}{rec_name} = $dummy;
        } else {
            $rec_hash{$raw_recs[$i]}{rec_group} = " ";
        }
        
        ($dummy) = ((split /\//, $raw_recs[$i])[-1] =~ /(\d+-\d+-\d+[.:]\d+[.:]\d+)/);
        if (! $dummy ) {
            $dummy = "unknown";
        }
        $dummy =~ s/\./:/gm;
        $dummy =~ s/:/ /;
        
        $rec_hash{$raw_recs[$i]}{rec_date} = $dummy;
 
    
        if ( -s "$raw_recs[$i]/marks.vdr") {
            $rec_hash{$raw_recs[$i]}{rec_marks} = 1;
        } else {
            $rec_hash{$raw_recs[$i]}{rec_marks} = 0;
        }
        if ( -s "$raw_recs[$i]/ptsmarks.vdr") {
            $rec_hash{$raw_recs[$i]}{rec_ptsmarks} = 1;
        } else {
            $rec_hash{$raw_recs[$i]}{rec_ptsmarks} = 0;
        }
    }
    return \%rec_hash;
}

sub sort_recording_list {
    my %rec_hash = %{ shift @_};
    my $sort_order = shift;
    if ($sort_order eq "date") {
        @recording_list =  (sort { $rec_hash{$b}{rec_date} cmp $rec_hash{$a}{rec_date} 
                                                ||
                    lc($rec_hash{$a}{rec_name}) cmp lc($rec_hash{$b}{rec_name})
                                                ||
                    lc ($rec_hash{$b}{rec_group}) cmp lc($rec_hash{$a}{rec_group}) 
        } keys %rec_hash);
    } elsif ($sort_order eq "name") {
    
        @recording_list =  (sort { lc($rec_hash{$a}{rec_name}) cmp lc($rec_hash{$b}{rec_name})      
                                                ||
                    lc($rec_hash{$b}{rec_group}) cmp lc($rec_hash{$a}{rec_group}) 
                                                ||
                    $rec_hash{$b}{rec_date} cmp $rec_hash{$a}{rec_date}
        } keys %rec_hash) ;
    
    } elsif ($sort_order eq "group") {
    
        @recording_list =  (sort { lc($rec_hash{$a}{rec_group}) cmp lc($rec_hash{$b}{rec_group})
                                                ||
                    lc($rec_hash{$a}{rec_name}) cmp lc($rec_hash{$b}{rec_name})     
                                                ||
                    $rec_hash{$b}{rec_date} cmp $rec_hash{$a}{rec_date}
        } keys %rec_hash) ;
    } 
    elsif ($sort_order eq "group_date" ){
        @recording_list =  (sort { lc($rec_hash{$a}{rec_group}) cmp lc($rec_hash{$b}{rec_group}) 
                                                 ||
                        lc($rec_hash{$b}{rec_date}) cmp lc($rec_hash{$a}{rec_date})             
#                                                 ||
#                         $rec_hash{$b}{rec_date} cmp $rec_hash{$a}{rec_date}
        } keys %rec_hash) ;
    } else {
        die "Did not get no real sortorder...but $sort_order\n";
    }
    #return \@recording_list;
}


sub Key_released {
    my $keysym = $Tk::event->K;
    #print 'Keysym='.  $keysym . ', numeric=', $Tk::event->N, "\n";
    if ($keysym eq "Shift_L") {
        $shift_status = 0;
        if ($dial) {
            $dial->CanvasBind("<Motion>", "");
        }
        $dial_pos = 0;
        dial_wheelevent (0,0);
    }
    elsif ($keysym eq "Control_L") {
        $ctrl_status = 0;
        if ($dial) {
            $dial->CanvasBind("<Motion>", "");
        }
        $dial_pos = 0;
        dial_wheelevent (0,0);
    }
    elsif ($keysym eq "Alt_L") {
        $snap_mark = 0;
    } 
    elsif (($keysym eq "s") ||  ($keysym eq "S")){
        $single_step_mode = 0;
    }
}


sub Key_pressed {
    
    my $state = $nb->raised();
    #print "The Recording Tab is in state $state\n";
    if ($nb->raised() ne  "Recording_page") {
        return
    }
    my $keysym = $Tk::event->K;
    #print 'Keysym='.  $keysym . ', numeric=', $Tk::event->N, "\n";
    if ($keysym eq "F2") {
        show_pic_at_diff(-60);
    }
    elsif ($keysym eq "F3") {
        show_pic_at_diff(60);
    }
    elsif ($keysym eq "KP_Home") {
        jumpmark(-1);
    }
    elsif ($keysym eq "KP_Prior") {
        jumpmark(1);
    }
    elsif ($keysym eq "KP_Left") {
        movemark(-1);
    }
    elsif ($keysym eq "KP_Right") {
        movemark(1);
    }
    elsif ($keysym eq "KP_Insert") {
        togglemark();
    }
    elsif ($keysym eq "KP_Down") {
        movie_cut($split);
    }
    elsif ($keysym eq "Left") {
        if ($ctrl_status) {
            dial_wheelevent("bla", -1);
            return;
        }
        my $diff = -1 * $config{wheel_step};
        if ($single_step_mode) {
            $diff = -0.5;
        }
        if ($shift_status) {
            $diff *= 10;
        } 
        $nav_queue += $diff;
        if (! $nav_busy) {
            $nav_busy = 1;
            $mw->after(100, \&process_nav_queue);
        }
        #print "nav busy is 1, returning\n";
        return;
    }
    elsif ($keysym eq "Right") {
        if ($ctrl_status) {
            dial_wheelevent("bla", 1);
            return;
        }
        my $diff = $config{wheel_step};
        if ($single_step_mode) {
            $diff = 0.5;
        }
        if ($shift_status) {
            $diff *= 10;
        } 
        $nav_queue += $diff;
        if (! $nav_busy) {
            $nav_busy = 1;
            $mw->after(100, \&process_nav_queue);
            #print "Setting nav busy to 1 and returning\n";
        }
        #print "nav busy is 1, returning\n";
        return;
    }
    elsif ($keysym eq "Shift_L") {
        $shift_status = 1;
    }
    elsif ($keysym eq "Control_L") {
        $ctrl_status = 1;
    }
    elsif ($keysym eq "s" || $keysym eq "S") {
        $single_step_mode = 1;
    }
    elsif ($keysym eq "Alt_L" || $keysym eq "Meta_L") {
        snap_to_mark();
    }
    elsif ($keysym eq "Up" || $keysym eq "Down" ) {
        if ($dial) {
            $dial->CanvasBind("<Motion>", "");
        }
        $dial_pos = 0;
        dial_wheelevent (0,0);
    } elsif ($keysym eq "1") {
        $config{wheel_step} = 0.5;
        update_step_size();
    } elsif ($keysym eq "2") {
        $config{wheel_step} = 1;
        update_step_size();
    } elsif ($keysym eq "3") {
        $config{wheel_step} = 2;
        update_step_size();
    } elsif ($keysym eq "4") {
        $config{wheel_step} = 5;
        update_step_size();
    } elsif ($keysym eq "5") {
        $config{wheel_step} = 10;
        update_step_size();
    } elsif ($keysym eq "6") {
        $config{wheel_step} = 20;
        update_step_size();
    } elsif ($keysym eq "7") {
        $config{wheel_step} = 50;
        update_step_size();
    } elsif ($keysym eq "8") {
        $config{wheel_step} = 100;
        update_step_size();
    } elsif ($keysym eq "9") {
        $config{wheel_step} = 500;
        update_step_size();
    }
    
}


sub snap_to_mark {
   my @marks;
    if ($chapter_mode) {
        @marks = @{$recording->{chapter_marks}};
    } else {
        @marks = @{$recording->{marks}};
    }
    if (scalar(@marks) == 0) {
        return;
    }
    my $min_dist = $recording->{total_frames} + 1;
    my $target = "";
    foreach (@marks) {
        if ((abs ($frameno - $recording->mark2frames($_))) < $min_dist) {
            $min_dist = (abs ($frameno - $recording->mark2frames($_)));
            $target = $_;
        }
    }
    my $newpos = $recording->mark2frames($target);
    show_pic_at($newpos * 0.04);
    $snap_mark = $newpos;
    return;
}
sub download_marks {
    my $home_dir = (getpwuid($>))[7];
    if ((! -s "/etc/marks2pts.conf") && (! -s "$home_dir/.marks2pts.conf")) {
        if (create_marks2pts_conf() == -1) {
            return;
        };
    }
    print "Downloading marks\n";
    my $result = system "pts2marks \"$indir\"";
    if (! $result) {
        print "Download worked...?\n";
        $recording->read_marks_file();
        update_prog();
        foreach(@{$recording->{marks}}) {
            print "now mark is $_\n";
        }
        print "finished showing marks\n";
    }
}
sub upload_marks {
    my $home_dir = (getpwuid($>))[7];
    if ((! -s "/etc/marks2pts.conf") && (! -s "$home_dir/.marks2pts.conf")) {
        if (create_marks2pts_conf() == -1) {
            return;
        };
    }
    print "uploading marks\n";
    my $result = system "marks2pts -upload \"$indir\" ";
    if ($result) {
        print "Upload worked...?\n";
    }
}
sub channel_selection_dialog {
    my $db = $rmw->DialogBox(-title => 'Please select a Channel', -buttons => ['Ok', 'Cancel'], 
                -default_button => 'Ok');
    # Create the vertical Scrollbar
    my $scrollbar = $db->Scrollbar( );
    
    my $lb1 = $db->Listbox( -selectmode => "single", -width => "0", -yscrollcommand => ['set' => $scrollbar]);
}

sub create_marks2pts_conf {
    print "Creating config with default values...\n";
    my $home_dir = (getpwuid($>))[7];
    if (! -d $home_dir) {
        error_box("Could not identify the your home directory.\nAborting");
        return -1;
    }
    open CFH, "$home_dir/.marks2pts.conf" or do {
            error_box("Could not open $home_dir/.marks2pts.conf for writing\nAborting");
            return -1;
    };

my $cfg = qq(
after_cutting=Yes
channel=/video/channels.conf
debug=Yes
debug_log=$home_dir/sharemarks_debug.log
download_noad=Yes
epg=/video/epg.data
fake_source=S19.2E
gop_drift=0
passwd=
svdrp=/usr/bin/svdrpsend.pl
timer=/video/timers.conf
use_svdrp=No
user_name=
video=/video
);
    print CFH $cfg;
    close CFH;
    return;

}

sub chapter_mode_click {

    my @marks = @{$recording->{marks}};
    my @chapter_marks = @{$recording->{chapter_marks}};
    
    my @new_marks = verify_chapter_marks();
    
    
    if (scalar (@new_marks) !=  scalar (@chapter_marks)) {
        my $a = yes_no_box("Do you want to remove all chapter marks outside the selected areas?");
        if ($a eq "Yes") {
            @{$recording->{chapter_marks}} = @new_marks;
        }
    }
    if ($chapter_mode == 1) {
        $prog_can->createRectangle(0, 0, $config{preview_width}, 30, -fill => 'yellow');
    } else {
        $prog_can->createRectangle(0, 0, $config{preview_width}, 30, -fill => 'white');
    }
    update_prog();
}

sub verify_chapter_marks {
    my @marks = @{$recording->{marks}};
    my @chapter_marks = @{$recording->{chapter_marks}};
    my @new_marks;
    
    for (my $i = 0; $i < scalar(@chapter_marks); $i++)  {
        my $cf = $recording->mark2frames($chapter_marks[$i]);
        #print "checking mark $chapter_marks[$i] at frame $cf \n";
        for (my $j = 0; $j < scalar(@marks); $j +=2) {
            my $ok = 0;
            my $f = $recording->mark2frames($marks[$j]);
            #print "comparing with mark $marks[$j] at frame $f \n";
            if ($f <= $cf) {
                #print "$f < $cf \n";
                my $ef = $recording->mark2frames($marks[$j+1]);
                #print "Ok now $ef should be bigger than $cf\n";
                if ($ef >= $cf) {
                    #print "$ef > $cf \n";
                    push @new_marks, $chapter_marks[$i];
                    #print "Mark $cf verified...\n";
                } 
            } else {
                #print "Mark $cf will be removed\n";
                last;
            }
        }
    }
    return @new_marks;
}
sub generate_chapters {
    my @marks = @{$recording->{marks}};
    my @auto_chapters;
    my @frame_marks;
    my $offset;
    my $end = 0;
    my $current_start;
    my $current_end;
    
    if (scalar(@{$recording->{chapter_marks}})) {
        if (yes_no_box("Do you want to replace the chapter marks?") eq "Yes") {
            @{$recording->{chapter_marks}} = ();
        }
    }
    
    for (my $i = 0; $i < scalar(@marks); $i+=2)  {
        my $startf = $recording->mark2frames($marks[$i]);
        my $endf;
        if ($marks[$i+1]) {
            $endf = $recording->mark2frames($marks[$i+1]);
        } else {
            $endf = $recording->{total_frames};
        }
        push @frame_marks, "$startf TO $endf";
        #print "$startf TO $endf\n"
    }
    
    my $counter = 0;
    ($current_start, $current_end) = split / TO /, $frame_marks[$counter];
    $offset = $current_start;
    #print "starting at $offset to $current_end\n";
    while (! $end) {
        if (($offset + $config{dvd_chapter_interval} * 25) <= $current_end) {
            $offset += $config{dvd_chapter_interval} * 25;
            push @auto_chapters, $offset;
            #print "setting $offset\n";
            next;
        } else {
            if (!$frame_marks[$counter+1]) {
                $end = 1;
                next;
            }
            my $rest = $current_end - $offset;
            $counter++;
            ($current_start, $current_end) = split / TO /, $frame_marks[$counter];
            $offset = $current_start - $rest;
        }
    }    
    
    foreach(@auto_chapters) {
        my ($real_frame, undef) = $recording->find_GOP_offset($_);
        push @{$recording->{chapter_marks}}, $recording->frames2mark($real_frame);
        #print "pushed $real_frame as ${$recording->{chapter_marks}}[-1]\n"
    }
    @{$recording->{chapter_marks}} = sort(@{$recording->{chapter_marks}});
    update_prog();
}

sub yes_no_box {
    my $question = shift || "No Question...strange"; 
    my $answer = $mw->Dialog(-title => 'Please Reply', 
       -text => $question, 
       -default_button => 'Yes', -buttons => [ 'Yes', 'No'], 
       -bitmap => 'question' )->Show( );
    return $answer;
}

sub error_box {
    my $question = shift || "No Error...strange"; 
    my $answer = $mw->Dialog(-title => 'Something nasty happend', 
       -text => $question, 
       -default_button => 'ok', -buttons => [ 'ok'], 
       -bitmap => 'error' )->Show( );
    
}

sub get_dir_from_user {

    my $user_dir = shift || "/tmp"; 
    my $db = $mw->DialogBox(-title => 'Please enter directory', 
        -default_button => 'Ok', 
        -buttons => [ 'Ok', 'Cancel'], 
    );
    
   
    $db->add('LabEntry', -textvariable => \$user_dir, -width => 20, 
          -label => 'Please enter directory',
          -width => "64",
          -labelPack => [-side => 'top'])->pack;
    
    my $answer  = $db->Show( );
    
    if ($answer eq "Ok") {
        #print "returning $user_dir\n";
        return $user_dir;
    } else {
        #print "returning cancel\n";
        return "cancel";
    }
}


sub preview_movie {
    my $cut_flag = shift;
    if ($mplayer_pid) {
        kill ("TERM", $mplayer_pid);
        $mplayer_pid = 0;
    }
    
    if (! $indir) {
        return
    }
    
    if ($mplayer_pid = fork) { 
        return;    
    } 
    else {
        defined($mplayer_pid)         or die "fork: $!";
        if ($cut_flag) {
            print "trying to run vdrplayer.pl $indir $frameno 1\n";
            exec "vdrplayer.pl \"$indir\" $frameno 1";
        } else {
            print "trying to run vdrplayer.pl $indir $frameno 0\n";
            exec "vdrplayer.pl \"$indir\" $frameno 0";
        }
        
    }
    
    
}

sub process_nav_queue {
  
    #print "Processing nav_queue\n";
    if (! $nav_queue) {
        $nav_busy = 0;
        return;
    }
    my $diff_sum = $nav_queue;
    $nav_queue = 0;
    $nav_busy = 0;
    if ($snap_mark) {
        #print "trying to move mark by $diff_sum\n"x10;
        movemark($diff_sum * 2);
        return;
    }
    show_pic_at_diff($diff_sum);
}

sub dial_canvas_Button_Press {
    #print "Left Button Down on dial Canvas\n";
    $dial->CanvasBind("<Motion>", [ \&dial_canvas_Motion]);
    $dial->coords($dial_line, 60, 60, 60, 0);
    $dial->itemconfigure($dial_line, -fill=> "black")
}
sub dial_canvas_Button_Release {
    #print "Left Button released on dial Canvas\n";
    $dial->CanvasBind("<Motion>", "");
    $dial_pos = 0;
    dial_wheelevent (0,0);
}
sub dial_canvas_Motion {
    my $canv = shift;
    my $xpos =  $Tk::event->x;
    #print "something in motion on $canv\n";
    #print "\$Tk::event->x $xpos\n";
    if ($xpos > 120) {$xpos = 120};
    if ($xpos < 0) {$xpos = 0};
    $xpos = int($xpos/6) - 10;
    $dial_pos= $xpos;
    my $new_pos = ($xpos +10) / 20;
    my $x = 60 - 60 * (cos( $new_pos * 3.14 ));
    my $y = 60 - 60 * (sin( $new_pos * 3.14 ));
    #print "new coords are: $x and $y\n";
    
    #print "dial pos is now $dial_pos\n";
    $dial->coords($dial_line, 60, 60, $x, $y);
    my $color = "red";
    if ($dial_pos == 0) {
        $color = "black";
        $dial_status = 0;
    }
    if (($dial_status == 0)  && (abs($dial_pos) == 1)) {
        #print "Now the callback should kick in from motion...\n";
        $dial_status = 1;
        $mw->after(100, \&dial_callback);
    }
    $dial->itemconfigure($dial_line, -fill=> $color)
    
}

sub dial_canvas_right {
    #print "Got a right click to the canvas\n";
    $dial_pos = 0;
    dial_wheelevent (0,0);
}
sub wheelevent {
    #print "Wheel args = @_, $config{wheel_step}\n";
    #if (! $config{wheel_step}) {
    #    $config{wheel_step} = 0.5;
    #}
    my $dummy = shift;
    my $delta = shift;
    #$delta *= $config{wheel_step};
    if ($ctrl_status) {
        dial_wheelevent($dummy,  ($delta/(abs($delta)))); # Here we always want to pass 1 or -1, no matter what stesze has been chosen for the wheel 
        return;
    }
    
    #print "nav_busy is $nav_busy at start, keystatus is $shift_status\n";
    if ($shift_status) {
        $delta *= 10;
    }
    
    if ($single_step_mode) {
        if ($delta > 0) {
            $delta = 0.5;
        } else {
            $delta = -0.5;
        }
    }
       
    $nav_queue += $delta;
    if (! $nav_busy) {
        $nav_busy = 1;
        $mw->after(100, \&process_nav_queue);
        #print "Setting nav busy to 1 and returning\n";
    }
    #print "nav busy is 1, returning\n";
    return;
   
   
}
sub dial_wheelevent {
    
    #print "in wheel event\n";
    my $dummy = shift;
    my $delta = shift;
    
    $dial_pos  += $delta;
        
    if ($dial_pos > 10) {
        $dial_pos = 10;
        #print "returning cause dialpos too big\n";
        return;
    }
    
    elsif ($dial_pos < -10) {
        $dial_pos = -10 ;
        #print "returning cause dialpos too small $dial_pos\n";
        return;
    }
    
    #print "dial_pos is $dial_pos\n";
    
    my $new_pos = ($dial_pos + 10) / 20;
    my $x = 60 - 60 * (cos( $new_pos * 3.14 ));
    my $y = 60 - 60 * (sin( $new_pos * 3.14 ));
    #print "new coords are: $x and $y\n";
    if ($dial_pos == 0) {
        $dial_status = 0;
    }
    if ($dial) {
        $dial->coords($dial_line, 60, 60, $x, $y);
        my $color = "red";
        if ($dial_status == 0) {
            $color = "black";
        }
        $dial->itemconfigure($dial_line, -fill=> $color)
    }
    if (($dial_status == 0) && (abs($dial_pos) == 1)) {
        #print "Now the callback should kick in from wheel event...\n";
        $dial_status = 1;
        $mw->after(100, \&dial_callback);
    }
        
    
}

sub dial_callback {
    if ($dial_pos == 0 ) {
        #print "stopping execution of callback\n";
        return;
    }
    my $step_size = $dial_hash{abs($dial_pos)};
    #print "Callback for dial: dial_pos is $dial_pos, step_size $step_size, status = $dial_status\n";
    
    my $factor = $dial_pos / abs($dial_pos);
    #print "jump " . ($factor * $step_size) . " secs\n";
    if ($factor * $step_size + ($frameno * 0.04) > $recording->{last_Iframe} * 0.04) {
        $dial_pos = 0;
        dial_wheelevent (0,0);
    }
    if ($factor * $step_size  + ($frameno * 0.04) < 0) {
        $dial_pos = 0;
        dial_wheelevent (0,0);
    }
    show_pic_at_diff($factor * $step_size);                
    $mw->after(10, \&dial_callback);
}




sub update_dvd_page {
    print "update_dvd_page called\n";
    if (! $indir) {
        return;
    }
    print "update_dvd_page started\n";
    #
    # First some basic vars are checked an set if necessary
    #
    
    if (! $dvd_output_dir) {
        $dvd_output_dir = $config{dvd_output_dir} || "/tmp";
    } 
    if (! $dvd_basename) {
        $dvd_basename = $config{dvd_basename} || "VDRsyncDVD";
    }
    if (! $dvd_root_label) {
        $dvd_root_label = $config{dvd_root_label} || "Start Movie";
    }
    if (! $dvd_root_menu_title) {
        $dvd_root_menu_title = $config{dvd_root_menu_title} || "Created by VDRsync";
    }
    if (! $dvd_movie_title) {
        $dvd_movie_title = $config{dvd_movie_title} || "Created by VDRsync";
    }
    
    #$dvd_burn_cmd = $progs{growisofs} ." -dvd-compat -Z $config{dvd_device} -dvd-video $dvd_output_dir/$dvd_basename";
    
    #~ if (! $dvd_output_dir) {
        #~ $dvd_output_dir = $config{dvd_output_dir} || "/tmp";
    #~ } 
 
    #~ if (! $dvd_basename) {
        #~ $dvd_basename = $config{dvd_basename} ||  "VDRsyncDVD";
    #~ }
   
    #~ if (-e "$indir/chapter_marks.vdr") {
        #~ open FH, "$indir/chapter_marks.vdr";
        #~ $chapter_strg = "";
        
        
        #~ while (<FH>) {
            #~ (my $rmark) = $_ =~ /(\d{1,2}:\d{1,2}:\d{1,2})\.\d{1,2}/;
            #~ #print "EXTRACTED MARK $rmark\n" x3;
            #~ $chapter_strg .= $rmark .","
        #~ }
        #~ chop $chapter_strg;
    #~ }
    
    if (-e "$indir/summary.vdr") {
        $dvd_movie_menu_text = "";
        open FH, "$indir/summary.vdr";
        while (<FH>) {
            $dvd_movie_menu_text .= $_;
        }
    } else {
        $dvd_movie_menu_text = "No Summary available!\n";
    }
    
    
    $dvd_menu_text->delete("1.0", "end") if $dvd_menu_text;
    $dvd_menu_text->insert("end", $dvd_movie_menu_text) if $dvd_menu_text;#print "read_one_line\n";
        
    if ( %streams && ($dvd_indir ne $indir)) {
        #print "$indir changed, update necessary\n";
        $dvd_video_opt = $streams{video}{Aspect_ratio};
        $dvd_indir = $indir;
        #$dvd_vdrsync_cmd = "./vdrsync_next.pl $dvd_indir -o $config{dvd_output_dir} -master-dvd";
        #if ($config{dvd_basename}) {
        #    $dvd_vdrsync_cmd .= " -basename $config{dvd_basename} ";
        #}
        $dvd_movie_menu_text = "No Summary Text Found";
        if (defined $dvd_stream_frame) {
            #print "Trying to delete $dvd_stream_frame\n";
            $dvd_stream_frame->destroy;
        }
        $dvd_stream_frame = $dvd_page->Frame(-relief => "groove", -bd => 4)->grid(-column => "1", -row => "0", -sticky => "nw", -rowspan => "2");
        $dvd_stream_frame->Label(-text => "Stream Information")->grid(-columnspan => 2);
        foreach (reverse sort(keys(%streams))) {
            
            $dvd_stream_frame->Label(-text => "stream $_", -anchor => "e")->grid(
            $dvd_stream_frame->Label(-text => $streams{$_}{Audio_type}), -sticky => "e"); 
            
            $dvd_stream_frame->Label(-text => "Bitrate", -anchor => "e")->grid(
            $dvd_stream_frame->Label(-text => $streams{$_}{Bitrate}), -sticky => "e"); 
            
            $dvd_stream_frame->Label(-text => "Real Bitrate", -anchor => "e")->grid(
            $dvd_stream_frame->Label(-text => $streams{$_}{Real_Bitrate}), -sticky => "e") if ($_ eq "video"); 
            
            $dvd_stream_frame->Label(-text => "Mode", -justify => "right")->grid(
            $dvd_stream_frame->Label(-text => $streams{$_}{Mode}), -sticky => "e") if ($streams{$_}{Audio_type}); 
            
            $dvd_stream_frame->Label(-text => "Aspect ratio", -justify => "right")->grid(
            $dvd_stream_frame->Label(-text => $streams{$_}{Aspect_ratio}), -sticky => "e") if ($streams{$_}{Aspect_ratio}); 
            
            $dvd_stream_frame->Label(-text => "Resolution", -justify => "right")->grid(
            $dvd_stream_frame->Label(-text => "$streams{$_}{Horizontal_size}x$streams{$_}{Vertical_size}"), -sticky => "e") if ($streams{$_}{Aspect_ratio}); 
                        
            $dvd_stream_frame->Label(-text => "Total Bytes", -justify => "right")->grid(
            
            $dvd_stream_frame->Label(-text => (int($streams{$_}{Total_bytes} / 1048576) . " MB")), -sticky => "e") ; #/ 
            
            if ($_ ne "video") {
                $dvd_stream_frame->Label(-text => "Include on DVD?", -justify => "right")->grid(
                $dvd_stream_frame->Checkbutton( 
                    -variable => \$streams{$_}{include},
                    #-command   => [ \&build_dvd_vdrsync_cmd ]                    
                    )) ;  
                $streams{$_}{include} = 1;
            }
            
            $dvd_stream_frame->Label(-text => "")->grid( -columnspan => "2");
        }
    } 
    
    build_dvd_vdrsync_cmd();
}

sub start_dvd_creation_run {
    
    
    if ($marks_changed) {
        if (yes_no_box("You have changed your cut marks.\nDo you want to save them?") eq "Yes") {
            $chapter_mode = 0;
            write_marks_file();
            print "Should save now....\n";
        }
    }
    if ($chapter_marks_changed) {
        if (yes_no_box("You have changed your chapter marks.\nDo you want to save them?") eq "Yes") {
            $chapter_mode = 1;
            write_marks_file();
        }
    }
    
    
    #~ if (-e "$indir/summary.vdr") {
        #~ my $text;
        #~ open FH, "$indir/summary.vdr";
        #~ while (<FH>) {
            #~ $text .= $_;
        #~ }
        #~ my $boxt = $dvd_menu_text->get("1.0", "end");
        #~ print "Comparing text\n";# $boxt\n to: $text\n";
        #~ if ($boxt ne $text) {
            #~ if ((yes_no_box("You need to save the summary.vdr file to make your changes effective.\nDo you want to save them now?\n")) eq "Yes") {
                #~ save_dvd_settings();
            #~ }
        #~ }
    #~ }
    #
    # Should we produce a DVD with a single Recording, the recording will be added automatically to the titleset, no need to do that by hand
    #
    if (! @titlesets && $indir) {
        add_titleset();
    }
    #
    # If the current recording has not been added to an existing list of titlesets, ask whether it should be added
    #
    if (@titlesets && ($indir ne $titlesets[-1]{rec_dir})) {
        if (yes_no_box("The current recording has not been added to the DVD. Do you want to add it now?") eq "Yes") {
            add_titleset();
        }
    }
    
    
    my $dir = $dvd_output_dir;
    

    my @dir_parts;
       
    @dir_parts = split /\//, $dir;
       
    if (! $dir_parts[0]) {
        shift @dir_parts;
    }
        
    my $help_dir  = "/";
        
    my $partsno = scalar(@dir_parts);
    if ($partsno > 1) {
        $help_dir  .= (join "/", @dir_parts[0..($partsno-2)]);
        #print "help dir is now $help_dir\n";
    } 
    if (! -d $help_dir) {
        die "Directory to complicated, refusing to create\n";
    } 
    #print "trying to check $help_dir/$dir_parts[-1]\n";

    if (! -d "$help_dir/$dir_parts[-1]") {
        #print "$dir does not yet exist\n";
        #print "trying to create $help_dir/$dir_parts[-1]\n";
        my $result = mkdir "$help_dir/$dir_parts[-1]";
        #print "Creating the dir returned $result\n";
    }
    
    if (! -d $dir) {
        error_box("I could not create $dir, too complicated...\n");
        die "I screwd it, $dir does not exist\n";
    }
    
    
    #if (! $dvd_vdrsync_cmd) {
    build_dvd_vdrsync_cmd;
    #}
    
    print "\nI will run:\n$dvd_vdrsync_cmd\n";
    
    my $answer = yes_no_box("Should I really run\n$dvd_vdrsync_cmd?\n");
    
    if ( $answer eq "Yes") {
        
        my $pid;           #         FORK Does not work
        if ($pid = fork) {
            #$rmw->Busy();
            #my $start = time();
            #my $end=time();
        } else {
            if ($config{queue_manager}) {
                
                if (! $progs{"queue_manager.pl"}) {
                    error_box("You choose to use queue_manager.pl, but it is not in your PATH!\nPlease deselect the queue_manager.pl option on the Configuration Page.\n");
                    return;
                }
                my $lfile = time() . "logfile";
                #exec $progs{"queue_manager.pl"} . " $dvd_vdrsync_cmd > $tmp_dir/$lfile  $tmp_dir/$lfile ";
                exec $progs{"queue_manager.pl"} .  " $dvd_vdrsync_cmd > $tmp_dir/$lfile  $tmp_dir/$lfile ";
            } else {
             exec $dvd_vdrsync_cmd;
            }
            #system  $dvd_vdrsync_cmd;
            #open CFH, ">>/home/peter/.vdrsync/jobs.txt";
            #print CFH "$dvd_vdrsync_cmd\n";
            #close CFH;
        } 
    }
    $percent_done = 0;
}


sub save_dvd_settings {
    #print "Trying to save some settings\n";
    open FH, ">$indir/summary.vdr" or die "Could not open summary.vdr to save the current text: $!\n";
    print FH $dvd_menu_text->get('1.0', 'end');
    close FH;
        
}
sub get_stream_info {
    my $cmd = $progs{"vdrsync.pl"} . " '$indir' -i -script-output -cut ";
    
    my @results = `$cmd`;
    my $rline = join "",@results;
    my @list = grep { $_ =~ /^[bce][0-9a-f]_/ } @results;
    foreach (@list) {
        my ($id, $param, $value) = ($_ =~ /([ecb][0-9a-f])_(.+)=(.+)/);
        #print "id: $id, parm: $param, value $value\n";
        if ($id =~ /^e/) {
            $id = "video";
        }
        $streams{$id}{$param} = $value;
    }
    ($selected_rec{s_video}) = ($rline =~ /total_video=(\S+)/);
    ($selected_rec{s_audio}) = ($rline =~ /total_audio=(\S+)/);
    ($selected_rec{len_cut}) = ($rline =~ /movie_length_cut : (\S+)/);
    ($selected_rec{len}    ) = ($rline =~ /movie_length : (\S+)/);
    ($selected_rec{size_cut})= ($rline =~ /movie_size_cut : (\S+)/);
    ($selected_rec{size}   ) = ($rline =~ /movie_size : (\S+)/);
    
    print qq" $selected_rec{s_video} 
        $selected_rec{s_audio} 
        $selected_rec{len_cut}
        $selected_rec{s_video}
        $selected_rec{s_audio}
        $selected_rec{len_cut}
        $selected_rec{len}
        $selected_rec{size_cut}
        $selected_rec{size}   
        ";
    foreach(keys(%streams)) {
        #print "found stream $_\n";
    }
}

sub start_vdrsync_run {
    #print "Starting Conversion...\n";
    my $cmd = "./vdrsync_next.pl $indir";
    if ($vs_cut_recording) {
        $cmd .= " -cut-recording ";
    }
    if ($vs_video_filter) {
        $cmd .= " -v-filter\" $vs_video_filter \" ";
    }
    if ($vs_audio_filter) {
        $cmd .= " -a-filter\" $vs_audio_filter \" ";
    }
    if ($vs_ac3_filter) {
        $cmd .= " -ac3-filter\" $vs_ac3_filter \" ";
    }
    if ($vs_basename) {
        $cmd .= " -basename \"$vs_basename\" ";
    }
    if ($vs_out_dir) {
        $cmd .= " -o \"$vs_out_dir\" ";
    }
    if ($vs_mkiso) {
        $cmd .= " -mkiso ";
    }
    if ($vs_master_dvd) {
        $cmd .= " -master-dvd ";
    }
    #print "running $cmd\n";
    my $vdrsync_pid = 0;
    if ($vdrsync_pid = fork) { 
        #print "Here is the parent...\n";
        #sleep 2;
        #open (FH, "tail -f -n 25 /tmp/vdrsync.log|") || die "Could not open file!\n";
       # while (! eof(FH)) {
       #     my $dummy = <FH>;
        #    if ($dummy =~/exiting/) {
         #       close FH;
          #      return
           # }
       #     print "read $dummy";
        #}
        #return;
    } 
    else {
        defined($vdrsync_pid)     or die "fork: $!";
        #open (FH, "$cmd |") || die "Could not open file!\n";
        #sleep 1;
        #$cmd = "echo hallohallo\n";
        print "executing $cmd  \n";
        #$cmd .=  " &> /tmp/vdrsync.log";
        exec $cmd;
        #my $dummy;
        #while ($dummy = <FH>) {
        #    my $dummy = <FH>;
        #    print "Got $dummy";
        #}
        #print "finished $cmd\n";
        #close FH;
    }
}


sub search_PRGs {
    my @path;
    if ( ! $config{path_str}) {
        @path = split(":",$ENV{PATH});
    } else {
        @path = split(":", $config{path_str});
    }
    
    my %missingh;
    my $miss = 0;
    
    foreach my $program(keys(%progs)) {
        foreach (@path) {
            if (-e "$_/$program") { 
                $progs{$program} = "$_/$program";
                last;
            }    
        }
        if (! $progs{$program}) {
            $missingh{$program} = 1;
            $miss = 1;
        }
    }
    if ((scalar(keys(%missingh))==1) && ((join" ",keys(%missingh)) =~ /requant/)) {
        print "there is only one missing, and that is a requant thing\n";
        $miss = 0;
    }
    if ($miss) {
        error_box ("The following recommended helper programs were not found:\n" . (join"\n",keys(%missingh)) .  "\n");
    }
}

sub init {
    print "Initializing...\n";
    
    if ($first_init) {
        search_PRGs();
        read_config();
        $first_init = 0;
    }
    
    if (! $indir) {
        $indir = rec_selection_dialog();
    }
    if ((! $indir) || (! -d $indir)) {
        print "No indir, stopping to init\n";
        return;
    } 
    remove_tmp_dir();
    create_tmp_dir();
    $rmw->Busy;
    %jpegs = ();
    print "got $indir as input directory\n";
    
    
    $recording = "";
    $recording = VDRRECORDING->new();
    $recording->init_recording($indir);
    #print "recording is $recording\n";    
    
    get_stream_info();
    
    ($current_gop, $frameno) = $recording->get_gop_at(0,1);
    my $first_pic =  decode_gop($current_gop, $frameno);
    if (! -s $first_pic) {
        error_box("Ooops, some problems to decode the very first picture. Either the capture program can not decode the file, or the capture program is not available\n");
    } else {
        $canvas_image    = $mw->Photo('-format'=> 'jpeg', -file => "$first_pic");
        $current_picture = $can->createImage($config{preview_width} / 2, $config{preview_height} / 2, -image => $canvas_image);
    }
    $total_length_label->configure(-text => ("Length: " . ($recording->frames2mark($recording->{total_frames}))) . " / $recording->{total_frames}");    
    update_mark_bars();
    update_prog();
    if (%selected_rec) {
        $dvd_root_label  = "$selected_rec{rec_name}";
        $dvd_movie_title = $selected_rec{rec_name};
        if ($selected_rec{rec_group} ne " ") {
            $dvd_root_menu_title = $selected_rec{rec_group};
        }
    }
    update_dvd_page();
    $rmw->Unbusy;
}



sub take_snapshot {
    if (! $jpegs{$frameno}) {
        print "Error: the current position is not available as a decoded picture\n";
        return;
    }
    my $frameno_formated = $jpegs{$frameno}; 
    print "checking $frameno_formated\n";
    if (! -s $frameno_formated) {
        print "Error: the requested jpeg file $frameno_formated does not exist\n";
        return;
    }
    my $db = $mw->DialogBox(-title => 'Please enter a name for the snapshot', -buttons => ['Ok', 'Cancel'], 
                     -default_button => 'Ok');
    my $file_name = "$indir/snapshot.jpg";
    $db->add('LabEntry', -textvariable => \$file_name, -width => 80, 
         -label => 'File Name', -labelPack => [-side => 'left'])->pack;
    my $answer = $db->Show();
   
    if ($answer eq "Ok") {
        my $cmd = "cp $frameno_formated \"$file_name\"";
        print "Trying to run $cmd\n";
        my $result = system $cmd;
        if ($result) {
            error_box($result);
        }
    }
}

sub show_pic_at_diff {
    
    my $diff = shift;
    #print "Showing picture at diff\n";
    
    $nav_busy = 1;
    my $now = $frameno * 0.04;
    if (($now + $diff) < 0) {
        $diff = $now * -1;
    }
    if (($recording->{total_frames} * 0.04) < $now + $diff) {
        $diff = $recording->{total_frames}* 0.04 - $now - 0.5;
    }
    #print "got Diff $diff to move\n";
    show_pic_at($now + $diff);
    $nav_busy = 0;
  
    #print "Finished picture at diff, nav_busy is $nav_busy\n";
}


sub show_pic_at {
    my $sec = shift;
    #print "Should show pic at $sec\n"; 
    if ($sec > ($recording->{last_Iframe} * 0.04)) {
        $sec = $recording->{last_Iframe} * 0.04;
    }
    if ($sec <= 0) {
        $sec = 0;
    }
    my( $frame_to_show, $dummy) = $recording->find_GOP_offset($sec * 25);
    
    #print "Should show pic at frame $frame_to_show\n"; 
    
    if ((! $jpegs{$frame_to_show}) || 
        (! -s $jpegs{$frame_to_show}) || 
        ($jpegs{$frame_to_show} eq "$tmp_dir/Decode_Failure.jpg")) {
        my $start = [gettimeofday];
        ($current_gop, $frame_to_show) = $recording->get_gop_at($sec,1);
        $get_gop_time += tv_interval($start);
        $start = [gettimeofday];
        decode_gop($current_gop, $frame_to_show);    
        
        $decode_gop_time += tv_interval($start);
    }
    
    my $frameno_formated = $jpegs{$frame_to_show};
    my $start = [gettimeofday];
    if ($canvas_image) {
        $canvas_image->delete;
    }
    $canvas_image = $mw->Photo('-format'=> 'jpeg', -file => $frameno_formated);
    $can->itemconfigure($current_picture, -image => $canvas_image);    
    $load_pic_time += tv_interval($start);
    
    $frameno = $frame_to_show;
    #print "frameno is now $frameno\n";
    update_prog();
    
    $pic_counter++;
#    print "Get GOP time: $get_gop_time\nDecode time: $decode_gop_time\nload time: $load_pic_time\npic count: $pic_counter";
    
    if ($pic_counter > 100) {
       # die ("FINAL Get GOP time: $get_gop_time\nDecode time: $decode_gop_time\n");
    }
    return;
}

sub decode_gop {
    my $gop_to_decode = shift;
    my $frame_to_decode = shift;
    open OF, ">$tmp_dir/current_gop.vdr" or die "could not open current_gop.vdr:$!\n"; ;                        
    print OF $gop_to_decode;                                
    close OF;
    $gop_to_decode = "";
    my $frameno_formated = sprintf "%06u",$frame_to_decode;
    
    #print "at decode_gop with capture_prog $config{capture_prog}\n";
    
    
    if ($config{capture_prog} =~ /gop2jpeg$/) {
        # print  "cat $tmp_dir/current_gop.vdr | " .  $progs{gop2jpeg} . " $tmp_dir/$frameno_formated" . "_%d.jpg   &> /dev/null"; 
        system "cat $tmp_dir/current_gop.vdr | " . $progs{gop2jpeg}  . "  $tmp_dir/$frameno_formated" . "_%d.jpg   &> /dev/null"; 
        
        $jpegs{$frame_to_decode} = "$tmp_dir/$frameno_formated" . "_1.jpg";
        if (! -s $jpegs{$frame_to_decode}) {
            generate_failure_jpg();
            $jpegs{$frame_to_decode} = "$tmp_dir/Decode_Failure.jpg"
        }    
        return $jpegs{$frame_to_decode};
        
    } elsif ($config{capture_prog} =~ /ffmpeg$/) {
        # print  " cat $tmp_dir/current_gop.vdr | " . $progs{ffmpeg} . "-i - -t 0.04  $tmp_dir/$frameno_formated" . "_%d.jpg   &> /dev/null"; 
        system " cat $tmp_dir/current_gop.vdr | " . $progs{ffmpeg} . " -i - -t 0.04 -s $config{preview_width}x$config{preview_height} $tmp_dir/$frameno_formated" . "_%d.jpg   &> /dev/null"; 
        $jpegs{$frame_to_decode} = "$tmp_dir/$frameno_formated" . "_1.jpg";
        if (! -s $jpegs{$frame_to_decode}) {
            generate_failure_jpg();
            $jpegs{$frame_to_decode} = "$tmp_dir/Decode_Failure.jpg"
        }    
        return $jpegs{$frame_to_decode};
    } elsif ($config{capture_prog} =~ /transcode$/) {
        # print  $progs{transcode} . "-i $tmp_dir/current_gop.vdr  -c 2-3 -z -k -F jpg  -x mpeg2,null -o $tmp_dir/$frameno_formated" . "_  -y im,null";
        system $progs{transcode} . " -i $tmp_dir/current_gop.vdr  -c 2-3 -z -k -F jpg  -x mpeg2,null -o $tmp_dir/$frameno_formated" . "_  -y im,null";
        $jpegs{$frame_to_decode} = "$tmp_dir/$frameno_formated" . "_000000.jpg";
        if (! -s $jpegs{$frame_to_decode}) {
            generate_failure_jpg();
            $jpegs{$frame_to_decode} = "$tmp_dir/Decode_Failure.jpg"
        }    
        return $jpegs{$frame_to_decode};
    } else {
        error_box ("It seems as if no capture Program is defined. This is fatal. Check the configuration.\n");
    }
    #print "formated frameno is $frameno_formated\n";
    #system " cat current_gop.vdr | ffmpeg -i - -t 0.04  $frameno_formated" . "_00000%d.jpg   &> /dev/null"; #
        
    #system ($progs{transcode} ." -i current_gop.vdr  -c 2-3 -z -k -F jpg  -x mpeg2,null -o " . $frameno_formated . "_  -y im,null");
    
    ##############
    # dont use something below, the above decode procedures are working...
    #############
    
    #print "ffmpeg -i current_gop.vdr -t 0.04 -f mpeg $frameno_formated" . "_%d.jpg"; 
    #print "decoding...�n";
    #system " ffmpeg -i current_gop.vdr -t 0.04  $frameno_formated" . "_00000%d.jpg  &> /dev/null"; 
    #print "| ffmpeg -i - -t 0.04  $frameno_formated" . "_00000%d.jpg  ";
    #open TFH, " | ffmpeg  -i - -t 0.04  $frameno_formated" . "_00000%d.jpg  "; 
    #print TFH $gop_to_decode;
    #close TFH;
    return;
}



sub open_recording {
    
    $indir = rec_selection_dialog();
    if (-d $indir)  {
        $rmw->Busy();
        init();
        $rmw->Unbusy();
    }
    return;
}


sub remove_tmp_dir {
    if (! $tmp_dir) {
        print "no tmp_dir defined, just returning\n";
        return;
    }
    opendir DH, "$tmp_dir" or die "can not open input dir: $!\n";
    my @allfiles = grep { ! /^\./  } readdir DH;
    
    foreach (@allfiles) {
        print "deleting $tmp_dir/$_\n";
        unlink ("$tmp_dir/$_");
        
    }
    
    rmdir $tmp_dir;
    print "finished cleaning...\n";
}
sub clean_up {
    
    if ($marks_changed) {
        if (yes_no_box("You have changed your cut marks.\nDo you want to save them?") eq "Yes") {
            $chapter_mode = 0;
            write_marks_file();
            print "Should save now....\n";
        }
    }
    if ($chapter_marks_changed) {
        if (yes_no_box("You have changed your chapter marks.\nDo you want to save them?") eq "Yes") {
            $chapter_mode = 1;
            write_marks_file();
        }
    }
    
    if (! $tmp_dir) {
        print "no tmp_dir defined, just exiting\n";
        Tk::exit;
    }
    remove_tmp_dir();
    Tk::exit;
}

sub jump_xy {
  my ($canv, $x, $y) = @_;
  my $xpos = $canv->canvasx($x);
  my $frame_pos = int( $recording->{total_frames} * $xpos  / $config{preview_width});
  show_pic_at($frame_pos * 0.04);                                        
  #show_pos_xy($canv, $x, $y);                                        
}



sub write_marks_file {
    my @marks;
    my $outfile = $indir;
    if ($chapter_mode) {
        @marks = @{$recording->{chapter_marks}};
        $outfile .=  "/chapter_marks.vdr";
        $chapter_marks_changed = 0;
        
    } else {
        @marks = @{$recording->{marks}};
        $outfile .=  "/marks.vdr";
        $marks_changed = 0;
    }
    #my @marks = @{$recording->{marks}};
    if (! @marks) {
        if (-f $outfile) {
            unlink $outfile;
            return;
        }
    }
    print "Trying to write the marks file\n";
    open OF, ">$outfile" || die "can not open the ouput file: $!\n";
    foreach (@marks) {
        print OF $_ . "\n";
    }
    close OF;
}

sub movemark {
    my $direction =shift;
    my @marks;
    if ($chapter_mode) {
        @marks = @{$recording->{chapter_marks}};
    } else {
        @marks = @{$recording->{marks}};
    }
    #my @marks = @{$recording->{marks}};
    if (! @marks) {print "no marks to move\n"; return}                        
    my $newpos = 0;
    foreach(@marks) {
        if ($recording->mark2frames($_) != $frameno) {next}                            
        $newpos = $recording->mark2frames($_);                    
        #print "current: $frameno, new: $newpos\n";
        last;
    }
    if (! $newpos) {print "You can only move a selected mark\n"; return}
    delmark();                                        
    show_pic_at_diff($direction * 0.5);                        
    setmark();            
    
}

sub jumpmark_click {
    my $dummy = shift;
    my $direction = shift;
    jumpmark($direction);
}
sub jumpmark {
    my $direction =shift;
    my @marks;
    if ($chapter_mode) {
        @marks = @{$recording->{chapter_marks}};
    } else {
        @marks = @{$recording->{marks}};
    }
    if (! @marks) {print "no marks to jump to\n"; return}
    my $newpos = 0;
    #print "BEFORE JUMP FRAMENO IS $frameno, direction is $direction\n";
    if ($direction == 1) {
        if ($frameno >= $recording->mark2frames($marks[-1])) { print "There is no next mark\n"; return}                                  #!!!FIXME!!!
        foreach(@marks) {
            if ($recording->mark2frames($_) <= $frameno) {next}
            $newpos = $recording->mark2frames($_);
            #print "current: $frameno, new: $newpos\n";
            last;
        }
        
        show_pic_at($newpos * 0.04);
        #print "the skript shoud have moved to $newpos\n";
        return;
    }
    
    my @rev_marks = reverse(@marks);
    if ($direction == -1) {
        #foreach(@rev_marks) {
        #    print "checking $_ " . $recording->mark2frames($_) . "\n";                            
        #}
        
        if ($frameno <= $recording->mark2frames($marks[0])) { print "There is no prev mark\n"; return}                             #!!!FIXME!!!
        foreach(@rev_marks) {
            #print "Checking mark ($marks[0]) " .  $recording->mark2frames($_) ."\n";
            if ($recording->mark2frames($_) >= $frameno  )     {
                #print "Skipping" .$recording->mark2frames($_) ."\n"; 
                next;
            }
            else {
                #print "accepted " . $recording->mark2frames($_) . "\n";    
            }
            $newpos = $recording->mark2frames($_);
            #print "current: $frameno, new: $newpos\n";
            last;
        }
        show_pic_at($newpos * 0.04);
        #print "the skript should have moved to $newpos\n";
        return;
    }
}


sub togglemark {
    #print "Toggle mark at frame: $frameno\n";
    my @marks;
    if ($chapter_mode) {
        @marks = @{$recording->{chapter_marks}};
    } else {
        @marks = @{$recording->{marks}};
    }
    my $counter = 0;
    my $hit;
    foreach(@marks) {                
        if ($frameno == $recording->mark2frames($_)) {                            
            #print "should del mark $counter" . $recording->mark2frames($_) . "\n";
            $hit = $counter;
        last;
        }
        $counter++;
    }
    #if ($frameno == $recording->mark2frames($marks[$counter])) {
    if (defined($hit)) {
        delmark();
    }
    else {
        setmark();
    }
    
}

sub delmark {
    #print "Current frameno: $frameno\n";
    #my @marks = @{$recording->{marks}};
    
    my @marks;
    if ($chapter_mode) {
        @marks = @{$recording->{chapter_marks}};
    } else {
        @marks = @{$recording->{marks}};
    }
    
    my $counter = 0;
    foreach(@marks) {                
        if ($frameno == $recording->mark2frames($_)) {                            
            #print "should del mark $counter" . $recording->mark2frames($_) . "\n";
        last;
        }
    $counter++;
    }
    
    #print "Just to be sure: counter is $counter and delmark is $marks[$counter]\n";
    my $before = scalar(@marks);                            
    splice @marks, $counter, 1;                            
    if (scalar(@marks) == $before)    {                    
        print "Delete Mark did not work\n";                
        }
    else {
        #print "Delete Mark did work\n";                    
    }
    if ($chapter_mode) {
        @{$recording->{chapter_marks}} = (sort(@marks));
        $chapter_marks_changed = 1
    } else {
        @{$recording->{marks}} = (sort(@marks));
        $marks_changed = 1;
    }
    #@{$recording->{marks}} = @marks;
    update_mark_bars();
}



sub update_mark_bars {
    my @marks = @{$recording->{marks}};
    my @chapter_marks = @{$recording->{chapter_marks}};
    foreach (@mark_bars) {
        $prog_can->delete($_)
    }
    foreach (@mark_ticks) {
        $prog_can->delete($_)
    }
    foreach (@chapter_ticks) {
        $prog_can->delete($_)
    }
    $mark_no = scalar(@marks);
    $cut_length = 0;
    $percent_done = 0;

    
    #if (! @marks) {return}
    if (scalar(@marks)) {
        #print "found at least one mark\n";
        
    }

    foreach (@marks) {
        my $startframe = $recording->mark2frames($_);
        my $startpos = int($startframe * $config{preview_width} / $recording->{total_frames});
        my $chapter_mark_tick   = $prog_can->createRectangle($startpos - 1, 0, $startpos + 1, 20, -fill => 'blue');
        push @mark_ticks, $chapter_mark_tick;
    }
    
    
    foreach (@chapter_marks) {
        my $startframe = $recording->mark2frames($_);
        my $startpos = int($startframe * $config{preview_width} / $recording->{total_frames});
        my $mark_tick   = $prog_can->createRectangle($startpos - 1, 20, $startpos + 1, 30, -fill => 'black');
        push @chapter_ticks, $mark_tick;
    }
    
    if (scalar(@marks)==1) {
        my $startframe = $recording->mark2frames($marks[0]);
        my $endframe = $recording->{total_frames};
        #print "should draw from $startframe to $endframe\n";
        my $startpos = int($startframe * $config{preview_width} / $recording->{total_frames});
        my $endpos = int($endframe * $config{preview_width} / $recording->{total_frames});
        my $mark_rect = $prog_can->createRectangle($startpos,10, $endpos, 20, -fill => 'red');
        push @mark_bars, $mark_rect;
        my $frame_length  = $endframe - $startframe;
        $cut_length = "After cutting: " . $recording->frames2mark($frame_length) . " / $frame_length";
        return;
    }
    my $frame_length = 0;
    for (my $i = 0; $i < (scalar(@marks)); $i += 2) {
        my $startframe = $recording->mark2frames($marks[$i]);
        my $endframe;
        if ($marks[$i+1]) {
            $endframe = $recording->mark2frames($marks[$i+1]);
        }
        else {
            $endframe = $recording->{total_frames};
        }
        $frame_length += $endframe - $startframe;
        #print "should draw from $startframe to $endframe\n";
        my $startpos = int($startframe * $config{preview_width} / $recording->{total_frames});
        my $endpos = int($endframe * $config{preview_width} / $recording->{total_frames});
        my $mark_rect = $prog_can->createRectangle($startpos,10, $endpos, 20, -fill => 'red');
        push @mark_bars, $mark_rect;
    }
    $cut_length = "After cutting: " .  $recording->frames2mark($frame_length) . " / $frame_length";
}

sub update_prog {
    $prog_can->delete($prog_rect);
    my $newlength = int($config{preview_width} * $frameno / $recording->{total_frames});
    #print "Now ProgressBar ends at $newlength\n";
    $prog_rect = $prog_can->createRectangle(0, 5, $newlength, 25, -fill => 'green');
    $current_pos = "Current Pos: " . $recording->frames2mark($frameno) . " / $frameno";
    update_mark_bars();
}

sub setmark {
    my @marks;
    if ($chapter_mode) {
        @marks = @{$recording->{chapter_marks}};
    } else {
        @marks = @{$recording->{marks}};
    }
    #print "set mark clicked for frame $frameno\n";
    my $mark = $recording->frames2mark($frameno);
    my $mark_exists = 0;
    foreach (@marks) {
        if ($_ eq $mark) {
            #print "mark exists\n"; 
            $mark_exists = 1; return;
        }
    }
    push @marks, $mark;
    if ($chapter_mode) {
        @{$recording->{chapter_marks}} = (sort(@marks));
        $chapter_marks_changed = 1;
    } else {
        @{$recording->{marks}} = (sort(@marks));
        $marks_changed = 1;
    }
    #@{$recording->{marks}} = (sort(@marks));
    #print "@{$recording->{marks}}\n";
    #print "got $mark from sub";
    update_prog();
    update_mark_bars();
    
        
}



sub create_tmp_dir {
    print "Trying to create Temp Dir\n";
    if (! -d $config{master_tmp_dir}) {
        error_box("WARNING, $config{master_tmp_dir} not available\nPlease define a valid tmp dir!\n");
        return "./";
    }
    my $dir = time(). $$ . "/";
    mkdir("/$config{master_tmp_dir}/$dir") ;
    if (! -d "/$config{master_tmp_dir}/$dir"){
        error_box("WARNING, I could not create the necessary tmp dir /$config{master_tmp_dir}/$dir\n");
        return;
    }
    print "Temp Dir is now /$config{master_tmp_dir}/$dir\n";
    $tmp_dir = "/$config{master_tmp_dir}/$dir";
    #return "/$config{master_tmp_dir}/$dir";
}


sub compute_chapter_times {
    if (scalar @{$recording->{chapter_marks}} == 0) {
        return;
    }
    if (scalar @{$recording->{marks}} < 2) {
        return @{$recording->{chapter_marks}};
    } 
    my @ver_marks = verify_chapter_marks();
    
    
}

sub create_chapter_images {
    my @marks = @{$recording->{chapter_marks}};
    if (! @marks) {
        return;
    }
    $percent_done =0;
    my $step_size = int (100/ scalar(@marks));
    for (my $i = 0; $i < scalar(@marks); $i++) {
        my $frames = $recording->mark2frames($marks[$i]);
        #print "Trying to get screenshot from frame $frames (mark $marks[$i])\n";
        my( $frame_to_show, $dummy) = $recording->find_GOP_offset($frames);
        my $gops_to_get = int ($config{dvd_chapter_menu_len} * 2.5);
        #if (! $jpegs{$frame_to_show}) {
            ($current_gop, $frame_to_show) = $recording->get_gop_at($frames * 0.04, $gops_to_get);
            #decode_gop($current_gop, $frame_to_show);    
        #}
        
        my $frameno_formated = sprintf "%06u",$frame_to_show;
        my $dir_name = sprintf("$tmp_dir" . "chapter%03u", ($i+1));
        mkdir ($dir_name);
        #print "cp ./$frameno_formated" . "_000001.jpg $dir_name/chap.jpg";
        #open FFFH, " |  ffmpeg -i - -t 0.08  /$dir_name/$frameno_formated" . "_00000%d.jpg  ";
        #print " |  ffmpeg -i - -t 0.08  /$dir_name/$frameno_formated" . "_00000%d.jpg ";
      
        #print FFFH $current_gop;
        #close FFFH;
        open OF, ">./current_gop.vdr";                        
        print OF $current_gop;                                
        close OF;
        system " cat current_gop.vdr | " . $progs{ffmpeg} . " -i - -t $config{dvd_chapter_menu_len}  /$dir_name/$frameno_formated" . "_00000%03d.jpg # &> /dev/null "; 
        $percent_done += $step_size;
        $rmw->update;
        #system "cp ./$frameno_formated" . "_000001.jpg $dir_name/chap.jpg";
    }
    $percent_done = 0;
}


sub movie_cut {
    
    my $dir = $indir;
    my @dir_parts = split /\//, $indir;

    if (! $dir_parts[0]) {
        shift @dir_parts;
    }
    my $partsno = scalar(@dir_parts);
    if ($partsno > 2) {
        $dir  = ("/" . (join "/", @dir_parts[0..($partsno-3)]));
        $dir .= "/%$dir_parts[-2]/$dir_parts[-1]/";
    } elsif  ($partsno == 2){
        $dir = "/%$dir_parts[-2]/$dir_parts[-1]/";
    } else {
        $dir = "/%$dir_parts[-1]/";
    }
    
    
    $dir = get_dir_from_user("$dir");
    
    if ($dir eq "cancel") {
        return
    }
    
    if (! -d $dir) {
        print "$dir does not yes exist\n";
        @dir_parts = split /\//, $dir;
        
        if (! $dir_parts[0]) {
            shift @dir_parts;
        }
        
        my $help_dir  = "/";
        
        $partsno = scalar(@dir_parts);
        if ($partsno > 2) {
            $help_dir  .= (join "/", @dir_parts[0..($partsno-3)]);
        } 
        if (! -d $help_dir) {
            die "Directory to complicated, refusing to create\n";
        } 
        print "trying to check $help_dir/$dir_parts[-2]\n";
        if (! -d "$help_dir/$dir_parts[-2]") {
            print "trying to create $help_dir/$dir_parts[-2]\n";
            mkdir "$help_dir/$dir_parts[-2]";
        }
        if (! -d "$help_dir/$dir_parts[-2]/$dir_parts[-1]") {
            mkdir "$help_dir/$dir_parts[-2]/$dir_parts[-1]";
        }
    }
    if (! -d $dir) {
        die "I screwd it\n";
    }
    
    $rmw->Busy();
    $recording->cut_movie("$dir", $split, $rmw, \$percent_done);
    $percent_done = 0;
    $rmw->update;
    $rmw->Unbusy();
}

sub generate_failure_jpg {
    if (-s "$tmp_dir/Decode_Failure.jpg") {
        return
    }
    my $jpg;
    my $uu_frame =  <<'End_JPEG';
M_]C_X``02D9)1@`!`0$`2`!(``#_X0`617AI9@``34T`*@````@```````#_
M_@`70W)E871E9"!W:71H(%1H92!'24U0_]L`0P`%`P0$!`,%!`0$!04%!@<,
M"`<'!P</"PL)#!$/$A(1#Q$1$Q8<%Q,4&A41$1@A&!H='1\?'Q,7(B0B'B0<
M'A\>_]L`0P$%!04'!@<."`@.'A01%!X>'AX>'AX>'AX>'AX>'AX>'AX>'AX>
M'AX>'AX>'AX>'AX>'AX>'AX>'AX>'AX>'AX>_\``$0@"0`+0`P$B``(1`0,1
M`?_$`!P``0`#`0$!`0$````````````%!@<$`P@"`?_$`%<0``$#!``$`@8%
M!P8+!0<$`P$``@,$!081!Q(A,1-!"!0B46%Q%3)U@;,C-3=TD:&R%C8X0E*"
M%R0S0V)R<Y*QP=$TDZ*TQ!@E5H.$AM-35Z/P97:4_\0`'`$!``,!`0$!`0``
M``````````(#!`4!!@<(_\0`/!$``00!`P($!`0%`@4%`0```0`"`Q$$$B$Q
M!4$346%Q(C*!L121H<$&-$)RT17P(R128N$6)C.2\8+_V@`,`P$``A$#$0`_
M`*"B(OBE_3J(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B+AO\`^8KA^JR_P%=RX;_^8KA^JR_P%61?./=9<W^6D_M/V7<B(JUJ1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1<-__,5P_59?
MX"NY<-__`#%</U67^`JR+YQ[K+F_RTG]I^R[D1%6M2(B(B(B(B(B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B+AO_YBN'ZK+_`5W+AO_P"8KA^J
MR_P%61?./=9<W^6D_M/V7<B(JUJ1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1<-__`#%</U67^`KN12:[2X%53Q^+&Z.ZL$?FB(BB
MK41$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1%<\%X993F=HENEDBI7TT4YIW&6<,/.&M<>GNTX*
M9J^!?$6"(O9;*6HT-\L57'O_`,1"L&!.<ST6<O<QQ:X7/H0='M3+(K?>[S;I
MVST%VKJ65IV'PU#F$?L*VN9#&UNH$V+Y_P#"^:AR.I9<TXA>QH8XM`+2;V!W
M(<//R7YO=HNEDKW4%WH*BAJ6]3'-&6G7O&^X^(Z+B6]XQ<G\8>&]WLE_9'-D
M5FA]8H*T,`?(-'0.O>6\KO(\S3K8VLBP+%+IF>1166U-:)'`OEE?]2&,=WN^
M'4#XD@*N2"BWP]P[C_"V8?5=3)1E@,?%\V^U58<#Y$?7LH%%KTF)<%K?4&V7
M+/;K/7,/))/2P?D&N^Z-VQ\G'YJK<5,!GPJJHYX*^.YV>XQ^)15L8T'CH='1
M(WH@[!T0=^\#Q^,]K2[8US1NE/&ZSC9$K8@'-+OEU-+=5>5C?S\U2D6D</\`
MAK373&I,ORV]-L./,<6LDY>:6<@Z/*/=O8'0DD'0\U+0X3PDOP?28SGE?3W'
ME)C9<80&RGR`VQG4^[9/P1N*\@';?C<*$O7<6.1S/B.DTXAKB`?(D#MW\NZR
M%%<>%V!5V<7*H8RJCH+;1,\2MK91ML3>N@!L;)T3W``!)*N--AG!FY5K;-;L
M]NC+E([PXIIH1ZN]_8`;C;Y_Z0WY%&8SWMU;"^+-6IY76L;&E,1U.+=SI:7:
M1ZT-OO2QU%/9[BESPW))K)=`TR,`?%*SZDT9[/;\.A'P((5UQ?AG9*;$J?+.
M(60266WUG6DIX&<T\S2-AW8]QUT&GH020HM@>YQ;7'/HK9^JXL,+)BZP_P"6
M@277Y`;E5[A%@O\`+_(JFT?2GT;X%(ZI\7U?Q>;3V-Y=<S=?7WO?DH3&<=N&
M1Y-#C]J$;ZR=SQ'XCN1IY&N<=GRZ-*^@^`5GX?TN85E=AF3UMPD-`Z*6CK*<
MMD:TR1GG#N5H(V`-`?U@L-X?V&/)^(E'8I:N6D95S2-,T0]INFN=T_9I7OQP
MUL?<DG@\\+D8_6'S399LM;&QI&II!::<22-B>`:O?LH.^6RJLUYK+36A@J:2
M9T,H:[8#FG1T?-<:LS\<B=Q3.(NJY70F]_1QJ"!SEOC^'SZ[;UU6@Y)PLP7$
M+Q(S*\XEAI'!IIJ:FAYZMXT-N<`"&#>P.AWKOY*IN.]]D<`^:Z,O6,?'T1R$
ME[FV`&DD^P%_DL81:?Q#X<6.APN'-<*O<]VLID$<XJ&@21$GE!.@W^MH$$`]
M1W!5>X7X)<,YN\U/!41T5#2,\2MK)1ML+>NNG39.CTV.Q.^BB<>0/#*W*LCZ
MOB28SLG536V#8((([$<WZ*HHM;GQ7@M)(;=2<0+G%7;Y!4RTW-3%WW,'3X\V
MOBJ]QIP:EP+(J.UTE?-6MGHQ4.?*P-()>YNAKR]E2?C/:TNV('D05#&ZSCSS
M-@IS7.L@.:YM@=]PJ*B*Q</,0N>;9''9K9RL/*9)YW_4AC&MN/O[@`>9(^:I
M:TO(:WE=&>>/'C=+*::-R5746PS8=P6I*TV>KSZYNN#7>&^HCB'JS7]N_AD:
MWY\VOBJ7Q0P2X8+>8J6HGCK**J9XE'61C39F^?3KHC8V-GN.O56OQWL;JV(]
M#=+GXO6<;)E$0#FN.XU-+=0]+&_W7?PYX:5N66NJOE9=:.QV2E=R25M5V+NF
MP!L#0V.I([ZZKBXAXG9,;BHI;-F5OR-M2Y[7BF:&NAY>77,`]W?9UV[%;?56
M3"'<`+5;I\KFIK"ZJ+Q<&T[BZ63Q)"6%NM]';';^H%@6?6W&K7>(J?%;[)>:
M)U.U[YWQ&,MDYG`LT0/(-._BM$\#8HQL+(&]_MY+C])ZI/U#,>2]P:US@&Z-
MJ&V[B+#KWJQ7%*O*=J<2OM-AU+ETU(UMGJI3##-XK27.#G-UR[YAU8[R\E!+
MZ@N]CP.3@/9[949;-%CT=8YU/<A3.+I7^),2WEY=CJ7CM_556-`)M6_`71ZU
MU9W3C``TG6ZCL3M1NJ[\4.^^VR^;\9MGTUDEKLWC^!Z_614WB\G-R<[PWFUL
M;UO>MA2G$S%?Y&9A58]Z_P"O>KLC=X_@^'S<S`[ZO,=:WKNK-26G$;5Q0PUF
M)Y'+>XY+K3&=SX#'X9$[-#J!O?7]BOO&?%,-?Q"K;[FF7?1\-2R(4]#1QF2I
M<&QM:7.Z'E&P=;!!]_DIMQKB/%@C>]JK\EDFZYX>=&/BT.8XZ=)U%P<`-JU<
M7Z5NOGA6'$\.O63VV[W"U,@=!:81-5&20-(:0\]!Y]&.5OXA\.+)0X7#FN%7
MN>ZV4R".85#0)(B3R@G0;_6T""`>H[@J&X;XA%DN/99<9+C44KK-;S5,9$!J
M8ADCN5WP]C]ZK$#FR!CA?U_=;G]6AEPW9$3]-$`VTV#8!!;L;WKTNU2$5QX6
M8+49O<JMKJZ.W6RWQ>-75D@V(F]=`#8ZGE<>I``!/P-LBQC@?4R^I09[>(:D
MGE$\\&H"?OC&A\W#YJ+,=[VZM@/4TK,GK&/CRF*G.<WG2TNJ^+H;?=9$BE,L
MM4-DR.NM5/7Q7"&FE+(ZF,`-E;K8<-$CJ#[RHM4D$&BNE'(V1@>W@BU<.&?#
M^Z9Q4U3J>IIZ"WT30^KK:@^Q&#LZ'O.@3Y``=2.BZ\]PC'L>LK;A9\^M>02^
M.V%]-3L:U[00X\_21VP-:WKS"UCAG:<4_P#9[NT3\BEAHZQX==*IL)YJ60MB
MYH@->T!TZ]?KE97>,5PJ3*<;L^+Y7/=H[G6MIJMYIRPTX<^-K2-@;)YG?[JW
MOQVLB;L"3Z^?D%\GC=7ER<^4%[FL82*#-B`+.IQ&Q/86.W-J@(MFR3@Y8<;O
M,IR'-Z:U6C3?5C(SQ*JH]D<Q$;>S0=C?7MV58CQ?!)<[%L9FO_N`4?CNN#X@
MUW./\V&G77[M_!9W8LC33J'U"ZT'7<.=FN+4X5=AKJ]KJB?0;J@(M@H\5X(7
M&LCMM%G%[CJYG".*6>'4)<>@[Q#0^9'S5$XE8?78/E$MDK963CD$L$[6Z$L9
MV`[7D=@@CW@]UX_'<QNK8CT-JW%ZO!DR^#3FNJP'-+;'F+Y5:1;)+P@L%+B5
MAR>ZY@RUT%;1QU-69X^9X<^-K@R)HZN/5WQ`&]%>E!PNP;,+36'AYE5=572C
M:'.IZ^,-:_?;7L-(!]_77GK:G^#EXVORL6LI_B3!`UV[2#1=I=I!NMS5#=8N
MBUJ/!^%MDG%LRW/:B2Z`\LS;9$70P.\VE_([>CW['W@*M\7L#?@M[IH8:WU^
MVUT7C4=3H`N'FTZZ;&QU'0@@_!0?C/8W4>W._'NM&/UG&R)A"VP766VT@.KF
MB1NJ2BT/`N'5+<\<?EN6WMEAQYK^2.4LYI:AP.B&#Y@CL2='ITVK#;>'W##+
M'OMF%YI<&WCD+H8;C#IDNAO0]AO[MD#KHZ4FXLC@#MOQON5"?KN)"]S3J(;L
MXAI+6^Y`K;OY=UC:+JN]OJ[3=*JV5\1AJJ65T4K#Y.:='Y_-<JSD5L5UVN#@
M'--@JX<,^']TSBIJG4]33T%OHFA]76U!]B,'9T/>=`GR``ZD=%UY[A&/8]96
MW"SY]:\@E\=L+Z:G8UKV@AQY^DCM@:UO7F%K'#.TXI_[/=VB?D4L-'6/#KI5
M-A/-2R%L7-$!KV@.G7K]<K(,RL.$45QM%/C.6OND55*65DLL!C%,W;`UWM`;
MZ%Q_NK=)`V.(&@21Y_8+Y/$ZK-F=0D;K<UC'4`&;&A9U.(V)["QVYM4M%KLN
M-<#:*3U6KSJ]54PZ.EI8/R0/_=.V/D2H/BIPZAQ:V6_(;)=VW>P7$\L%1RZ<
MQQ!(:[70[`/7IV(("H=C/:TNL&O(@KK0=;QY96Q$.:7<:FEH/L2%7[;AUZN&
M%5^7TS(#:Z";P9W.D`>'>QV;Y_Y1JE.'.!?RPL>27/Z5]2^A*9L_A^K^)XVV
MR.UOF'+_`)/OH]_@OU8\0BKN$-^S!UQJ(Y;=5QP-I6@>'('.B&S\?;_<%<_1
MS_F-Q)^S&?A5*MAA:Z1H<-B"?NL74NI2Q8<\D+_B8]K>.+++&]@['GU\PL51
M3V!8I=,SR**RVIK1(X%\LK_J0QCN]WPZ@?$D!:!)B7!:WU!MERSVZSUS#R23
MTL'Y!KONC=L?)Q^:HC@<\:M@/4TNEE]5@QI/"(<YU736EQ`\S7"R%%=>*F`S
MX554<\%?'<[/<8_$HJV,:#QT.CHD;T0=@Z(._>!)\/>&]'=<:ER_+KT+'CT;
MRQD@;N6<@Z/*.O3>QV))!Z=$&/(7EE;A'=7Q&XS<K5;';"@;)XH#F[[4JSPW
MQC^6.94..^O>H^M"0^/X7B<O)&Y_U=C>^77?S7!E=I^@<GN=E]8]8]1JI*?Q
M>3DY^1Q&];.MZ[;*W'A!9.&$?$:V5N(Y;<JBN@$VJ.MIR#,#$]IY7<C0-`D^
M?99#Q7_2;DWVI4?B%6R0!D(=WOL;[+#A=4?E=3=$+#`P&G-TF]1%[B^*]/K:
M_F581?<:LEKO%T93MI;FP/IC'*'$@M#NH\NA"K2V_CY+S\)^''^G0,=__!%_
MU6(*&3&V.32WT^RU]$S9<W$$TM7;AMZ.(_9$1%G761$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$6U8+_17S#[3_P"5,L56R<*<CP=G"&\X
M?E=^EM<EPN#I?R5-)(X1\L.B"UCAWC(T5S1V;@-1.\>;+;_<PWKX$=,Z/G^!
M)B'_`!"WR1B5C"'#8>87R>'F.P<C);)$\ZGDBF.((IHYJNWFI'T7FFU0Y5EM
M6"RWT5!X;GNZ!SOKD#WD!H_WA[U[^C`+?%C><UE9%-*(J-GBMISJ8Q<DQ<&'
M8]HZZ=1U`54XC<2:6[6"+$<1M/T)C<1!=%O\K4$'8+]$]-]=;))&R5"\*LXK
M,$R3Z1A@%523L\&KIB[7B,WOH?)P/8_,>:DR>.-[&W8;>_NJ,GI>7FXN5,6Z
M7RZ:;>],((!/`)W]K"M?TMP#_P#A?+/^];_^9?CBGGF&WSAY:L5Q>W7>F%NJ
MQ)&:UK"!'RR`MY@]Q)VYOW!===2<!KW4NN3+Y>\?,IYWT3:9SFM)[ANF/`_W
MB/=I5;B)5<.66JEMF$4=QDGCF+ZBX5GUI6\I`:.O3KU^JW[TD<YK'46T?+DJ
M6)##-D1%S)RYIOX[TM-=R=CY;6KIQY+H.$_#FEI-BA?1"1X;V+Q#%RD_'VG_
M`+2L36L87GN+W+!8\%XATM4^AIW<U#7TPW)!WT"._39`(!Z'1'1>L47`NP!]
M=%<;UDU0`?!IGP%D8=Y%W,UF^OQ(^!49F-G<'AP`H<GC97=.R)>F1.Q9(7N<
M'.(+6V'`DD&^!SO=+OP8F#T6\NFHNE2^NY)2WOR$P`CY<I=^TK$U?^$&?4^)
MON%IO="ZX8_=6>'60-ZN;T(YF@D;Z$@C8\NO162EI.`ENKFW<7N^7%D;O$9;
M'T[M$]PTDL:"/AS?,E>.:V9C*<!0HWLIQ32=,R,@/B<\/=J:6B[L`4:XHCOL
MNOTEP)K9@=36'5;-;W>M./UOJPGK][G_`+UY>EJZ2++K)01CEHH+6#`P?5:3
M(\'7W-9^P*A<5<UJLZRE]UFA]7IHV"&DI][\.,$GJ?-Q))/[/)7QF8</\]Q*
MV6WB#4U]JN]KC\*.OIHR\3-T!UTUW?0V"._4'J0K'2LF,C0:NJOO2PP8.1T^
M/#ED87>'KU!NY&O<4!S7!I?CT1?TB7+[)D_&B5;X%?ILLOZQ-^%(M`X89=P>
MPF^U$5LJ+E^4IG"2[UL3W<YYFZB8QC=@'J[9:/JCNLNX3WJV6/BC;+U=*GU>
M@@FE=)+R.=R@QO`Z-!)ZD>2\^%@B&H&CO^84ZFR7Y\@B<T/C`;8HGX7\#]N?
M129_I(__`'?_`.L7MZ2_Z8+I_LH/PFJ(-[MG^&W^4?K/_NO^4?KOC^&[_(^L
M\_-RZYOJ]=:W\%Z\<;[:LDXD5]WLM5ZU12QPADOAN9LMC:#T<`>X/DJWN;X+
MA?\`5_E;L>"4=1@>6FA$036P-MV]_173""7^BKES7=0VY:'P_P"S'_BF"$TO
MHO9A5T?2IEK?"E<WOR'P&D?+E<[]I4)BF5V"AX`Y)B]57^'=ZVN\6GI_!>>=
MGY#KS!O*/J.[D=ODN3@[G=MQR"Z8]DM))5X]=V<M0V/JZ)VM<P'F".^NHT".
MW6YLK`6`G^FO8[KG3860Z/(<UA-3!]5\P&FZOG@^Y"SR+_*-^86S>EY_/VU_
M9;?Q9%QBDX'6FI;6T=POV2S<X\"W%AB87$]`]Q8TD;]Q^XKI]+F5C^(=NB:0
M71VMG-\-RR*L1^'COW!W'&_FM1S?Q?5\8B-S0&R?,--[-['?;VKRM8TML]',
MF+!^(=52?G".VCP2/K#\E,1K^\!^P+$U<.$V<5&"Y*:_P/6J&HC\&LI]ZYV;
MWL;Z<P/;?Q'FJ,61L<H<[A=7KN)+EX#XHA;MB!YT0:^M4J>MLXFDS>C=@\]9
MUJVU#8XR[OX7)*!]VFQ_N7+-1<`JNM-V^G+Y21/=XCK6V!V@>_('!AT/[_WJ
ML\7\]CS&LHJ*UT1M]BM<?A4-,=`ZT!S.`Z#H``.N@._4JT-$+'VX&Q0K=<Z2
M63J>5CZ(G,$;M3BX:>Q%"^;O>ME<K]_1'Q_[0?\`CSK$5J5VRS'Y_1UL^)Q5
M_->:>L=++3>#(.5IEF=OFY>4]'M['S66J&4X.+:/](6OH4,D39];2+E>18JP
M3L?8^:+:LB_HE8W]IO\`QJA8JM>P',<,N/#%W#_.9JRAIX9S-25E/&7\NW%W
M8!Q!!<[R((=Y)BD6YI-6"%YUZ.0M@E8TNT2-<0!9JB#0[\JA<,OTD8Q]L4GX
MS%:/27_3!=/]E!^$U<]0>'^/Y]BU9C-[KZ^AI:^&>X5-5"YH8ULK#MK>0.(#
M0X]B5R\<;[:LDXD5]WLM5ZU12QPADOAN9LMC:#T<`>X/DI&F0.98NQ]E7&7S
M]5CR`QP;X;AN"*.H;&^":NO)73""7^BKES7=0VY:`]W_`&8_\5P\!?YC<2_L
M-WX4ZXL4RNP4/`'),7JJ_P`.[UM=XM/3^"\\[/R'7F#>4?4=W([?)<O"/);)
M8L5SFANM;ZO472U.IZ-GA/?XLGARC6V@@=7-ZG0ZJYLC=<9OAO\`E<V;%G.-
MF-##;I6D;'<6S<>8V._HN?@YGE'AM5=*.\6Y]?9KO`(:R.,CG``<-@$@$:>X
M$;'?OT5IHL+X0Y=6-I,7R^X6NX5!U#2UL7,WF/9HV!OY<Y*I_#*NP".EN5MS
MJ@JWMJ71NI:RF;[=.6AW-U!WH['33AT[*W68<$,7NU/D%)D-^O-112">FI/5
MRWVVG;228V`Z(![C[U&`VQH>6D>O(5G4VF/)EDQVRMD-;M%M<0-KL$;<&Z68
M9CC]=BV2UMAN7(:FD>&ES#MKP0'-</@005$J>X@9+/E^7U^05$+8#5/')$#O
MD8UH:T;\SH#9\SM0*Q2:=1T\=E]1B&8P,,_ST+KSK?\`5;=P_P#Z+68_:#OX
M:99EPR_21C'VQ2?C,5QP_+,?H.`>2XO5U_AW>MK'2T]/X,AYV\L(WS!O*/J.
M[D=E1<%K:6VYM8KC6R^%2TMRIYYG\I=RL;(USCH;)T`>@ZK5(]I,6_`'W7"P
MX)6MSK:1J<ZMN?A''FKEZ3!)XOW,$D@0P`?#\DU<'!_`8\UK+A4W&X?1UGM<
M0EK9P!S:.R`-]!T:XDG>M=NJ\^.-]M62<2*^[V6J]:HI8X0R7PW,V6QM!Z.`
M/<'R7=P4SBUXM+=[1D--+-9+U3^!5&(;='T<W>O,$/<#KKV([(3&[*)>?ALK
MQK<R'H,;<<$2AC=JW'&K8]P+H>:F)+GP%MDAA@Q[(;R8SH3OG=&U_P`1I[3_
M`.$+M]+S^?-I/_\`C!^*]1=3!P/L<AN%+<+[DSV]8J!S##&3Y"1Y8TZ^7["O
M'TC<KL&6Y5;Z['J_UVGAH1%(_P`%\>G^(\ZT]H/8A72.`@>TEM[<+GX<#G=3
M@E8V4M`?9DOD@=CQ^0![76TUQY_1/PS^S&?@0KS]$1Q'$.Y-WT-I>2/_`)T7
M_51/%K*[!?>'N#6JU5_K%9:Z%L59'X+V^$\11-UMS0#U:X=">R\_1SRFQ8EF
ME;<<@KO4J66W/A8_PGR;>9(W`:8">S3^Q>>(W\6UU[;?9#B3_P#IZ6'0=9U[
M4;W>2-N>-UG->`*ZH`Z`2N_XE;+Z0A+^''#61W5QMO4_.&!8S6/;)5S2,.VN
MD<0?>"5I?&'*[!?\'P>VVFO]9JK70^%61^"]GA/\*%NMN:`>K7=M]EGB<!%(
M">:^Z[&?#([.PWM:2&EUFN+81OY;K1<VJ,!M_"[!(LMM=WKJ-]O8ZF^CW`-:
M_P`*,N+]O;U.^G?^MV50L65\$+)>*6[6[',KBJZ6021.+V$!P]X,W51^$<0<
M<J\*9@W$*WU%3;(7<U'64_66G[Z^/39T1OH=$$+V9:N`M%)ZU)D^071C>HI&
MT[F%WP+C&W]Q"V.EUD/86]N:L4OG8L$XK7X^2V8[N^0N+7`DGML#ON#2I7%6
M_P!ORG/KG?K7!404E6Z-S&3M:UX(C:UVPTD=7`GOYJL*5RZKM-=DE;56*A?0
M6Q[QZM3O^LQ@`&CU/7ION>_=12YDA+GDE?;X<;8L>-C00``*/(H<'U\UMW#_
M`/HM9C]H._AIEE>#XY5Y9E5#8*)[(Y:MY!D<-AC0"YSC[]`$Z\^RO&'Y9C]!
MP#R7%ZNO\.[UM8Z6GI_!D/.WEA&^8-Y1]1W<CLJ;PYR63$,SM^0,A\<4SSXD
M6]<['-+7`'WZ)U\=+5(YCC$"=J%_FN'AQ943<YT;:>7N+;&Q.D4=^1?T6CWJ
MV<$L,KY;/<X[]D5QIG<E287AC&/'=O1S.WN!=KMM37%.IL=;Z-MEJ<<M\UOM
M3KE_B]/-(7O9ITX=LDD]7!Q[GNH3)O\``=?;M4Y)-?[_`$\]9(:BHM\-.>8R
M..W:)86C9WOVB.O0A?GB+FV$W;@O;<<QMKZ">GKN=MN>V1SHXPZ7VG2$<I+N
M8.(#CHNUY+27-:UXMH%&JY7#;#--+BOTS.<'M+B^](V-[<<]P*`VO??PP_\`
MHOYG]IP?QTZ[?1S_`)C<2?LQGX52JWC>2V2DX#Y/C-16\EVKJZ*6GI_">>=C
M7PDGF`Y1T8[N1V75P6RNP8[BN;T-XK_5JBYT+8J-G@O?XKQ',"-M:0.KV]]=
MU7%(T21DGAI_=;<W%G=AYC6L)+I6D;'<7'N/,;'?T*L?HP"WQ8WG-96132B*
MC9XK:<ZF,7),7!AV/:.NG4=0%"_2W`/_`.%\L_[UO_YE5.%6<5F"9)](PP"J
MI)V>#5TQ=KQ&;WT/DX'L?F/-7.NI.`U[J77)E\O>/F4\[Z)M,YS6D]PW3'@?
M[Q'NTD<@="UK=-B^?VM,S$?!U"::02Z)-)!C)["B'!N_J#5;KDXIYYAM\X>6
MK%<7MUWIA;JL21FM:P@1\L@+>8/<2=N;]P4KQV)IN$W#BBI.E%)1"5X;V<\0
MQ:)^/MO/WE4SB)5<.66JEMF$4=QDGCF+ZBX5GUI6\I`:.O3KU^JW[U9<2S3#
MK_P^I\(XAFLI64#^:@N-,TN<P==`@`G8!([$$:[$`KSQ-9>QSA9`%]MNR]_"
M?AX\?(@B>6L>YQ:=W_$"-5<G<W7*AO1O_3)9/E4?^7D4#Q7_`$FY-]J5'XA6
MG8%?.#.$Y725-NK+K<9B'AUTJHG".E:6.^K&UH<2[HWZO3F)VLGX@U])=<ZO
MESH)?&I*JOFFADY2WF8YY(.B`1T]X5<@#,<,L$WV]ENPI),CJSY_#<UOA@`N
M%6=1/[\&CZ52O_&FX4E7PRX;T]/5P32PVYS96,D#G1D1PC3@.W8]_<LD5PS:
M#!(L8L#\7K)YKN^`&[,>)`UDG(W8',T#ZW-]4E4]4Y#BZ2SZ<>RZ'1H6PX@8
MT$;N^84=W$\>6^WF-T1$5"ZJ(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B*T<.;OBUIN54[+;`
M^\T,\'AM9&[E?&[F!Y@=@]@1T([JKHI,<6FPJ<B!L\9C<2`?(D'Z$;A;!19C
MP;L56RZV+!KO47"%W/`VMG_)L>.Q_P`H_L?,@K-\RR*XY7D=7?;HYIJ*EP]E
M@TV-H&FM:/<`%#HK))W/&G8#T%+'B=*@Q9#*"YSZJW.+C7D+X'LB(BI721$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$6V<"^'.(Y9@]9=\A-2R=ER-)')'/R-`+(N0:UW+I-?>%DV662KQ
MS)*^QUPU/1S&,G6@X?U7#X$$$?`K6N'\LD/HNY;-"]T<D=T#F.:=%I'JQ!"_
M6?V2?B?8,5S2R0M=<:Q[+7=&M'1DH.A([78=^O\`9<Q=*2%KX6A@^*K]]Z7Q
M>)U*;'ZC,Z=]Q%Y9OPTAH</8&R/<!>/`GA=9,FL,EYRGQVQ5=3ZO;HXY?#,A
M8USGN^/8C^XY9=F]OIK3F=[M=&'"FH[A/!$''9#&2.:-GSZ!?0%JN])%QTQC
M!;.__P!UXY230'7^<G,#N=Q]Y'0?,N]ZPGBA^DK)_M>J_%<HY,;&0@-&X-$^
M>RMZ+EY61U&1TKCI>P/:WL`7$#;S(`)]2JZBV6U<8<Q?#%18AA]NA@A8UA93
MT3YGN(&MN+-#K\E.<0;>[+^"U9E]^Q46#)+;.UKG>JN@,[.9@)T[J6D/Z;WH
MM.CW58QFN:2QUT+X_=;7];G@F8S)A#0YP;L\$@GBVT-O8FE6^`EHM-RPWB!/
M<;715DU);V/II*BG;(Z%WAU!VPN!+3MHZCW#W+(EM7HY_P`QN)/V8S\*I4/P
MRPFUT5F.?Y[^0L,!W24CA[=?)Y`-\V].WGY^R"3(PF2.,-];_/NL[.HLQ,W,
M=(2?B8&@;DDL&P'F?_)66HK=E=\KN(6:4T=-14U"RHG934-)"P-9&'N`&R!U
M))&S_P```%K&1UM7PMEAQO`<,-=610L=6W>>@DE=*]PWII;_`-=#>M="54R`
M.LW\([U^RWY/5I(!''X?_%>"=.H``#FW5Z@;`V>%\\HOHVSBMXM8]>+5E^)-
MME\I:<S4%Q91OAYG>327=>^MC>B">@(VOG)1FA\,`@V#]%=TWJ?XQTD;VZ7L
MJQ8<-]P01R#]$1!W7T;Q!NU7P@MUIMV%8Y0NI9:4/J+O-3F0S2;T07`CKT#N
MI[.``&DAA#P7$T!]4ZCU)V+)'#&S4^2Z!.D;<V=_/BBOG)%H><<3&YEC+Z&[
MXU:X;L)6/BN-+'RNY1]9I!V>OP=KX*PXI1TC_1;R>K?2P.J67$!LQC!>T<U/
MT#NX[G]JD(&O<0QU@`GCR5<G4Y8(F/GBTN<]K*L'YC5W7'T!6-HI_AO&R7B)
MC44K&OC?=J5KFN&PX&9NP1YA6;TCJ>GI>+5SAIH(H(A%!ID;`UH_)-\@JQ%<
M9DOO2U.S@W-;B:>6EU^Q`K]5G2+Z6XZWG#L-R.AG_D=172\R4@,(G8&TL,?.
M[VBP#VGD[[CH`.H67R7*IXMYO8+-]$VRSO<]T;WT,7(#']9SB#W(:UVE=+C"
M-V@.MWDN;@=<DRX!E.A+(J)+B1M5\#DC;T]EG*+Z"R/*KS@]UEQSAU@0AHJ)
MW@R5LMNEEDJ7CHYW,-;&]C9WO730Z+\9!1GB-PLO-_O.+BQY-9&F8SLIG0MJ
M8@.9W1W4CE#NA)T0#OJ0I'$&X#K(]-OS53?XA>-$DD6F-Q`!U`N%\$M[>NY(
M6`(BD\3NWT'DMON_@LG;2SMD?$]H(D9OVFD'WC8^]8P`3NOHY'.:PEHL^7FH
MQ%JOI(X]1T&34.26>.,6J^4K9XC$W3`\-&]`=@6ECOF2N_AQ24F(\&K]G-PI
M8):VXGU*U":,.T>K>=N_]+F/RB6C\,1*6$\=_1<;_6XW8,>4QMEY`#>^HFB/
MIO?LL;1:SP3QVQ4^,WSB)DU$VOHK1^3IJ5X!9+-H'V@>AZN8!O8]HGR"Z:7C
MS=9ZUM-><<L=38WNY9*)E.=MC^!<2"=>\:/P7K8&AH=(ZKXVM)>JSNFDCQ(?
M$#-G'4&[U=#8V:]AZK'46C\?,/MV+Y)25MC'+9[O3^LTS-G49Z<S1O\`J]6D
M?ZVO)3O'JCI*;AWPYEIZ6"&26W;D='&&EY\&#J2._<_M7CL9S==GY5*+K44W
MX<QM)$UUVJ@2;'TI8VBUWT4:2EK.(E?%5TT-1&+3(X-EC#@#XT/71\^I647`
M`5]0`-`2NZ?>56Z+3&U]\W^BUPYPERY<;3\@:;\]5_:EXK7;]:+3'Z,./WB.
MUT3+E+<'LDJVT[1,]OBSC1?KF(TT#6_(>Y9$OHBTW>TV/T8\=N-VLD5Z;'62
M^KTTSM1>-X\_*YX\V@<W31WT^8OQ&AVL'_I/W"YG\1320C&=&"3XK=@:OX7;
M>7YKYW1?0?"OB5/GF2-PO*+#99;;70R"&."G+!&6L+M:)/333U&B#H[6,Y!1
M0XSGM=0^&*J"V7)[`Q_^=9')T!Z>8'7IYJN2%K6A[76#LM6'U.66=^--%H>T
M!P%V"#8YH=Q2@D6V47&/B!7OUC.&V]M$#IL%+;I9AKW$M(!^X!>W'.S4MQX:
MV3.Y<>^@+U/4"GKJ7P3$7;Y^KFD`[VS8)ZZ=U)Z*9QFEA<QUUZ4LS>M31Y$<
M.3$&ZS0IX<0:O<4*]Q:PU%M%OI*/-O1VJ1!2P"]8S+SN>R,!\D(!.R1U(Y"[
MYF-4G@OC7\JN(MLMTD?/2Q/]9J@1L>$SJ0?@3RM_O*!QSJ:!OJ6J/J\9BGDD
M&GPB01["P?J.%346FY1##Q'XZ?0]J;#2T+ZGU2%T,8`$,0)?(`.AWI[A\P%=
M<CRJ\X/=9<<X=8$(:*B=X,E;+;I99*EXZ.=S#6QO8V=[UTT.BFW&!LEVP-75
MVL\O6GM\.-D5R.;J(+@`T>I(YO:@#P5\^HM_R"C/$;A9>;_><7%CR:R-,QG9
M3.A;4Q`<SNCNI'*'="3H@'?4A8`JYH?"(WL%;.F=2&<UX+=+F&B+OUV(Y!!1
M$6RVKC#F+X8J+$,/MT,$+&L+*>B?,]Q`UMQ9H=?DHQ,8[YC7TM69V3D0`&&,
M.\[<&@?H>?98TB^@>(-O=E_!:LR^_8J+!DEMG:USO570&=G,P$Z=U+2']-[T
M6G1[KY^4IX?"(%V"+5?2NI#J$;G:=+FDM(N]Q7!'(WY1%)XG<8[1D]LN<T;9
M8J:JCDE8YO,'L#AS`CSV-A7_`-)"P,H>*#3;:9@ANM-#-!'"T!I=_D]-`Z=2
MT'^\HMBU1EX['[JR3.$68S&</F:2#_;5C\C:RU%]%<6L4LC>$\MHM,4!NV)"
MF=6NCC`>\/C',2?,'FY^O]E5+AO3TEGX%YEDU53023U3VT%*Z2,.<TD!O,W?
M8_E2>G]CX*]V(6OTD]K_`"_W2YD'\11SXOCL8;UA@'<ZB*/L0=2R)%K/!+';
M%3XS?.(F343:^BM'Y.FI7@%DDV@?:!Z'JY@&]CVB?(+II>/-UGK6TUYQRQU-
MC>[EDHF4YVV/X%Q()U[QH_!1;`T-#I'5?&UJZ7JL[II(\2'Q`S9QU!N]70V-
MFO8>JQU%H_'S#[=B^24E;8QRV>[T_K-,S9U&>G,T;_J]6D?ZVO):+F=9A^-\
M,<$NMVQ:"[U[[9$:6$@,A<\PQ%SY=#V]=-`@[V?FO1BFW!QK2H/Z^PQP20QE
M_BV`-@00"=[VV(H[[+YS6N^E+:+39LRMD%HM=%;H7V\/?'2T[8FN=XCQLAH`
M)T!U56S[/Y,NM=-0OQVS6L4\WB,?0P\A(T1RGX=?W+2O23L%TR;BI8K/9Z9T
M]5/;AH=FL;XK]N<?)H\RILC:8GAF_P`O[K+DY<C<_%?D#PQ4MB[%`-W)X_PL
M!5CQ"3#66R\#)X+E+6NA'T8:4CD;)ROWXFR.F^3W^:O^:73'>'&/283C#*6X
MWZ4:NMV?$UQB=K19'O>B.HZ?5_UB2.3@;2TM1A7$>2HIH9GQ65SHW/8'%A\.
M;J">QZ!09!IE#+!.]^7"T9'4S+@OR"QS66W2;IQ!<!?&P-]]R/*UE"+7?11I
M*6LXB5\5730U$8M,C@V6,.`/C0]='SZE'<9)<>N$E%AV*V6V6N*0M+)8G/FG
MT>I>\.'4_?KWE1;`WPP][JOTM6S=5G_%OQ<>'66@&]0`WOT)[>1^BR)%L'I'
MV>TB+&LOM%%'1-OU)XL\,8`;S<K'AVATV0_1]_+OWKOL<-GX6<+[9EM3:*6Z
M9->_;HA5-YF4\9','`?ZI:3K1)>!O04OPI$CFD[#>_10;UYK\2*:.,E\ATAM
MB]0N[/D*-GR6((OH#A5Q.N&<9_;K'DM@L%6R7Q713BD_*0EL3G=.8D:.M?>L
MCXI1QP\2,CBBC;'&RYSM:UHT&CG/0!0DA:V,/:ZQ=<4KL/J<TN4[%GBT.#0[
M9VH437D/)5M$19UV41$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M%M6"_P!%?,/M/_E3*K<(^)U;@%-=::.C]=AK(^:)ADY1%,!H/['8([CST%#6
MG.+M;<`N6%04]$ZWW&?QYI'L<9FN]CHTAP`'Y-O<'N55UK=D$%CHSN!7W7S\
M'1VR-R8LIH+)'Z@/2FU[;A:9Z.D\U5QJMU342.EFF%3)(]QV7.,3R2?F5&WF
MWTMUX]UMLKG<M+5Y+)#,=Z]AU001ORZ'NH#!\FK\0R2GOUMAII:JG:]K&5#7
M.8>9I:=AI!['WKBO=TJ;M?JV]3AD5365+ZEXAVUK7O<7'EV20`3TZJ(E;X0:
M>;M7.P)3U"2=IIIC#0?(V3QZ6MOXT<1,FPG)CB>*P4UBME)!&8714K"90YH)
M(Y@6@`[;T&]@]5VBYY3>/1HR:X99)5254M0#3OJ(A&70<T&B``/9YN?KI4:B
MXZYO!;8J2HCM-?)$-,J:JE+IA\=AP!/QU\U%W3BUE]UQ:ZX_=9J6NAN<OB2S
M2QN\6,#DTR/E<&M;[`Z<OF[WK8<IA+CJ-$&AV'ZKYV/H.2V.&/P&!S'-+GW;
MG4;)^6]^39]*5X]%N>FI<<SVIK*<5--%20/FA/\`G&!E07-^\;"[#7VKCGBS
MK5RPV;*;5SR4,#7GP98C_5`]V@T'IL$`CH2%DV%9Q=L3M5[MMNIZ*6&\P""I
M-0QSG-:&O;MG*X`'4A[[[!0%IN%;:KE3W*W5,E-5T[Q)%*PZ+2/_`.]O-4MR
MPV-L9%C>Q]5T)N@239L^6TZ7VTQNOBF@&QY'@^BFL8?-BG$:UR7FGDI7VVY1
M.JHWM]I@:\%W[NJVGCQE_$K%<C%;9+LZ/'*R*-]+)'1PRQL=R@.:7EA.R1S#
M9ZAW3MTQSB%F]?F]73UMUMEJIZR%G(ZHI(GL?*WR#]O(.O+H/V*3PSBSF6+6
M]MMI*N"MH&#4=-6Q>(U@]P((<!\-Z7D4S8PZ,.(!X(_=3S>F3Y;XLM\+'/:"
M',=N#?=IHT0>+'!I2EGXI\8;Q4.@M5UJ:R1K2YPAMM.[E`[EQ\/0'Q*R]:9>
MN-^;W&@DH:<VRU0R-+7BBI>4D'H1[9=KYCJLS5,[PZOB+O==+I6*Z#63`R*Z
MV;R:OD@`>WU7]8USWM8T;<XZ`^*TZW<1N)O#WEL%Q8[PZ<<C*6YT_,`T=--<
M""6^[3B-=EF+7%K@YIT0=@K4:#COG$%(VGK8[1=>7L^LI-N_\#FC]R8[PPDZ
MBT^BCU?&DR&M:(&RMWL.-$'L0:/K?"LN9-M&<<#JO.JW'*2R7BDJ6QQ34[.1
MM4.=C2?>YIYR.NR"P]>ZY^'T,EQ]&#+J"C89:F.N,KHVC;N5O@/)U\F._85G
MF><0\HS0117FL8*2)W-'2T[/#B:[MO7<GYDZ\ESX#F^0817RU=CJ6-$P#9X)
M6<\4H';8]XV>H(/4^]:#DQF6^U43^]+E-Z+EMP?#!`<)`]K;)``((;J._8]N
M2NCA!:[C<N(]@-!13U+::Y4\\[HV$B*-LK7.<X]@``>ZG?27_3!=/]E!^$U>
MEQXX9O53TSX#;;?%#.R=\%'3EC)RUP=RO/,7%IUH@$;&PJ;G.3U^7Y)/?KG#
M30U,[6-<RG:YK!RM#1H.)/8>]5/?$V$QM-F[X6W'Q\Z7J3<N=@:T,+:#K-D@
M[[#R[+3_`$O?Y\VG[,'XKU3N`UUI;/Q7LE76O;'`^1\#GN.@TR,<QI^`YB%'
M<1\XNV=W6GN5WIZ*":G@\!@I6.:TMYB[9YG..]DJKKR:<'(\5OG:GT[I;V=(
M;@S[$M+37K?^5NO&#..*^'9E74K;O)!:Y9G24$GJ,#F&(G8:'&,[+>QV=]-^
M>U4I.(O%C(K%<X3<:FLMGJLC*YS+?"&-B+2'!SQ&.78)\P?<OYC?&G-[-;F6
MZ2>BNU-&T-8VX0F0@#L.8$$_>2O'+>,&99':9;3-+0T-!,SDE@HZ8-#V^[;B
MX@?(A7/R&NMPD=OV_P!E<W%Z1+"&1NQ(B6U\>V]=ZTW?UY[K/D1%SU]@MVP.
ME=Q.X'U&(A[#>;%4L?1ND/\`FR3R[/NT9&_<U0OI'7.EHZRSX#:G_P"(6"E8
MV0#^M,YHZGWD-T=^][EV>C3&ZQTV2YY6ROCMENHG0E@.A/(=/Y?F--'S>%D5
MZN-5=[O5W2M?SU-7,^:4_P"DX[/W=5T)9:QVW\QV^@X_WZ+X[I^#JZO-1N*,
MEP'D]X%_D+/IJ6R<*H7Y-P`R[$K>/$ND=0*ID+?K2-_)O``\R3$X?>/>L<MU
MKN-PNT=IHZ*>:NED\)L#6'GYMZT1Y:\]]E[XQ?[OC5WCNMDK9*.K8-<S=$.:
M>[7`]'#X%:'/Q\S=].]L5/9*>H>WE-5'2'Q?GU<6_N5>N*5K=9((VXY"W_AL
M_!GF.*QKVR'5NZM+J`-[&QL#M14CZ3\\%*[$\7;,R6IM%N(G+3O1<(VC?_=[
M^1'O71QVADK.$7#NYT[#)2P4+8I9&C88YT,6@?=U8X?<L8N==67.OGK[A4RU
M-5.\OEED=MSC[R5<\&XKY7B-I-HHS15MOV2RGKH3(V/9V>71!T3UUO6U+\0R
M1S]6P=^E<+/_`*-D8L&-X%/?$22":#M0.JC1K<[*Z>B/:[B,QN%W-%.VW_1K
MX!4EA$9D,L1#0?,Z:3T[:6+W'\X5'^U=_P`2M(I..F=07LW(OM\L8@,,=$Z%
MS:>,%S3S!K7`EWLZ!).@3[UF4TAEF?*X`.>XN.NW55S/C,;6,-U:V=-Q<MN;
M-DY+0-8:``;XOG8>?DORMJR+^B5C?VF_\:H6*JT5N<7:KX=T.#24]$+;13F>
M.5K'>,7%SW:)YN76Y#_5'DH02-8'WW%?9:.J8<F2Z`Q_T2!Q]@'#]U->CC^F
M:P__`%'_`)>535LLUMOWI.UMLNS&RTC[K5/?$[M(6![PT_`EHV/,=%G>%9'7
M8EDU)D%NBIY:JEY^1E0USF'F8YAV&D'LX^?=?FY9#<JS+)\G;(*2XS51JP^G
MVT1R%W-[.R2!OWDJ<<S&QM:=Z=?TV6;+Z=D39DLK#I#HM`/<.MQOZ6%JG%#B
MOG%JS*X8[97162CH9W4U/##2,+GL!TUWM-/UAHCE`Z$*7XI5%^J_1LLE3DWK
M'TM+<0Z?UB/D?HNGY-MT->QR^795)_'K.'TK(W0V4U+&\HK#1DS#X_6Y=_W=
M*O9-Q+R;),/AQJ\OIJJ&*I-2:IS7>L/<2\Z)YN70YR``T:``\EI=DL(?\9-C
M8=A^JX\'1,AKL>L=C/#<"YP-N=0()O2-N]$D_DICT<<BCLW$&.VU9::"\QFB
MF8[ZI>?\F2/G[/\`?*O,%B=PAPW-KS)[%;65)MMH<3[7A'JUX/OTXN/QB7S[
M!+)!,R:%[F21N#F.:=%I!V"%<>(_$G(<\IZ&GO#**&*C+G,;2QN8'N(`YG<S
MCL].FM=RJ8<AK(Z/S"Z^O*Z'4NC39&:'QD>&^O$'GH-M_/@^B]N`UUI;/Q7L
ME76O;'`^1\#GN.@TR,<QI^`YB%?.,&<<5\.S*NI6W>2"URS.DH)/48',,1.P
MT.,9V6]CL[Z;\]K"EH^-\:<WLUN9;I)Z*[4T;0UC;A"9"`.PY@03]Y*\@R`V
M,QEQ;O=A3ZITATV6W+9$R7X=):_WL$$@T=S>W"_LG$7BQD5BN<)N-366SU61
ME<YEOA#&Q%I#@YXC'+L$^8/N6;K0<MXP9ED=IEM,TM#0T$S.26"CI@T/;[MN
M+B!\B%GRJG>'$4XN]UT.EXSH&.U0LBL\,_<T-_HI?"+?2W7,K-;*YW+2U==#
M#,=Z]ASP"-^70]ULW&CB)DV$Y,<3Q6"FL5LI((S"Z*E83*'-!)',"T`';>@W
ML'JL#8YS'A['%KFG8(.B"M0HN.N;P6V*DJ([37R1#3*FJI2Z8?'8<`3\=?-6
MX\S6,+;+2>X^RP]7Z=+D9,<PC;*UH(T.-"S5.X(/EN%>1<\IO'HT9-<,LDJI
M*J6H!IWU$0C+H.:#1``'L\W/UTOG57VZ<6LONN+77'[K-2UT-SE\26:6-WBQ
M@<FF1\K@UK?8'3E\W>]4)1RI6R%NDDT*W5G0NGS839A*UK=3RX!O`%`;;#R\
MD7T]C=MI\TQ_ASF%8]G@V&.47![CU_(M]DG^_$'?WE\PJWV#B'?[)@UQP^C;
M2&@N!>9)'L<96![0UP:0X``@>8/<IBS-C)U\'[C<+SK_`$V?-C8<<T]I._\`
MVN!:[]#?T5UX094V_<6;[274GU++8YX)&./0;!,8^YO,P?-?WC%22X=PGQ7!
MI7-]:?43UE7R^9#G!N_@?$_\*R2SW"IM5VI+G1N#:FDF9/$3V#FD$;^&PIKB
M)FEWSF]Q7:\1TL4L4`@9'3-<U@:"3V<XG>W'S4QDCP2T_-^QW/\`OU5+^BO'
M4HY8Z$0`)'_<T%K?T/Z!:7PJA?DW`#+L2MX\2Z1U`JF0M^M(W\F\`#S),3A]
MX]ZQRW6NXW"[1VFCHIYJZ63PFP-8>?FWK1'EKSWV7OC%_N^-7>.ZV2MDHZM@
MUS-T0YI[M<#T</@5H<_'S-WT[VQ4]DIZA[>4U4=(?%^?5Q;^Y>:XI6MUD@C;
MCD*?X;/P9YCBL:]LAU;NK2Z@#>QL;`[45(^D_/!2NQ/%VS,EJ;1;B)RT[T7"
M-HW_`-WOY$>]?OCS^B?AG]F,_`A6.W.NK+G7SU]PJ9:FJG>7RRR.VYQ]Y*L&
M6YQ=LFQZQV.OIZ*.FLL`@IG0L<'O:&-9MY+B"=,'8#S7KLEK_$)_JJOHJX.B
MRX_X-C3?A%Q<?5P/'U/Y*KKZVXJ\2*#!<YM5-469D[*VF'KM6#^59!SN#6L]
M^G;<1Y_/J/DE6CB/G%VSNZT]RN]/1034\'@,%*QS6EO,7;/,YQWLE1Q\GP6.
MT\FE;UCHS>IY,!E%QM#[WKD"OLK#QCX?,Q]\62X[-Z_C%QU)!.QW/X)=U#''
MW>XGY'J.LIP%_F-Q+^PW?A3JIX?Q&ON-X_6X^RGM]SM-8"'TEPC=(QF_K<O*
MYI&_GW&QH]5Q8IF5RQNVWZWV^EHG07NF-+4>*U[C&PM>/8(<-'3SWWV"DV6)
MLHD;MS8\ME5-@Y\N"_$E(<06Z77\P#@=_(@#?S5_]$;])%P^QY?QH5D=Q_.%
M1_M7?\2I[ASFETP6]S7>T4]'//-3.IG-JF.<T-+FN)`:YIWM@\_>JY-(99GR
MN`#GN+CKMU5+Y&F%K!R+70QL.2/J,^0[Y7A@'TN_NMHX\_HGX9_9C/P(5Z\9
M8);[P6P._P!O8Z:EH*7U>I+!L1N+(V'?N`=$X;^(]ZSC+<XNV38]8['7T]%'
M366`04SH6.#WM#&LV\EQ!.F#L!YJQ<+[_P`3,;L<M9C%JJ+G9997-?$ZE=40
MB0`;.F>TTZ(WU`/Q6KQF2/<-Z('Z4N(WIN1B8T$EM$D;WD`F@0XNVOL:.R]/
M1GM]?4<5K;7P4<\E)2MG,\S6$LCYH7M&SV&R0%5^*_Z3<F^U*C\0K=.%^1\2
M<@RN&XY%0BP8U;HI9JA@I#2PR'D<!OG]IVM\W?0Y=GKI?/>9W*.\9?>+M#OP
MJRNFGCWWY7/)'[B%&=K68[6CS/.W96],FFR>KRRR`"HVCX3J`^(FB:&_?VI1
M*(BP+ZQ$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$4UCN*7_(+?<KA:*#UFFM<7BUC_&8SPF:<=Z<X$]&.[;[*%4BT@`D<JML
ML;W.8UP);R+W%[[^6R(B**L1$1$1$1$1$1$1%,XABU]RVXR6_'Z'UVIBA,SV
M>*R/3`0TG;R!W</VJ+AIJB:I]6@@DEG)($<;2YQ([Z`[]BI:30-*H31ESF!P
ML<B]Q?%^2\D7ZFCDAE=%-&^.1ATYKAHM/N(7Y458#:(B(O41$1%-C+,@;B)Q
M-M?RV9TGBNIFPQCF=S<VRX-YCUUW/D/<%"(BD7%W)5<<,<5Z&@6;-"K)[GU]
M41$458B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(O9U'5MI&UCJ6=M,\Z;,8
MSR$]>@=V\C^Q>KPD#E>*(B\7J(B(B(B_1CD$8D,;@P]`[70_>B\M?E$1%ZB(
MIJAQ2_UV*UF44M!XEHHI?"J*CQF#D?[/3E+N8_7;V![_`#4@TNX"KDECB`+W
M`6:W-;G@>Y4*BFL<Q2_Y%0W*NL]!ZS3VR(2UC_&8SPF$.(.G.!/1CNV^RA4+
M2`"1RC98WN<QK@2WD7N+W%^6R(B**L1$4CC=DNF1WF"SV:E]:KI^;PXO$:SF
MY6EQZN(`Z`GJ5Z`2:"A)(V-I>\T!N2>`%'*RXIGN78M2.I+#>YJ.G<\O,7(Q
M[.8ZV=/!&^@4'=J"KM5SJ;97Q>#5TLKH9H^8.Y7M.B-@D'K[BN92:YS#8-%5
MRPP94>F1H>T[[@$?X5HR?B'FF2TAH[SD%544SOK0M#8F._UFL`!^]5=$7CGN
M>;<;7L&/#CMT0L#1Y``#]$1$45<B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(BOV&\(\OR>TMNT$5';J!XYHIZ^8QMD'O``)U\2-'R
M59PBVQ7C,K-:JC_(U==##)H_U7/`=^[:T#TG;W65'$!^.MD=%;;5!$R&F9TC
M#G1M>7:[;TX#X`+3%&S09'[@;4N+GY>0<IF'C$-<X%Q<1=`$#86+))\]E>N&
M^"WW"<%X@1WD4KXZNUDT\U-,)&2<L4_-KL1KF'<#NOG6VT-9<J^&@H*:6IJI
MW!D44;>9SC[@%LOH]22.X<\1XG2.,;+:"UI/1I,,^R!\=#]B_7HLVVG+,IR!
MU134M50T;8J>JJ->'3<[7ETCM]`!R-V?=M:G1-F$36[`W]UPHLV7ICL^>8A[
MFEG`H$EH`VL^8OZTH:EX"9O+`QT]39*.H>W;:::K/B_+V6D?O5`R[&;UBEV=
M:[[1/I:@-YF[(<V1ODYKAT(6E5W#2S5U?)7UG&;&JBKD?SOFDJF%[G>_?B*2
MX\U%FGX88Y3#*[3D-[MU2())Z:J9)(^)S'[<0'$Z]B,$GS^:A)C-T$@57J#:
MOP^M3G*CC=()`\T:C<W3ML;/([;[]UF>"8'DV:SR,L=#SPQ'4M3*[DBC/N+C
MW/P&RK3=>!>;T=#)5TK[3=!&"7LHZO;AKO\`7:T?=M6+BO5SXIP3PO';1(ZF
MBNU,:BL?$>4RGD8]S21W!=+^QH'98W9+Q=+)6BLM-?444X&N>)Y&Q[B.Q'P/
M10>R&$ACP2>^ZUXV3U'J#79&.]K660T%I-@$BR;%60>!MZKUQBP7?);O':K)
M125E7(-\K=`-:.[G$]&CXE:'/P#SAE.]T519*BH8WF-+'5GQ?EU:&_O4QPKE
M?C7H_P"799;SX=TEJ!2LF;]:-NXV`@^1!E<?N'N6.VZZ7&WW6*ZT=9/#712>
M(V=KSS\V][)\]^>^Z\T11-;K!)._/`4QDY^=/,,5[6-C.G=MZG4";W%#<#:S
MW7G<Z&LME?-07"FEIJJ!Y9+%(W3FD>1"M>!<,LKS.E?6VRF@IZ!I+35U<GAQ
M$CN!T).O>`0%=O2;I8:V;$<D9$R*JO-O_+`#6RT1N!/_`'NM^X#W+U])NLEL
M3,?P2V2/I[52V]LKHV'0E/,Y@+O?KD)^;B5(XS(W/+]PVOK?"H9UK(S(L=D`
M#7RZK)W#=&SMMKWV&_NK5P#X;9%AF;5=PN3Z"IH9K<^%E125'.WG\2,\NB`>
MS3Y:Z+$>'AR'_"+1C%98HKR996TSI0TM!+'AV^8$?5YE?O1$DD&?W.$2.\,V
MI[BS?0D2Q`'7OZG]JK'`K]-EE_6)OPI%8=+FQ:=A9[^H6)GC03]0,Y#W"-I^
M6@?A?L19^N^ZKUYH;Y=^(=5:Z]T4E\J[HZEF((:QU0Z3D/4=`.96BEX)YW+<
MZBCFI*.DBI^7GJZBHY8-D`@!VB7=_('1Z+R/])'_`.[_`/UBDO2?NEQJ.)M7
M;)JV=]#2Q0F"G+SX;"Z-I)#>VR2>O=5".,,<]]FC2Z#LS,?D0XN.6M#H]1)%
MU5#86//A5OB#PVR;"(8:F[14\U'.[D954LA?'S:WRG8!!UVV.ODJ_CECNV17
M:*U66BDK*N7JUC/(#N23T`'O/1:]C$TEQ]%'(XJM[I6T-P#*?F._#`=`_0]W
M5[OVE?GA+([&.!&7YE0Z9='S"DBF`]J-OY-H(]QW*3_='N4CC,<\5LTBU4WK
M.3%BR!X#I6R>&.P)-42.W.XOLH:IX#9Q%3%\<UFJ:EK>8TD58?&_\30W]ZI>
M=8?>L,N4%OOD<,<\\`G8(I`\<I<6]2//;2H:*JJA<&U@J9A4^)S^-SGGYM]^
M;OOXK8?2\_G[:_LMOXLB@61/B<]H((KO?/T6EF1GXV?#CSR->UX<=FZ2--?]
MQVW6+J?P7$;SF=WEM=D9"^IB@-0X2R<@Y`YK3U]^W!0"V/T1OTD7#['E_&A5
M6/&))6L/!6_K.9)A8,N1'\S18M1%CX'YQ<J9D\\=OM7B?Y*.NJ"R23Y-:UQ'
MR.BJ=F>*WO$+N;7?*0P3%O/&X.YF2-_M-<.X_>/-<V2W2XW2_55=<:ZHJJDR
MN_*RR%SAHG0'N`\@.RUSB=/)D'HZ8AD5P<9;C#5&F\9_5SF_E6'9\R?"82?@
MK0R*1KM`((WYY7/=E9^)/"<AS7,D.F@"-)()%&S8VHW2SVBX=Y778M;\BH+?
MZY27"<P4[('<\KG!SVG;1V&V.Z]EU95PLS'&,==?;S14]/2L<UKVBH:][2XZ
M&PTGS]Q6CNOMVQ_T4[%4V:MDHJB>LE@=+%T>&&><D`]P?9'4=5AAJJZLE\*:
MLGD,[P'^)(3S'?<[/5)8XHP!1)(!_-.GY?4,R21VIK6,D<W@DD-->8`V[[JU
M8)PPRS,:-U?;::"FH&DCUNKD\.-Q'?70DZ]X&ETYIPER_%[6ZZU$5)<+>P;D
MJ*"4R-C'O((!U\=:'FKCZ4-9+9YK!A-N>^GM-);F2^"PZ;(>9S&\WOT(]_-Q
M*C?1=O%7'G+\=D>9K9<Z6434S^L9<UN^;7;>@6GW@_)6>#")/!-WQ?K[>2QC
MJ?47X1ZFTMT"W:*-Z1_W7\U;\4LEACDFF9##&^221P:QC!MSB>@``[E:?;N!
M6<5-#'4U+[3;7RC;*>KJBV4_#36D;^!*F^`^,T+..]YI'L$D5C=5&G#NOM,F
M$33]P<3\]+*\SO\`7Y-DE;>+C/)+)/*XL#CL1LW[+&CR`'32J$;(V:Y!9)(K
MV6]^;DYF4<?$<&!K6N+B+/Q70`L=A9*FLRX9Y9B-D;=[Y2T\%.ZH%.`V=KW%
MQYB#H>1#25QV+!<BOF*5>2VRD;4T5+4>KO8QVY72>QT:P#9_RC5I&7RRS^BC
MC$DTKY7_`$D1S/<2=!]0`-GW``+JX47:OL?HV95=+74&GK(+D[PI0`2TN93M
M)&_/1*M_#Q&2MZTW^BP?ZSFMPR_X3()?#X(!&JO,D?JJ'<.$6<V[&ZB_7"VP
MTM+3Q&:1DE0SQ`P>?*">OP[KKQ?@SEM^L=->HY[31452P21/JZHMYFGL=-:[
M7WJB5EUNE9/+/5W&LJ)9M^(^29SB_??9)ZK2;/PFR2Z6"CNV4Y'0V"V^&!3?
M2522YK"-@!I(#1KRV#\%7&QDCOA83]?WI;<W(RL2$'(R&,)/.@G:N`-1)/K^
MB@\XX59=B-M^E*Z"FJ[>-<]31R^(QFSH;V`0#[]:4+@N(WG,[O+:[(R%]3%`
M:APEDY!R!S6GK[]N"WW`L>M-EX9YM:J#-*/**9UOD>Z*G:.2E<8I.O1[OK:'
MN^HJ-Z(WZ2+A]CR_C0JTXC/%8.`[UO\`5<Z/K^1^!RI+#G151TEMV`=VG<?[
MI0]BX(YQ<Z5D\T=OM7B'\E'7SEDDGR:UKB/OTHX<),\.2R6%MD>9XVA[I^<"
MG##O3O$[>1Z=^AZ*LY/=KG=K_55]RKJBJJC*[\K(\DC1Z`>X#R`[+:N/]^O!
MX6X/R7&IC;<Z%LE:UCR/'/@Q'VCW(VXG7;K\E!K('-<:/P^O/Z;+3/E=5@R(
M8B]A\6Q\III`O_JMVU^6_DJ1D_!C,;%8IKR76VXTM.TOG]1G+W1M'<D%HV!Y
MZWI9PMK]$B>23++U:I'%U'/;3))$3[+B)&-!(^3W#[UBTS0R9[!V:XA53,8&
M->P5=_HM_3,K).3-BY#@XLTD$"K#@>19XI?E7JZ.S0<&;4:NHISB9KG1TD8#
M/$$VY7'?3FUOQ.Y]RHJUW(OZ*V-_;C__`%"\@;8?OV_PK.JR^&['&D&Y`-Q=
M;.W'D?54S&.'V3Y+C=1?K+1-JZ:GJ/5G,8_\JY_L=`WS'MMZ_/W*U1\!LZ?1
M.F#[0*EK.8T7KGY8?#HWE_\`%I6#AC<*ZU>C+EE?;JJ6EJH[DX,EC.G-YFT[
M3H^1T3U6;<*+A64W%+'JJ.HE$TUSABD?S';VR2!KP3Y[#BK_``X6Z`X$ZAYK
MEG,ZE.<ET3VM$3B!;22:`-'<5SSS^6]:K:2IHJZ:AJX)(:F&0QRQ/&G-<#H@
MCW[6CV7@?FUPM\5;4_1EI;*`8XZ^H+'G?;V6M=H_`Z*ZN,]11V7TB'W.:G$E
M/!54=5/$!]<-;&YPU\=']JMO$G!CQ1OW\IL0R^U5\<L+&BBGG+70\K0--`!+
M=]R"!U)1F,W4\5J(-5=?51RNMS&+'>'")DC=1<6EP!H?#R`.3N?)9-G_``]R
M7"/`?>Z>`05#BV&>&8/8\@;(]X^\!;C=^&V3U/`.S8;%%2FZTE:Z:5IG`9RF
M29W1WGT>U8CGM@S;%::ELN3-JF4(>9*1IF\6#>M'D()`.NXZ'MT6@9#-,/1.
MQR02R!YN3P7<QV?RM1YJ<`8UT@+2-N/R]%GZH[)GBPWMF8XF0?$!8)IU'YNP
MNQ?/<4L\S'A]D>*7.W6Z[0TXJ+B>6G;%,'`GF#>I\NK@K;%P"S0L;ZQ7V"DE
M?]6*:K=SGY:81^]9K:H;K<[O24EN%345\DK6TS6.//S[Z:/E[]^7=:E5<')Z
M=PJ<VX@6.T5<PYW,J)_%E/S+G-V?EOYJF*-KR2&$CWK]:71ZAF3XK8V29+6N
M-W\!<3[-!)`\^50<ZPG(L+KHZ6^T0B$P)AFC<'QR@=^5P\QL=#H]>RTC!?Z*
M^8?:?_*F4WQDHZ:#T>K#%#?(\@92W!L<-Q:W0D:/&;H=3T`]GN?JJ$P7^BOF
M'VG_`,J97MA$4K@/^DG]%RI>HOS\"*23D3-;L"+I_-'<7Y'A/1S_`)C<2?LQ
MGX52L=MM#67*OAH*"FEJ:J=P9%%&WF<X^X!;%Z.?\QN)/V8S\*I7KZ+-MIRS
M*<@=44U+54-&V*GJJC7ATW.UY=([?0`<C=GW;41%XS(F>_W6B3J`Z?/U#(JZ
M,=#U+6@?J5#4O`3-Y8&.GJ;)1U#V[;3359\7Y>RTC]ZH&78S>L4NSK7?:)]+
M4!O,W9#FR-\G-<.A"TJNX:6:NKY*^LXS8U45<C^=\TE4PO<[W[\127'FHLT_
M##'*897:<AO=NJ1!)/353))'Q.8_;B`XG7L1@D^?S7DF,W02!5>H-KW#ZU.<
MJ.-T@D#S1J-S=.VQL\CMOOW6:8%@62YM/(RQT;70PD"6IF=R11GW$^9^`!*U
MKA'PFRC%>)-IO55-:ZVA@\9LTM'4\_AET+VC8<&GN0.FU&\4ZF;%.!^&8]:9
M'4T=WIS55KXSHRGD8]S2?,%TO[&@=E5?1LDD9QAL[&2.:V1M0'M!T'#P)#H^
M_J`5*-D44S&$$NVWOS5>;DYO4.G9.0Q[6Q4\!NFR0+!)-BB:-;;>JK_%?])N
M3?:E1^(5659N*_Z3<F^U*C\0JLK#+_\`([W*^HZ?_*1?VC[!$1%6MB(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B[;#<9;1?*"
M[0`&6BJ8ZA@/8EC@X#]RW3B'@T/%6O@S3";S;7^LP,;64U3*6/B>T:&]`Z.M
M`@Z[;!(*^?D5\4P:TL<+!7*SNG/GF9D02:)&@BZL$'D$6.XL;KZ5X<XI0XGP
M_P`\HF9%0W:YOMCC6QT9YHZ;44W(WF\R?:WVUH=/?F?`;,;7C=WN-IR'8LM[
MI_5ZE^B1&?:`)UUY=/<#KW@^2S9%8<JBPL%:5CCZ"',R&Y,A?XM$F@""`*JO
M(@$;>]\K8:[@1=JNI=4XKD-DNEJ>=PS.J2'!OES<H(/S!Z^X*K<1,%H<-M5*
M)<GMUSO$LQ;-24;PYL#`T]2>_4Z[@?>J.B@^2(@Z64?=:L?#SV/:9<C4T=M`
M!/N;/Z`+=+3+9>*W"ZTXO47BEM>36,".E]:=RLG8!R@`^8+0W>MD%N]:*B(N
M"%7:@^LS')[+:;?&"XED_-))KR;S`#K]Y^!61(IG(8ZB]ED>OW6=O2,F`N9B
MSZ(W$FM()%[G2;V%^8-+6N".16*HQF^<.\GK&T%%=_RE-5/(#(YM`=2>@ZM8
M1O0VTCS"Z:7@-=8*YM3><CL=-8F.YI*UE0=NC^`<``=>\Z'Q6.(O&SM+0V1M
MUQO2G+TJ=LTDF)-X8?NX:0[>JL;BC7N/1:1Q\S&W9/DM'1V([M%G@]6IGC8#
MSTYG-^'1H'^KOS5[RNS4_&RP6:_8_=:"&_T=-ZO7T53)RGW]-`D#F+B#K1#N
MX(TOGQ$&42YQ>+#OI[*+NA-CBA;C/T.BNB1=W\UC:[Y[;KZ;X`X+!AV75@K\
MCMM9>Y:!S74%$\R>#%XD9+WNZ:.^4:T.Y[^62\"OTV67]8F_"D6?(O3DM^#2
MV@TWS[*$?1)?^9=+-J=,T-O2!5`C8`\;\?JM#/\`21_^[_\`UB]O27_3!=/]
ME!^$U9LBK=-;"RN3:V1=,\/)CR-7R,T57.X-\^G'ZK:L%_HKYA]I_P#*F7+P
M/O%FNF'9!PWOE?';VW0^-15$I`8)=-&B3TWMC"!YZ([Z6/JT<.;'CM_N551Y
M#D;+"P0<U/42-!8Z3F`Y3LCRV>X[*V.<E[`!P*]U@R^DQQX\[I'GXGZP0++2
M*JAO=5Y<*YOX)5]GF]<RK*+%:[5&[;IVS.>^0>YC"T;)]V]_`KM]+LC^7UK&
M^HM;3_\`RR+QMW#SAY9;A%<[YQ0L]?0T[Q(ZFHPU\DVCOE(:]Q`/GH'[NZIG
M%_,!FV;U-YAB?%2-8V"E8_ZPC;O1/Q)+CKRWI6RZ8X7-JB2.]K!@>/F=2CF,
MA>UC76=!8+-4!>Y.UG>AMPJ@MC]$;])%P^QY?QH5CBU?T7;M:[/G]=4W>Y4=
MO@=:I(VRU4[8FEQEB(:"X@;T"=?`K/AD"=I/FNO_`!(QS^E3M:+.E>[N!E^K
MZZ2KH\AQV>UOD+S6,JG$-;OS`;W^&]?%>/&[(K'#CUCX>XQ6-KJ"SCGJ*IAV
MV6;1'0CH?K/)UL;=H=EEM>X.KIW-(+3*X@@]#U7BO73M#2UC:OG>TAZ7.^:.
M7*FUAF[0&AN]59W-D#CCV6U9%_1*QO[3?^-4+%02#L="B*N67Q"-N`!^2V=/
MP?P;9!JO6]SN*K4;KZ>:W[(*&W\;<=M%SM-XH:/*J&#P*NCJG\GBCN2-`G6]
MD$`CVM'1"85C-OX-FLRO,+O03744[HJ"W4LO,]Q=W/4`[.M;UH`G96`HK_Q8
MU>(6_%YW^M+DG^'Y!$<1LY$!_ITBZ)LM#KX^EUM:T7@SG$=@XI/OMYDY:>Y^
M+%6R@;#/$<'\^O<'@;^&U8L@X'7"INM1<['D%A=8)Y'2PU,M40(XR=Z.@0==
MM@]?@LHQNFM]9?Z&ENU8:*@EG:RHJ`.L3">KON6F/X683XGC1\7;#ZIW]H,\
M4#W<OB=3_P#W2]AN2/2YM@'SI5]3#,/+$T4IC<YH!^`O!`NN."/MV5CXK4%N
MM/HWV&W6NZQW6EAN?(RKC;RME=NH+^7X!W,`?,#:BL%_HKYA]I_\J907&#++
M!4X_9<'Q&66HLUGV]]5("/'E.QL`@?VGG>ALNZ=!US)2FR&ME^$7\-?HJ^F]
M)EGP0)7%I,OB;C<C58L;43^E\+]1N#)&O+0X-()!['X+Z+XHXM4\7FVG)L/O
M=NEI(J,12TE1.6&!W,7$D`'3NH!!U]4:V%\Y(LT4P8TM<+!^G"[/4.G/R98Y
MX9-#V71K4*=SM8\N;7TGPQQ6BQ3!\\HVY%07:YOMFZV*B)?'3:BGY!S_`-9Q
MV[8T-:'3JJ?Z(WZ2+A]CR_C0K'$5PRP',(;\OJN>[H#Y(<EDDUF:K.D;4*X!
MW]./KRO>X_G"H_VKO^)6R\?/T5\-?LUOX$*Q-%0R72US:Y_RNID]/\>>";57
MADFJYMI'GM^JV;T1/TA7/[)?^+$L=JO^TR_ZY_XKS1>.EU1M97%_JO8,'PLR
M7)U7K#15<:;[][M%KN1?T5L;^W'_`/J%D2)')H#MN12EFX7XHQ'56AP=[T"*
M_5;5@O\`17S#[3_Y4RS;AE^DC&/MBD_&8J\BDZ;46;?*J(.F>$W(&J_%)/'%
M@#SWX]%K7'*EH*[T@9*.Z5?J=#/)21U$^P/"88V!SMGH-#KU7M4<![]+,*C'
MLCL5SH"=QU(J"P@>\AH<!]Q*Q]%(S1N<YSVW9OE4,Z9EP0118\^G0T--M!!K
MO5@C\UM?&J[T-NX8V'`YKY#?KY1SB:JJ8I/$$0`>`TN/G[8:!WTWJ!L+MPFC
MIN(7`2#";?<Z.FO=LK'3,@J'\OB`O>\'ILZ(D(V`>HZK!T4_Q=O+B-B*KT]U
M2>@!N*V)DE/:_P`350^;>_AV%;D4M:Q>QR\)>*>.UN5UEO=%-XP>::0R>KAS
M"P.=L#75^^F^@*F.('"&^7W*[EDU+DECDM%?.ZH965%60(V..PTD`C31T&CK
M0'98:BB)V:2PMVNQNIOZ3E&=N2R<"33I<=%@BR10O8B_,K?^)U!;K7Z--DM]
MJNK+M20W/E;5L9RME=SSE_*/<'<P!\P-J*P7^BOF'VG_`,J98JBD<JW:@WM7
MZ*N/H!;!X3I;/B>)=<_%=;&OK^BVKT<_YC<2?LQGX52J]P&S&UXW=[C:<AV+
M+>Z?U>I?HD1GV@"==>73W`Z]X/DLV10;DEH9I'RW];6B7HD<SLGQ'6)M/&Q&
MD`"CYV+X6PUW`B[5=2ZIQ7(;)=+4\[AF=4D.#?+FY00?F#U]P56XB8+0X;:J
M42Y/;KG>)9BV:DHWAS8&!IZD]^IUW`^]4=%X^2(@Z64?=68^'GL>TRY&IH[:
M`"?<V?T`6\4%-0\7.%-DL5+=:.BR:P#PF05+^431AH;T[G1:UG4`Z+='N"NW
M@]PU_D?Q$MM7D&26H774K:2VTLADDDW$\.+N@Y0&\Q[=>G7R7STBM;E-U->Y
MMN%=_)8)>@SF*7&AGTQ/U;:02-7(!OBSQ5^HY5FXK_I-R;[4J/Q"JRB+(]VI
MQ/FOH,>+P8FQW>D`?D$1$45<B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(M*P#A_:*K![CF^9UM;0V:$%E(RE<ULM3(#KIS-(UOV1T[[
MZ@#K9'&Z0TU9,W-BPXQ)+W(``W))X`'<K-47[J#"9Y#3L>R$N/AM>\.<&[Z`
MD``G7GH?)?A5K4$1$1>HB(B(B(B(BD,;M%5?[]16:A,0J:R411F1VF@GWD`]
M%ILG!_'Z*7U2\<4L?HJ\'3H`&N#7>XDR-(^\!71P/D%M"Y^9U7%PWADSJ)WH
M`DUY[`K(D5NXD\/[S@U7`VO?!5454"ZEK*<[CD'N^!T0=?L)5>L-`RZ7JCMT
ME9!1-J9FQ&HF.HX@3KF<?<%!T;FNT$;J^',@GA\>-ULYL+B12V86>.P9%56B
M*YTET9!R:JJ5W-%)S,:[H?AO7S!42HN!::*MBD;*P/9P18^J(B+Q6(B(B(B(
MB(B(B(B(B(BV"'AKA>-X];KAQ&R6NH:RXQ^+#14,8+V-T#[7LNZC8WT`WTZJ
M)QOAY9\OS^MMN+WN9V-T<#)Y;A4QZ>UI:-MT0WKS<PZ@#32>OGI.+("!W/:]
MUQF]>Q'-?)OH;?Q:3I-&MC6YO85SV6:HM'S"S<(Z2PUKL<RJ\5MW@#?!CFB_
M(S'F`=H^$-=-GOY>:SA521F,T2#[;K=AY;<IA>UKF_W`M/Y'LB(BK6M$1$1$
M1$1$1$1$1$1$1$1$1$1$5JS#!KEC..6&^5E72307N`3P,B+N9@+&/T[8`WIX
M[;\U55)[',-.5./D19#/$B-C<?D:/ZHB(HJY$1$1$1$1$1$1$1$1$1$1$1$1
M$1:%><'M-%P0LV<15%:;C75A@EB<]O@AH=*-@<O-O\FWNX]RILC<^Z["UER<
MR/&+!)_6X-'N03^RSU$106I$1:=58!8KSPN.88957&6JH2?I6@JY&2.B`&W%
MA:QO0?6Z]V[[$$*R.)TEZ>RQY>=%B%GB[!QJ^P)XORM9BB(JUL1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$73:J.6XW2DM\'^5JIV0L_UG.#1
M^\K7_2<N$5MEL.!VS<=OM5&R1S!_6>06MW[R&@G?^F5G/#!S&\2<:=)KE%VI
MM[_VK59_27;(WC!="_?*Z*G+/EX+1_Q!6MGPXSB.Y`_=?/90\;K,$;N&,<X>
M]AOZ`E4*R6RLO-XI+5;XO%JJN5L43?(N)UU]P]Y6O7RR<(N'LK+)D=+=,EO3
M6-=5&GE,<<)(WH:>W70@Z/,??I5?T<S".,5D\;7^?Y-]N;P7Z4+Q<$[>)^2B
MIYN?Z2F(W_9YSR_^'21U'#XE`DFMUYE^)E]2&&7N:QK-1TFB221R-Z%=NY6@
M7C#^'-QX17O-L5BN\<E-,V-D=7*/R+^:,%NAO8T_>]GOW55X0X%3Y8^X7:]U
MSK?C]JC\2LG;]9W0GE:2#KH"2='RZ=5;,,_HIY;]IC_C3+]8,#/Z+>70476I
M97<\H;WY`8"3\N5KOV%:/#8Y[7$?TW7GRN0,O)@QYXFR$GQA&'$V6@Z=]_*S
M]=URTM7P$N-:VT?0E\MS)'>&RYOJ':![!Q!>X`?'E^8"I/%3"JK!<I?:IIO6
M::1@FI*C6O$C)(ZCR(((/[?-5,=3H+;/2<!BL^"TE3TN$5M?ZP#]8>S$.O\`
M>#_WJFQ-$YQ`!%<;<KJ!DG3NH00LD<YLFH$..JBT6""=QY$<;KUNF!\,[%@^
M+Y+D-;=(#74,<TM+3'GDJY'1,<0W?1C02=]NXZA?S'\;X4\1Z>LM.*45RQ^^
MPPF6#UF9SVS:]^W.!&R-@:(WL;T5X\>?T3\,_LQGX$*K_HSPU$O%^V/A#BR&
M&=\VO)GA.:-_WG-5Y+!.(M`HUV\PN1&W(?TJ3..0\/9K(^+;X7&@1P0:K>_1
M9[*RKMMPDB<9*:JIY'1OY7<KF.!((V/O"YU8.)<L-1Q$R.:F(,3[I4%A'8CQ
M'=1\UH?#_"[5AMGCS_B*SPHV>W;+4X?E:B3NTN:?V@'YNT.^)D!>\M'`[KZ?
M(ZG'C8S)I!;W`4T<DD<#_>R[>*+);1Z.N'62[DBZ25`G9')]=D8$AU[QRMDC
M;KR^Y5'A+2<.[G)!:,JI;O+=JVO93TKJ5P;$&OY6MYNN_K$[Z'HJ_P`0\ONF
M:Y'+>+D0P:Y*>!IVR",=FC_B3YE?SAC^DG&/M>D_&:K73-=,*%C8;KGP].E@
MZ8\2.+7DN>=)JB;-`^0X]5T<6;'08WQ#NUDMC9&TE*]@C#W\SNL;7'9^9*U/
MB+@_"7!+K3RW>:\3"2#FCM=._F?(>8@O<\ZTWL-;!Z'NL_\`2"_3#D'^TB_!
M8K=Z7O\`/FT_9@_%>K::P2NT@T17YE8R^?)=T^(RN:'L)=1HFFL.Y^O/*?R/
MX?9]B5SN'#^EK[3>+7'XKZ&HE+Q,W1/3;G=]$`@]^A'4%4#A=A=3F^1&WLJ&
MTE'3QF>MJG#I#&/W;/E]Y\E??1);)'F%ZKI#RT<%K<)GGZK29&$;^YK_`-A4
MAZ,<U*+'G?);VU\IIF/;1$Z]88&S?D]Z.@20T]#W"DR)DQC<15W=>BHR<[)Z
M;'F11O+M&C27&R->QW/-<B_KLH62X<!;=4&W?0%_NK&'D=</'<WF_P!)K0]H
M(_NCY*#XO8+;L;@M=_QNMDKL>N[.>F?)]>-VM\I.AOH>G3?0@]MF7_PFX%_^
MSMH__P"P?_A4=Q)XFT.5X;0XQ;L3BL=-150J(O#J_$:`&O!:&\C=;+][WY*$
MCH7,()%]J!'[+1AP]1BRHW-9(&W\>M[7"JY`#C1!K@+MP?!<:H,(;GG$*HJF
M6V9_)0T-,=25)Z]2>AZZ.@".@V3I36'6W@OG.34MEMUCO]HJI.=S&&HYHY@U
MA<027/(.@3TUV[KSX\@NX4\-WTO_`&,4`#M=N?P8=;^/1_[U6/1O_3)9/E4?
M^7D4QICF;"&@C:]N;5#O'R^G3]0=,]KAXFD!U!NFP!7!XW)W]E5<\MM-9\UO
M5JH@X4U'72PQ!SMD-:\@;/GT"A59N*_Z3<F^U*C\0JLKGR`!Y`\U];A.+\:-
MSC9+1]D4GBKK(S(*1V1QU4EJ#CZRVF_RA'*=<O4>>E&(H@T;5\C-;"VZORY^
MB^H>/LO#:'(;>W-*2]357JGY#U$@,$?.[OMPZ[VN'@*_"7XOFIM=/<V1!LAK
M?%(YC2DS^"&=?KB/>_CI5KTOOYY6?[//XCD]&O\`F;Q%^SX_PZA=HR?\Z6T.
M_P!E^:-PO_;39?$=OIVO;YP-A];]U2,^FX926>)N%TE[AN'K#3(ZM(+/"Y7;
M`TX]>;E_>M%XC8+PFP6Z4\]WJ+M*R2'<5JIW\TDIYCM[GG6F]AK8.P>_98&M
MJ]+W^?-I^S!^*]8XWM,;Y"T6*[>Z^CS,:6/,Q<1DSPUPD).K<T&UO_LCLNVW
M8+PTXA8O57;$GUF,RVZ0>N"K>96,CUMSB"X].4.((<.K3L>:B;5)P(CN4-C?
M;+[<?$D$/TK+*Z-O,3KF#6N;IOS;OX%=?HZ?S!XD#R^C6_A5"Q:G)$\9'0AP
M_P"*E)(UK&/#!9YV\BJ<3"EGR<G&=D2:8R-/Q;_$T'<\FNUFE;N,6&MP?-9K
M/!,^:DDB;44KY-<_AN)&G:\P6N&_/6UH%TP/AG8L'Q?)<AK;I`:ZACFEI:8\
M\E7(Z)CB&[Z,:"3OMW'4+E]+O](%L^RF?BRK]\>?T3\,_LQGX$*DZ-D;Y:%U
MQ^:IBR\G,Q^GW(6F2]1&Q--/^/\`"]L?QOA3Q'IZRTXI17+'[[#"98/69G/;
M-KW[<X$;(V!HC>QO167XEB5UR/,HL7IXQ#6&5T<Q>.D(9OG<[Y:/S.AYJU>C
M/#42\7[8^$.+(89WS:\F>$YHW_><U7K@G4T,OI$YC)&YA\;UPP'R=_C+2=?,
M=?EM>,C;.&$BK-;;6IY.9/TIV5'&\O#8P\:CJ+2215G>MKHJNW`<#\5K9++5
MVN]9+54[C'45D<Y8SG'1P:&O:#H[\B/B5%\3,'QZ+$*/.\'JJB:R5,OA3T]1
MUDIG]1W[ZV-$'?4@@D'I)7#B+A5)7U%)5\&K4RHAE='*UU8-AP.B#^1]ZX\K
MXKVFZ\/Z_$+1@]/9*>K>R3GAK`YK'-D8XGD\,;)#-=_-'NA+7`D>E`@W[U]U
MYC1=39-$]C).1J+GL+2T\G2'&JY&D>BY.%N!6FZ6"OS+,*Z6AQR@=R:BZ25#
M^GL@^[9`Z=23H:T5-V<\#\FOE%8Z3'\AM4U34,AAG$_,)'.<`&N!>_E!WKH/
M/N%[Y&"[T3L>-']1MR/K/+_M)^_]XM_<LWX7?I*QC[6I?Q6J)+8BQ@:#8!-C
MS5S6RYS,G)?*YI8Y[6AKJ`T^8')/)NUW<9L>MN+<0[A9+2V1M'`R(L$C^9WM
M1M<>OS)5.6C^DK^F*[_[.G_!8LX67(:&RN`\RN[T>1\O3X'O-DL:2?,T$779
M30-O%$ZZME?;Q41FJ;%]<Q<PYPWX\N]+D15`T5T'-U-(7U!Q0FX:LP7#79%2
M7J2UNHVFUMIR/$;'X4>O$]H=>7D]_7:RK%[)@N5\8K59;)3W-EAJ8I/%94/Y
M9>=L4C^A!/3;6_O5@X\=>$W#0^7T:S\"%5;T<?TS6'_ZC_R\JZDSP_(:P@5\
M/;V7PG3,=V/T:;)9([4!+0U;`@NWKSVN_-6+(,4X28AD%;2Y!>[I<9A.[PZ&
MW`'U:/?LMD>>[]=]$:]WFJU8CPJ9E-YJ+LV\262,,-MIV;\:0GZP<1KM\7#[
MU$<6_P!)^2_:<_\`&5;N$V'8O)A5VS_,VU%3;+=+X,='"XM,CO9ZD@@G9>T`
M;`[[Z*D'7*6,:!5_[*Z1:,;`;/D32.+PT;'>S5!H`%$\7Y79[J3Q\<#\LN\&
M/TV.WNRU58_PJ>J-2YPYS]4';W@$GH/9(6:<0<;FQ',+AC\TPF-*\<DH&N=C
MFAS3KR.B-CW[6IX?Q&PV?,;-;;/POL]&:BX00Q5<CVR2Q%TC0'CV-APWL=>X
M[JK^DO\`I@NG^R@_":O9VL=#K%$@UL*572I<J'J?X=X>UCF%U/<'FP0+!LUS
MQ:N?$;!>$V"72GGN]1=IFR0[BM5._FDE/,07N>=<K>PUL'8.M]E%W'$>'^9<
M/+MDF!T==::ZS-,E125$KGA[`TN/=SN[0X@@]VD$+]>E[_/FT_9@_%>O[Z.9
MW@O$D'M]&LZ?_*J%<X,.0Z'2*W]^+7-A=DLZ/%U$SO+[;R?A(+PT@C@['D[^
MJJ7!S!Z'+:FZ7"]5DM)9;-3^L5CHA^4<-.(`Z'0TQQ)T>VO/8GXKKP$J)?4W
MXSD%'&3RBM\=SB/](M\0_N:?DJGPLSZMP2Y54L=%#<*"NC$5922G0D:-Z(.C
MHC;AU!&B>BO5I?P3S>ZP6L6*[8Y<JV011/ADW$9''30.K@-D@?5`5,&@L`;6
MKO?Z4NIU09+,J22?Q/"H:3&:K;XM0!!.^_!%+-:ZPTM?GKL?Q*H?<*:HJA#0
MRR'J]KM:+CH:UY]!K1Z+2;W9.$7#V=EER.FNF37IK&NJO5Y3%'"2-ZZ/;KH0
M=;<??I<_#;'!AGI)4E@K*AD_J[I1#-K0>'T[G,.O(Z=K7O5'XO15,/%#)658
M<)#<9G#F_L.<2S[N4M4:$49>6B[KV5AD=GY<>,V5PC$8?8-%UF@21O5;[5N5
M)<0Z?AM/8J:[81/<*:K=4"*HMU6[9C86N/.-[WU`'1Q[CLK-0X5A&$XE;[[Q
M']>KJ^YL\2FM=*[D+6:!]H@@[`(W[0UO6B0L@@+!/&9!M@<.8?#?5;-Z6[97
M9C9:IAYJ*6U@0N'U2X2/+M?<YG[EY&YI:^4M%BMNV_>E++BECGQ\!LSM+]9+
MK^*F@$-U<]^>:'*4>(<.N(UKKFX!%7V6_4<1E;05<I>R=OP)<X]]#>^A(V.N
MUF>$XO<<JRREQVC'A3S/(D>]IU"UO5[G#X`'IYG0\U=?1=BJ).+%.^$.,<=)
M,Z;78,Y=#?\`>+5:^`LM%)Q_RV2$L+)&UCJ8CMRFI:>GW?N5C(F3Z'$59HTL
MF1G3]+_%1,>7AC`YNHV038HGDC:]U$7R+@AB==+C]7:;U?ZVF/AU-9%4%H:\
M='``/:"0=]-:^)4SQ4%B'HVV'^31K#:3<]T_K>O%&W3EP=KIT=S#Y`=U@]R;
M4,N-2VJYO6!,\2\W?GV=[^];'E/]$[&/M-WXE2D<NL2#2!L>WJ$S>GG&DPWF
M5SR9&W;K'RNW`X'I795GAK@MIN&/UF99C<);?CM&_P`,"$?E:J3I[+>AZ=0.
M@Z]>VB18+3-P*R"X162.PWRS25+Q%#7/J"=./0<P+W`;.OZI'OTK/9;Q:+1Z
M-%CN%9C%/D=)!5O;44TLG(V)QEE'B$\KNNR!V_KA4YG%#!&/:]G!^TM<T[!%
M8-@_]RIZ8HFM!(W`)L$\_19S+G9\L[FMD.ESFMT/:T#3MN"X$GN;L;[*C\1\
M4JL,RZKL-3*)A%I\,P&O%C<-M=KR/D1[P5;O1IO[K5Q%BM4K@ZBO$;J::-W5
MI<`7,)'GU!;\GE0'%S-QGV2P7D6KZ-\*D;3&+Q_%YN5[W<V^5O\`;UK7DN3A
M2V1W$W&1%OF^E*<G7N$@)_=M96N:S(!CXO9=V>&;)Z,YF:*>6'5QR!SMMSOL
MO+B58VXWGEYLL8(AIJEW@@^4;O:9_P"%P5>6B^DDYCN,=YY-;#:<.^?@,_Y:
M6=*J=H;*YH[$K?TJ9T^##*_ES6D^Y`1$15+>B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(O6BJ9:.L@JX'<LT$C9(W>YS3L']H6T>D51Q9)8L>XDVI
MG/2UE,VGJN7KX3]DM!^()>PGWM`\UB2O?#[B*_&\?N>-W6T,OMCN#3S4DD_A
M&-YUMS7<KM=AY=P"".N],$C=+HW\']"%Q>J8DQEBR\<6^,FQL-37;$;[6-B+
M\E3K-<:NT76ENE!*8JJEE;+$_P!SFG8^8^"UR[\0N%N5R,NV785<3>N1K9GT
M4VHYB!H;/.T_N)`T-G2Q_P`:*.O\>FB<R)LO/''(_G(`.P"0!OYZ'R"U>;BW
MB]8[UBY<)[!4U9ZNE:]K0X^\CPCO[RI8\@:""X`>HL*GK&*Z5\<C(7.<`1;7
MAC@-MN18/OM2M=YO]ON'HTWRIH;#3V&@EK64U!31G?B-$D6WN=_6<2)-G_1\
M];64<+<^KL&N=0]E+'7VVM9X=;12'396]=$'1T1LCL002"%_>(W$2[YG'2T4
MM-2VVTT?6FH*1O+&PZUL^\@=!V`'8#9W35[/DDR-<P\"K_\`"ATOHK68DL.2
MS:1Q<6V37%#5R3M=^:V.ES/@S;:YMYMN!71]RC=XD4,TP]78_N"-R.\_]'IY
M!9UGV5W/,\DFO=T+1(\!D43/J0QCLQO[2?B22H%%5)D/>-.P'H*70Q.DP8LG
MB@N<ZJMSBX@>0OA?2><W7$;?PMX?PY?CTUVI:BUQ>&^"<QRP%L$6RW1&][[;
M';S5*FXFXEC-DK*'AIC-5;:RM9R2W"M?S2L'^C[3NONZ@`]=%5;/,]_E3B>,
MV'Z*]3^@J80>-ZQXGCZC8S?+RCE^IO6SW5*6B;+.K_AUP-ZWX\UQNF?P\SP`
M,L.O4XENHZ3\1(MH-<4:_,6K)PSO5JQ_.K;>[[225M#2O>^2)D;7N+N1P80'
M$`D/+3U/EM:=E^?\'\LNGTE?;-EU5.&\C!XC&LC;[FM$VA]W=8:BHCR'1LT`
M`CU"ZN9T:#+R!D.<YKP*!:XC;GLM+R"Y\%Y;)61V7'LE@N3H7"EDGD:8VR:]
MDN_*GIOX%5CA9&^3B7C+6-+B+M3.T/<)6DG]@*K:UJV<8+3:865-IX<6.DO,
M<08VO#AWY=%W*&`[/GIW7:DQ[7O#GD-KR'^%1DX\^+CNBQFNE+P1;GC;:A\Q
MX]E`<?W-?Q@R`M((\6,?>(F`K8^/]WP*DRB@H<SQFLN#G40DBJZ2<LD8.=PY
M.7F:".F^I\^R^:KK7U=TN=3<J^8S555*Z6:0_P!9SCLE6SB]GO\`A`OE)<_H
MKZ-]7IO`\/UCQ>;VG.WOE;KZW;7DK&Y+0)#W<1L1?<K'-T65\F&PDZ8FN!<U
MVD@TT"J(.]']U/Y-Q+L5)B53BG#S'Y;+0UO2LJ:A_-/*TC1;W=W'39<>A(`"
MI6`Y;=<+R&.\VES"\-,<L3_J31G6VN_8#OR("@$6=T[W.#KXX]%V8.E8L,+X
M=-A_S622[W)W*V&NS+@U?:AURO>"W6EN$IYIFT,X\-[CW/1[!U]X:"JMQ$RS
M&+O:J6S8KB4=DHZ>8RNF=)SS3'E(`<>IZ;\W%4=%)^2]X((&_H%3C]%QX'M>
M'/.G@%[B!]"?O:U#!^)%EBPT89G=BEO-GB?STKX7:FA.R=#J.Q)T0X$`D=0K
MUP9R#")^(%':L'PN:E$D<KZJX5TADFC8&'0:.9W*"[E!._/6NJS3`^(-LQ_'
MC9+QA5JR"G\=TS7U.@]I<`"`2UVNWDI6X\8G4]IJ;?AN)6K%O6F\DU13Z?,1
M\'!K='XD'7EH]5JAR&M#7.<-O3?VM<#J/2)IW310P.`?>_B4RS_46@W?>JW*
MI^?O^E>)=]-%^5]:NT[8>7^OS2D-U\^BTKTDJZBLMIQ_`+:R(>I4T<M6]C0"
MXM;R1@GOY.<=^]I608]<Y++?J&[Q00U$M'.R=D<P)8YS3L;T0=;`\UZY9?:[
M)<BK;[<2TU-7)SO#-\K1K0:-[.@``/DLPF`8_P`W']%VW]-<_*QR?_CB:?J[
M8#;T%GW46B(LR[:T3COG5ISR_P!!<+13UT$5-2^"\53&M<7<Y/3E<[IU3A'G
M5IQ&P97;[E3UTLMXI60TYIV-<UK@R4'GYG#0_*#MOS6=HK_Q#_%\7O\`["Y?
M^CXWX(8-'0*[[['5S[HM"X[9Q:<[R.AN5HIZV"&GHQ`\53&M<7<[G;'*YPUH
MA9ZB@V1S6E@X/[+5+AQRSQSN^9EU_P#U5_9:'PHSFTXGC.6VRXT]=+->:00T
M[H&-<UK@R5NW[<"!N0=@?-9[&X-D:X]@05_$1TCG-#3V2'#BAFDF;R^K^@H?
MHM`XZYM:L[RBCNEHIZV"&"B;3N;5,:UQ<'O=L<KG#6G#S6IYS=<1M_"WA_#E
M^/37:EJ+7%X;X)S'+`6P1;+=$;WOML=O-?-BNN>9[_*G$\9L/T5ZG]!4P@\;
MUCQ/'U&QF^7E'+]3>MGNM,>3_P#(YW)]/5<++Z&+Q((0?#C+K(=1`TFM[!Y\
MOKLK3-Q-Q+&;)64/#3&:JVUE:SDEN%:_FE8/]'VG=?=U`!ZZ*S/&KY<L=OU+
M>[7/X593/YV.(V#OH01Y@@D'YJ-14/G>\@\5Q6U+K8W2\;'8]H&K7\Q<2XN[
M;D]J[<+8[CQ`X6Y5*+CF&#UT5U<`)9K;-TE(\S[;/W[/Q4!FN881/C,U@P[#
M/HQDSV.DKJB7GG(:=\O7F.B?]+7P6=HI.RGN!L#?O0M9X.A8T+FEKG4W<-UN
M+17&U]O+A:)PPXBTN/62NQ?);1],X[7.YY(`[3XG=-EN];[`ZV-$`@CSN&!9
M'PX9F]GHL(P6J=7U59&PU5QE+S3Q\VWN8WG=U#>8[Z:^/99]PYS>CQ6EKJ.X
MXM;;_2UCF/='5@;86@CH2UP\_<K.>,E';(9G8?@%DL%;*PL];;J5[0?=IC?V
M'8^!6B&9K6MU.&WI9^A7(ZGTV66:40P.^/N)-+":Y<T&]N^QNE#>D34Q57&"
M]NA<'-C,,1(_M-A8'#[CL?<KCQ-GI\/X%XUB4443+E=(6SU1Y1SM83XC@3WZ
MO<&_)K@L4FJYJBX/KJMQJII)3+*9229'$[/,>_4]U,Y_EMSS3(#>;HR"*3PF
M0QQ0`B.-C?)H))[DGOW)57X@?\1W=WV/*WGI+S^$A)MD(L^KF@!O[GZ*OHB+
M(OH5K>/\2<2N&"4&*<0,>K;A%;="DJ*-X#PT#30=N;K0Z=R"`.G11=@R[#,?
MXMVO);)9KC16.CA>Q]/S"2=[W12,YO:?KN]O];L%G"+1^)>:NK%;UOLN..AX
MS?$#2X->'`MU'3\7)`X!4SG-UI[[F-WO-(R5E/6UDD\;90`\-<XD`@$C?WE6
MOA9Q"H<<LMQQC)+0Z[8]<7<\L+':>Q^@"1U&]\K?,$$`@K.T4&S.:_6.5JGZ
M=!/C#&>/A%5O1%<$$;V/-:_;,\X7XU>J.NQ?!ZMTC:ACI:FX2^(^&/F',8F%
MY'/K>B2-'2IW%_)Z#+\\K+];(:F&FG9$UK*AK6O'*P-.PTD=Q[U444GY#WMT
M&J]!2HQ>CX^-/^(:7%]5;G$[$@]_9:%QVSBTYWD=#<K13UL$-/1B!XJF-:XN
MYW.V.5SAK1"<)\XM.)XYEUMN-/6RS7FC;!3&G8US6N#)6[?S.!`W(.V^Q6>H
MGXA_B>+W_P!A>CI&,,(85'0*[[[&QO[J[\,LNL&/TMRMF2XQ'?+?7NC<[;@'
MPN8'#;=CN>8]00?BK509KP?Q^M9>,>P>[RW.!W/3BLJ-11O\B/RC^WO(*QY%
MZS)>P``#;C8*&3T7'R)'/<YPU<@.<`=JW`/EMLIR^97>;MF<N6RU'@W-U0V=
MCXAH1%NN0-WY``#K[NJTBNXD</,OCAJL^PVK?=8HPQU5;9=>*![QSMZ?`EVO
M(K&T7C,A[;[WS>ZLR.D8TX9L6E@II:2T@>5CMZ*^<0\LQ.YV.GL.(8H+/1PU
M`J'U$LG--,0US0'=SKVCW<?N4SCG$VQ5V)TN+<1,>EO5'1`"DJJ=_+/$T#0'
M=O8=-APV``0>ZRI%Z,EX<7?MM^2@[HN*^%L1O8V#J.H'SU7=K7:OB?BV.6&L
MMG#/&JFUU%:WDFN%9)S3-'^C[3NO7IU`'?6UG6&9'<,5R6DOMM<WUBF=LM?]
M61I&G-=\""5#HHOG>X@\5Q7968_2L:"-[`-6OYBXDEW;<GT6QWC..$%YK'WR
MY8%<WW>4^)-%'4<L$DGF20\=SW/)U\P5R<1^)]ERKAC08Y2662UUE/6";P88
MVBECC;X@:UI#MDZ<W?LCKM9.BF<N0@C;?G8++'_#^)&]C[<=!MMN)`KL`3PK
MYPOXD56'T]7::VW0WBPUN_6*&8Z&R-%S201U&M@C1T.W=6`Y'P+;)ZVS!;XZ
M?ZW@.J"(=_/Q=Z^[7P61HO&Y+VM#=B!YBU;/T7'EE=*"YCG<Z7%M^]']>58>
M(&04627\5UMLE/9:..%L$-)"06M:W?7H`-G?N5X]&:PMJ<NJ,JKR(K98H'S2
M3/\`JB0M('[&\SOAH>]9,K_>N(S'\.X,)QVR?0M#S<U9*:KQI:L]-\QY&ZV1
MU^``Z`:/L,C1)XDAXW]RJNIX<SL,86(V@[X2;^5O<[FR2-ASON56<YO;LCR^
MZWQP+165+Y&-/=K-Z8#\F@!0R(L[G%Q)*[$438HVQL&P%#V"(B+Q6(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B+TI:>HJIFP4T$L\KOJLC87./R`1>$@"RO-%-S8AED-.:F;%[W
M'"!LR.H)0T#W[Y=*$4BTCD*$<T<OR.!]C:(NEUOKVB`NHJD"HUX&XG?E=]N7
MIU[CM[UTW3'[]:H&5%TLERH87G3)*FE?&UWR+@`4TGR0S1@@:A9XW4:B*5H,
M;R*X4?KE!8+K5TW_`.M!1R/9_O`:7@!/"]?(R,6\@#U44BVS+6/C]%'&8Y&.
M8]MT<'-<-$'Q*GH0L35LT7AD"^0#^:Q=.S_QK9'::TN<WF[TFK^ODB(I:@QC
M);A2BKH,>N]73D;$L-%(]FOF!I5!I/"V22LC%O('NHE%Z5,$]-.Z"IADAE8=
M.9(TM</F"O2W4%=<JD4UNHJFLG(V(X(G2./W`;2C=*1>T-U$[+G12%WL5[LX
M:;M9KC;P\Z::JF?%S?+F`5R]''],UA_^H_\`+RJ<<9=(&':RLN7F-@Q),EOQ
M!K2=CS0NKW6>HK1Q;_2?DOVG/_&5#VBQ7N[AQM-FN-P#3IQI:9\NOGR@KQS"
M'%HW4XLEKX&S/^$$`[GB_51Z+HN-!76ZI--<**IHYP-F*>)T;A]Q&US@$G0&
MR5$BE>UP<+!V1%,.Q3*6TGKCL:O(IM;\8T,G)KW\W+I3_`['Z3(N)-MMUTM[
MZRVN\7UEOM!HU#(6\SFZU[31Y^2FR)SG!OFLN1G0PP23W88"35=A=>ZI"*V<
M0L9J[?F][I+99:V.@@K960!D+W-$8<=:)WL:\]JJQ,?+(V.-CGO<=-:T;)/N
M`47L+315N/D,GC$C#L0#^?FORBEZS%LFHJ,UE9CMWIJ8#9FEHI&,`]_,1I1"
M\+2.5..5D@MA!]D1=UILUWN[W,M-JKK@YGUA2T[Y2WY\H.ELO`F@KK=@W$NF
MN%%4T<XMC28YXG1N'Y*H\B-JZ&$RN`XY^RP=3ZFS!A=(*<06[71W<!Z^=\+#
M$1%0NFB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(BL/#[(+[CF1-K<<@$]REB=!$SP3*[VM?5:.YZ?'Y*O+;O1<8QM+E]9
M;X89LA@H0;>R36SL/Z#X%XC!^[WJ_&87R@`TN5UO)9C8,DCV:QL*/!L@;^F^
M_HI#'J[TB7W&"O?1553!SATE/5,IH6O;OJW1Y2W[M%5OTJ+/0VSB+#44,#(?
M7Z)L\[6#0,G.]I=KX@#?QV?-<#(N-.37EU*]^5B9[]/$CI:>&/W[^JQH4_Z7
MO\^;3]F#\5ZVR'5CN.^Q'/[+YG#C\#K$`'A@N:^Q&*VV(U;[^FP[JS9_F=QP
M[A-@4]GIJ/Z0JK5%''631![Z9H@BYN0'IL['?IT['RY>`^87KB!5WO$,PJ6W
M6CGM[I09(F!S=/:T@%H']L$>X@:4'QY_1/PS^S&?@0KP]$;])%P^QY?QH59X
MK_Q367MMM]%B_`X_^@39.@>("\AW<$/-4>1]%7^!&(4N4<264%R8)J*@8^IJ
M(SVE#'!H:?@7.;L>8!"D<SXTY?49!4-QZXMM5J@D,=)!#!&1R-.@7<S3LD#M
MV"Z/1MO--:N+E1354C8VW&&6EC+CH>)SM>T??RD#XD*J9+PXR^UY-4VF+'KG
M5!LSFP2P4SWLE9OV7!P&M:U\O/2S`O;`/"\S=?HNX]N-/U9XS@"`QI8'56]Z
MB`=KNA?DM,XH7RXY)Z-./WJ[2LEK:BY#Q7M8&!Q:9V@Z'0=&CLL"6^<3;+<<
M=]&?'K/=H1!6P7$&6,.#N7F,[P-CIO3@L#4<W5K;JYH*_P#A?POPTO@UI\1]
M5Q5[5Z5PK?P8H[37\3['27ML;J*2<\S)/JO>&.,;3[P7AHUY[TM1XJ9#QHL^
M6UKK=#74]FBE(HS1T+)8?"'U2X\KNNNX=Y[6*XCC]SR>^16>SMC?6RL>^)KW
M\@=R-+B`>P.ATWT^2T*TW[CECM8RW1P9#,6$-$-31.J6N'N#BT]/B"O<=Y$6
MG<;\C[*GJ^*V3-$H,;B&UHD\K/Q#FB>":[*H\0<UO&;U-!)>H*<5M'&8#)$P
ML,FW;]IOD=[[:'P6XWRQ9M@N+VS'.&MBYJB2`2W2Z,;&Y\DO8M'.?GY'0(UY
MJH>D['31RXK<IZ6FH\DJ*3Q;E%#KHX!FN;WZ=S@'S`^"LO%F3,,KL5FS7AY=
M[P^BEI1'5T5NJY&NC>"3OD8?:<"2TZ&_9'<=M+&ECI+)+A7'-?[JUQ\B=N3!
MA>&QK(G%_P`+MV:AP#56#\1;V/DN_AM'Q-NU74X]Q*LKZ^PUL#VNEJ&1`Q.`
MV.K.IW\M@Z((TLQX+6[Z'](FDM/.7^I5==3\Q\^2&9N_W+ELL'&N[U;::CJL
MUYB=%\U741,;\W/<`/VKUX%LJX^/UNCN$SIZQM16"HE<\O+Y!#-S.+CU))V=
MGNH>)K?'L=CR?HM)Q#CX^:0]E.C-M9L`:.]6:OCUI*K'697Z1=PL4SG-@J+Q
M4.G+3H^&PN>X`^1(:1OXK1<ZDXO4]V?:,$L$EHQ^A/@THIF0_E0/ZY+B2`3L
M@=.G?95%H[]2XUZ3-;=:YXCI&W>IBF>>S&R<S.8_`<P)^`4SQAMG%.U996U]
MDN^3UEDK9344KZ"LF>R)KNO)RL/L@$Z'EK6O</6$-8\B[U;US7^%5DM?+E8T
M;]&GP@6B0$M+N];@:JKGMPK%>;5E65<&K[_A#M`@O%FC=54-:6,:][6MYG#V
M.G9I![`['38VJCP/H+9C^"Y!Q.N-%'65%M=X%!'(/9;)IOM?,ND8-]P.;7=5
MY]%Q?K<>N-=7UV4Q6F"F>ZJ-=6S1L?'K1;RO=[8(.M`$>]6K@NV++.$&4\/8
M9XX[I))ZW2L>[7B#\F0/D'Q@$^7,%ZUP?*TT;HU?<]EY-`[%P)6>(TQF1NH,
MNFM)&H<F@>2/4JK-XV<1A<C6?3;"TNWZN:6/PM?V=<N]?'>_BK1P5SJ[WCCB
MZ1C(*.GR`EU?3Q,!:YT--(6EI(VWV@3T/GUVLU;@6:NN1MPQ6\>LAW*6^J/Y
M?GS:Y=?'>E;.$5LJ<2X^VBU7KPX:J"1\4@:\.`?)3NY6[[$[>T=/-40R3^(W
M635CE=3J.%TO\%.,=C-7ANV;7`%@T/(U1\^%*<2>+>=V[,[]9:2[1,HH*N:G
MCC-)$2&!Q&MEN^WFIOA%CUSL/"EV98Y8F7?)[G*Z*C+PTBEA#G-+@"1YM=\]
MM\M[HW%W#\H_PF7V2&P7*HBJ*N2HBEAIGO8YCCS;#@-=`>ONT5?,#J;MEG`-
MMAQ*[U%#D5DJ"_PJ>J,$D\9<]P&P1T(>1UZ<S!O7=71%[IW:[)%U_P"%S,V/
M'BZ7!^%#&M<6>(>U4?GTT:NK_P#U>N-7'C]27^GJ+K;:JX4+I0*FGE93AKF$
M^UKET6G6]:_853N->%4U)QHI;':(V4L5[,$D4;1ID3I9#&=#R',TG7EM1;7\
M:'5GJ8GSOQ]ZY?%JO^.]:^/9>%?)D^*\2K#79O55E16T<E-4O%35&HD9").;
MEYB3K^L=`^:KDD#F:7!U6-SV6S$Q'P91EB?$':'#3&*U=P2+/![^M*^\7LXK
MN']=3X%@ICM-+;X&&HF;$UTDCW-#NI<#Y$$GN25*\*,TR/+N'&>1Y!7"M=16
MQW@R&)K'`/BFV#R@;^J%6?2+P^]UN<NR:SVZJN=MNE/"]DU)$96AS6!NCR[U
ML-:0>QVI_@WBE_QWAAG=7>[=+0-K[:[U>.8<LA#(9MDM[M^L.^EH:9OQ);OI
M%^U5LN3-'T[_`$2.4:3*XL))K67:AJWYOF_1?/2*6P_'Z_*<CI;#;#"*NJY_
M#,SBUGLL<\[(!\FE,QQZOQ7(ZJPW,PFKI>3Q#"XN9[3&O&B0/)P7(T.TZJVX
M7Z%^)B\;P-0UU==ZNK]K42B(HJ]$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1%S7.=]+;:JIC#2^&%\C0[L2&DC:Z5PW_\Q7#]
M5E_@*G&+>`5GRW%D#W-Y`/V7<B(H+0B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B[;'=[G8[E'<;173T57']66)VCKS'Q'P/1<2+T
M$@V%%[&O:6N%@JWY!Q-SN^T+J&YY'524SQROCB8R$/'N=R-&Q\"HK+\KO^6U
MT5=D-?Z[40Q>%&_P61Z9LG6F-`[DJ%13=*]WS.)6:'`Q8"#%$UM75`"KYJAW
M[J:OV5W^^VFV6JZU_K%':XA%1Q^"QOA,#6MUMK03T:T=2>R_.(Y1?<2N4EQQ
M^N]2JI(3`]_A,DVPN:XC3P1W:WKWZ*'1>>([5JO=3_"0>$8=`T'M0K?<[<;E
M?LRR&<S\Y$O-S\PZ$'>]].RN;.+/$1MO%",IK/!#>7F+&&37^TY>??QVJ2B-
MD>SY32\GP\?(KQHPZN+`->UJ>N>99+<\:AQRX762IMD,QG9%(QA=XA+B7%^N
M=QV]W<GNH%$7CG%W)5D4,<((C:`";V%;GD^ZZ+=75MMK8ZVWU<])4Q'<<T,A
M8]I^!'4*YLXP<2&4WJXRB<LUK9IX2[_>+-_O5$129*]GRDA59&#C9)!FC:ZO
M,`_==-TN%==*Z2NN59/654IV^69Y>YWWE2&,95D6,3/EL-XJJ`O.WMC=MCS[
MRT[:?O"AD40YP.H'=6O@B?'X;F@M\JV_)7:Y\6>(EQIS!4935M81H^`R.!W^
M]&UI_>JWC]]NU@OL-\M588+C"7EDY8V0@N:6N)#@0=AQ[CS4:BDZ5[B"7&PJ
M8L#%B8Z..)H:>0``#[CNNJ[W"LNUTJ;G<)O&JZJ5TLTG*&\SW'9.@`!]P4_C
M7$3-<<I&TEGR&J@IF=&0O#96,'N:UX('W*K(O&R.:=0.ZG+B031B.1@+1V(!
M'Y*SY)Q!S3(J=U->,AK)Z=XT^%I$4;OFU@`/WA5^WUM7;ZR*MH*J:EJ8CS1R
MPO+'M/O!'4+P1'/<XV397L6+#"SPXV!K?(``?DKT_B]Q'=2>JG**CP]:V(8@
M_P#WPWF_>J;4UU;4U[KA4UE1-6.?XCIWR%TA=_:YCUW\5SHCI7O^8DJ$&#C8
M]^#&UM\T`+_)7<\6N(CK>:!V3U#X"SD/-#$7D:UU>6\Q^>U4[1<[C:*YE=:Z
MVHHJJ/ZLL$A8X?#8\O@N1%ZZ5[B"23218.-"US8XVM#N:`%^_FKX_C#Q(=3^
M`<HFY-:V*>$._P!X,W^]4VZW&X76M?6W.MJ*VI?]:6>0O<?O*Y41\KW_`#$E
M>8^#BXQ)AC:TGR`'V5KQOB/F^.6X6^SY!404C?J1/8R5K/@WG:>4?+2_C.(V
M:M;=`[(*F7Z5B\&L\9K).>/3ARCF!Y!I[OJZ[JJHO?&DJM1_-1/3<,N+C$VS
MR=(W[[[;[[J0QR]7/';S3WBSU/JU=3\WA2^&U_+S-+3T<"#T<1U"9'>[GD5Z
MGO-YJ?6JZHY?%E\-K.;E:&CHT`#HT#H/)1Z*&IVG3>RT>!%XOC:1JJKK>N:O
MFKWI$1%%6HB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(N&_P#YBN'ZK+_`5W+AO_YBN'ZK+_`59%\X]UES?Y:3^T_9=R(BK6I$
M1$1$1$1%;\;X>WF\XC6Y6^KMULM%(2#45TCVB4CN&!K7%W70^).AL[U`XQ:9
M[]D5OLU,=2UM0R%KM;Y>8Z+OD!U^Y:EZ25Y@H:JV\/+-^1M=FIXS+&T_6E+=
MMYO?II!W[WE:(HVZ'2/X'W7'S\R89,6'C[.=9)YIHY^I.P6.*P8OA65Y,PR6
M.QU=9$#HRAH;'OW<[M-W\-KNX/XM'F&?4%GJ.;U/9FJN4Z/AL&R-^6SIN_BK
M5Q>XFW>6_5&.XO626:Q6QYI88Z%WA&3D/*22W1Y=@Z`Z:UYKV.)NCQ)#MQMW
M7F9GSG)&'B-!?6HEUTT<#8;DGRV5:N_"SB#:J<SU>+UAC:-DP%DY`]Y$9<53
M7-<UQ:X%K@=$$=05;L:XF9O8*QE129#6U#`=N@JY731/'N+7$Z^8T?BHVZ3W
M'-LWJ*BBH"ZONU47LIHCOVW'>@3Y?$]AW47B(@>'=^15V+)G,<X9>C2!>IMC
M\P;][M02+7G\&K9;?#I<FXCV*SW-[0?5"`_EWVV7/:1\]:5-XCX%>L'K88[B
M8:FDJ074M93G<4H]WP.B.G[-H_&E8-3@HXO6L'*D$44ED\;$7[$@`_2U4T5_
MP+A9>LTQ&KOUHJJ;GIZOU44LGLN>0&$NYNP`#]_W2K!;>"]NN#W6ZCXCV"HO
MC6G_`!&'3QS`=1S!VSKSTWI[EZW%E<`0.5"?KW3X'.8^3=IH[$U[T#0]>%D"
MT/$>#V7Y'98[PSZ/MM%,.:&2OG,?B-/9P`:XZ/D3K?DJY0XK5'./Y*7:KIK1
M4,F?#-/4O`BB+03LGMHZZ'SV%]!\=<(AR*MM%*[,+/8J.BI>2&DK)@S9WKG`
M)'30`^Y6X^-K:YSA==KI8.L];_#RPP12!NL$EVDNH#B@.;_3\EA%YX=Y3;KO
M=;:RA%>ZTQ,FK):1W/'&Q["]KMD`ZY0?+R*J:T3#\;O[J;.8;'E7JU)9H)#6
M&GD=X=?&P2@:Y3H@AKM;Z:<J;C%BNF2WNGL]GIC45<YTUN]!H'=SCY`#N52^
M/C2#O?W73Q<S:3Q9&D,`LT1RT$DWL+Y%<#E1J+7*K@]9:%YH+EQ.QZDNXZ.I
M'ZY6N_LEY>"/O;]RK?$WAQ7X)0VJIKKG1UOTB'\HIN8M;RAI^L>X/-T*]?C2
ML:7$;#V4<?KF#D2-BC?\3N-G"]KVL"]NZHZ*W73!:N@X9VW.7U\#Z:OJ#`VG
M##SL.Y!LGM_FS^U1V`XW/EV6T6/4U3'32U?B<LLC26MY(W/Z@?ZNE7X3]0;6
MYX^JUC/QS$^8.^%EZCY:>?R4$BL$V+U$7$(8<:J(SFYMM_C\IY.8R!G-KOK9
MVKM5\#<AH[M4PUMVM-#;(.4?257+X4<A+0=,!ZDC>NNAT/52;CR/O2.%1/U?
M"@T^)(!J%CU'I^?'*RE%?X^&IESL8Q#E%GEB]3];=<&2;@:P=QL>?WJP4_"'
M&:VH;0VWBM8:JX//+'`(VZ>[W!PD._N!4FXLKN!^H5,O7L&*M3SN+^5QV\S0
MV^JR!=]RLEWMM%25MPME72TU:SGI998BULS=`[:3W&B#T]X4K><.N=CSB'%+
MW)3T,TL\47K+G_D0Q[@!+S=/8Z[/;6CO6EOG%#!*6]X+AMLERZRVYELHVQ,J
M:B4-CJ@(HV\T9WU'L[^1"G%BN>U_F%GS^OP8LL`L%DE[[G:K!%>97S79+5<+
MU<X;9:J22KK)N;PX8Q[3M-+CKY`$_<O*XT=5;J^>@K8'P5-.\QRQN[L<#H@J
M]6O#J^W<5;9CN/9;1OK9V%T-SH)26QDL?S#;3O>@0?\`655RFBN,68W.VU=1
M)<;A'7RT\DHVYT\@>6D]>I)(^?55.B+6V1O=+H0YS9I]+'#26AW!!W)%[[5M
M[VHA%K,'!J&WT-/-F>;V?&JFH;SQTLNI)`/CM[?OUL?%5[B+PVNV(4=/=65E
M+=[+4D"&OI';82>P<.NMZ.B"0?>O78TK6ZB%3!US`GE$4<ED\;&C[$BC]"51
MT5UX9\.[EGE)>);95PQ2VR-CA%(#N9SP_E:#V'U-=?>K/;>#5`^I9:[IQ#L-
M%?'GE%OC(E<'GLPGG:>;X`'X;1F-*\!P&Q3)ZY@XTCHI)/B;R`"2-K[`[5WX
M61HIK-L9N>(Y%46.[,8*B'1#V';)&'J'-/N/_4*%5+FEIH\KI12LF8)(S;3N
M#Z(B+1,$PC&,OQYD4&606O)Q(X>IU@U%,W?L<IT#OY<QZ=E..-TAIO*IS,R/
M#C\26]/F`37J:[>JSM%9\TP'*\1D=]-6F:.G!TVJB_*0N]WMCH/D='X*L*+F
M.8:<**L@R(LA@DB<'-/<&PBM&-\/<TR.F;56?'JR>G=U;,[EB8[Y.>0#]RL'
MH^8E0Y-F4E3>&-?:[5!ZU4,>/9>=^RUWP[DCS#=>:\.(7%;)\CO$QM]SJ[7:
M6.+:6EI93$!&.@+BW1<2/?T'DKV1,#/$D//`"Y61GY,F4<3#`MH!<YUT+X%#
M<D\\@`**O_#?.;%3NJ+EC5;'"T;?)$&S-:/>3&2`/FJFKWAW%G-L;K62"\5%
MRI0?RE+72&5KA[@3U;]Q_:HG"\=?FN52T+:^EMC7LDJ9)JEWL1M!V>OW_!1<
MR-Q`BNSV*N@R,N!KW9VD-:+U-OZV#9%>YM5I%K]/PAQFMJ&T-MXK6&JN#SRQ
MP"-NGN]P<)#O[@5G.9XU=<2R">R7B)K*F+3@YAVR1I[.:?,'_J#U7DF/)&+<
M-OS^RGB=7Q,N0QQ.^*KH@@UY@.`L>RAD6J4W!.\36&T7UU\M5-;JZD953U%2
M_P`)E*QS&N`<3W)YM=.G3J0O2LX--J[)57##\QM>334C>::FIVAK]>YNGNV>
MAT#K?DI_A)O^E9__`%#TVZ\7O7!H&ZW-4/J0LPM5NK[K7QT%LHYZRKEWX<,+
M"][M`DZ`[Z`)^Y?BMI:FAK)J.L@DIZB%Y9+%(TM<QP[@@]BM3]%ZS1UG$2*[
M/N=)3R6WFY:61VI:GQ(96GD'GRZV?@I?B+PPH:_++[=W9]CE,^:JFG-++.T2
ML))/(1S?6\E)N(Y\(D;YJF?K\&/U%V)*:`:#=$FR>-AQ5&UAZ*W6O!:NOX9W
M+.65\#*:@J!`ZG+#SO.XQL'M_G!^Q5_';:^\Y!;K/%*V*2NJHJ9KW#8:7O#0
M3\!M9S&X58YX779F0R!Y:[Y"0[T(%G]%PHK!Q"Q>HP[*JFP5-5%52P-8XRQM
M(:>9H=V/S5ZN_`?)J&ZMI_I*V^H"$2SW&=YA@BVXCE.]DGIOH-=1O2FW'D<2
M`..5FEZQA1,C>^0`/%M]1M_D;<K)45KS[#1B]5004U]MM\]<:[E=0/Y^5P('
M*=;ZG?16JCX.MHJ""HS3,K1B\U0WGBI9M23:_P!(%S=?<3I>C'D+BT#A>2=9
MPHXFRN?L[C8DFN:`%[>RRI%?^('#"X8S98LAM]UHK]8I'!OKM&>C"3H<P!(`
M)Z;!/7H==-T!5R1NC-."TXF9!F1^)`ZQQ]?(@[@^A17'_!;Q!_\`A2X_[H_Z
MJG+Z"]*#*,CL68VRGLU\N%OA?;P][*>=S&N=XCQL@'OH!70Q,<QSWWM7'JN=
MU+-RHLJ'&Q@VWAQMU_TUY>ZQ&_X[?K!(UEZL]=;R_P"H9X7,#OD3T/W*+7T%
MP1R*Y\1;7?\`#LPF^E*04?BQ5$S07Q'?+];S()#@3U&BLPX;\/;CG%!>ZFW5
M44<EJB8_P7,+G3EP>0UNO/\`)Z^\+U^/>DQ;ZK]]N5#'ZSX?C,SJ88M-D66D
M.^4C:QOL1V5,1:A'P9N-/1N??,GQZTUHC,@H9JH.F[;T0.F_EM5WAOP_O6;S
MU#J)\%';Z4;JJVI.HX^F]?$ZZZ\O,A0./*"&UN5K;UG!=&^42#2WD]M^*/>^
MU6JBBUUO!FWW-DL&*\1+'>[E&TN]4;IA=KOHA[M_/6OB%E5RHJNVW">@KZ=]
M/54\ACEB>-%K@=$*,D+X]W!68?4\7-);"ZR.000?R(!KU7.BT['N$,\M@I[[
MEN2V[%:.J`-,*L!TL@(V#RES=;'76]^\!=<_!=TEFN=ZL^9V2[6ZAIGS^)3D
MN<[E:YQ:0"0TZ'O*L&),1=+*[^(.G-<6F3@U=&KXJZJ_JLF16OA=A-5GN03V
M>DKH:.2&E=4E\K2X$![&ZZ>?MC]BLMNX,7CP629!?['CSI>L,%;4@3/'D>7R
M_;OW@*#,>1XU-&RNR>L86-(8I9`'"MM[W]._T67HK_AW#B"]LNE1<<MM-FHK
M=5NI73U#AJ5P\V[+01]ZGAP8I+K2SG#L]LV05D+.=U*T"-Q'W/=\@3H?%2;B
MRN%@?J%7-U[`A>62/JN=G4+\S5#ZE9$BDK-9ZFXY118^\^JU-36QT9\1I_)O
M<\,ZCX$]?DM)KN"-39Z^49'EMEL]M:0V*KG?[4Y(!(9&2-ZWH[([=M*+())!
M;0K\KJN)B.#)GT2+`W)(]*!OV&ZR1%K%WX'7TMH*K%;K09';JQW**J%PC;'W
MVYWM$<O0]02=C6MZ71;^"]MN$KK=1<2+!4WMK3_B4(#P7`=1S!^^GG[/3W*?
MX.:ZT_;_`&5D/\1]-#=7B[>SB1[BK;]:6/JWY%P]O-GQ"CRR.KMUSM%40!/0
MR/=X1/0<X<QI;UV/@1HZZ;KE[ME;9KO56JXPF&KI9712LWO3@?(^8]Q6J^C9
M>X:NNN/#Z\?EK5>J>3PXW'ZLH;[6O=MH)W[V-7D$;7O\-VQ/'NI]5RYL?&&7
M`=36[N'FWO1\P-PL>1264VB>P9)<;+4'FDHJA\)=K7,&G0=]XT?O4:J""#17
M5CD;(P/:;!W"(B+Q31$1$1$1$1<-_P#S%</U67^`KN7#?_S%</U67^`JR+YQ
M[K+F_P`M)_:?LNY$15K4B(B(B(B(M"]'*)DO&*R"0`AOCO`/O$$FO^JAN+LT
ME1Q0R5\I)<+E,P;]S7%H_<`OYPGO46/\1K'=IWAD,54&2O/9K'@L<?N#B58/
M2/L4EFXHU]1R$4]R#:N%VNAV-/\`OY@[]H6OG%V[._9?/$Z.N@N_JBH>X=9'
MY$%2?HH5$4/%"6.0@.GMLL<>_-P=&[_@URS3)Z2>AR2YT54TMG@JY8Y`>^P\
M@K^XS>:['K_17JVO#*JDE$C-]C[VGX$;!^!6P9#_`(,>*4C;V<A9B-_D8!5Q
M53`8I"!K>R6@GIKF!!UW;M>L`EA#`=P?SM0R'.Z?U%V4]I,<C0"0"=):35@;
MT0>1W6'+8?1+I*>HXCU=1*UKI*:VR/AW_5<7L:2/N<1]Z_E)@/"^Q2"LR7B3
M1W6%G7U6V-#G2?`N8YYT?D/F%7<6SB@Q7BQ/DEDH7QV62:2/U4`-=ZLX]@-Z
MV--/?N.Z]B9X$C7R$<^=J/4,K_5L*?'Q&NLM-$M+03Y"ZLG\E3;W755SO%9<
M:U[GU-3.^64N/7F)V5L+'NNWHFU$E>XR/M5P:VD<_J0/%8W0/P$KQ\NB\[Y@
MW#;)KK-?;+Q)M=IHZMYFDI*MK6R1.<=N#0Y[3K>]#73WE1G%7+L<IL+M_#O"
MII*JV4DGBU=:X:]8DV3H=!L<Q))UKHT#H%-K#"'N>11!'/-K-/E,ZD[&AQV.
M#F/:XVTC0&\@D@"SP`+M2^`2R0^BUF#XI'QN^D2W;3HZ+:8$?>"1]ZS;A9*^
M'B7C+XW%KC=:9A(]SI6@C]A*O.&W>TT_HV95:9[I117">X<\-(^H:V:1NJ?J
MUA.R/9=U`\C[EG_#R>"ES_':FIFC@@ANM+)++(X-:QHE:2XD]``.NRHR.%Q>
MP^ZOPHG:,^QR]WU^`*U>DJQK>,-V+0`71P$_$^"P?\E9_2^_GE9_L\_B.51]
M(2X4%TXJ7&LME=35M,^*`-FIY6R,=J)H.G-)!T58/2FO%IO.5VJ:T72AN,4=
M"6O?2U#96M=XCCHEI.BIR.%3>X^Y63"C>).F6#M&Z_3X6<K\<!?YC<2_L-WX
M4Z[?1MU;\:SS(X&@U]!;/\7=KJWV)7G]I8S]BA^"ETMEOPWB%!7W&CI)JNS.
MCIHYYVL=,_PYARL!.W'9'0>\+AX%9I;L4OM;1WUKC9;O!ZO5D-)Y._*X@=2-
M.<#KK[6_)>PO:TQ$GS_=>=1Q9IFY[8VDDF,UY@!I('GL"/T6>RR22ROEE>Y\
MCW%SG..RXGN2?>MHX_?HPX:?9@_`@7%6<..'-+.^ODXJ6]UJ!YVPPL;)5<O]
MG37$[^/+]REO27=0#"L`9;/%;1&C>:5LW^4$7AP<O-\=:W\5%L+HX9-7IW]5
M;/U&',ZCAB$'8NY:0/D.VX&_LN;+OZ*&*_:9_CJ55_1Q_3-8?_J/_+RJT8)=
M\5RWA`.'N0WV&Q5E%4&:CJ:C0C<"YS@220/Z[VD$CH01M2/#&S\-L(SFWU-7
MG=+=;H?%$,D!;'1TP,3P722%Q:=C;1U[N'3S5@9KDCD!%#3W\EB?D?AL+-Q'
ML=K<92`&DV'60;JJWWW5-KOZ2[?_`/:H_P#S+5[^D]654W%:MI9JF62"GAA$
M,3GDMCW&TGE'8;)VH^LN%`[TA&W1M=3&@_E+'-ZT)6^%X8J`>?GWKEUUWO6D
M](2X4%TXJ7&LME=35M,^*`-FIY6R,=J)H.G-)!T53(X>"\7_`%?Y72Q(G?ZA
MC.+>(?R-M_51O##`[GG=UGIJ.>&CI*5@DJZN4;;$T[UT\R='IL=CU"N'\F>"
MUGJ`VLX@W:KJH7`\U!!IO,/-KO#</V.*X^`F56*TQY!C61U+J*AOU+X'K8[1
M.Y7M.SY;#SU[`CKW7I4X%PYL4AKKSQ*I+K1LZMI+7&U\\WN;L/<&??\`M"E$
MQHC#F@$][/'ZA5YV3,[-DAFD>Q@K0&-O58WWTN[[5LI/TO&@9W:G:&S;`"?_
M`)LG_5?WCS^B?AG]F,_`A7%Z4MWM-YS*V3VBZ45QA9;PQ\E+4-E:UWB/.B6D
M@'1'13+:G#^)/"W'K+<,LHL=N]CC$.JPAK'M#0SH7%H.PUIV#L$$:5LE/EE:
M#N>/T7.PP_&P>GSR-.EA.K8DBPX"P-ZM43T?OTP8_P#[67\%ZLV#4E/6>E/6
M,J&M<V.\U\S6GS>PRN;^P@'[EP8116##^-]B9%E5ON5OA:Y\UP#FQ0,<8Y!R
M\Q<1TZ==^:@+GDC['QGN63VJ2*I;!>JB>)S'@LFC,KM@.'<.:2-CR*I81&QN
MKL[_``NIE1OS<F8P7\<%`D$;DNKFJ7CQCKJJX<4<BEJWN<Z*OE@9S'LR-Q8T
M#X:`6@<(GNNG`?/;57.,E)1PNJ:<.ZAC_#<_0]WM1M/S/Q7IE5BX<\1KF[*+
M7G-#CU55@.K*.X!K2)-:)',YO7IUUL$]=K@S#(\4P[AQ48#AER-XJKA)S7*X
MM;IA'38;Y'8`&AL`;Z[*FUOAR.D<11OOS:R2Y`S,.#"AC<)&EE@M(T:2+-D5
MVH4=[77Z-LDD6%\1I8GNCD9;HW-<TZ+2(ZC1!\BL5@FDAJ&5$;R)&/#VNWU!
M!V"M:X"7>TVW#>($%QNE%1S5=O8RFCJ*AL;IG>'4#3`X@N.W#H/>/>LB5$Q_
MX,=>OW77Z=&1U+-)&Q+/K\`6T^EXQHSRUO`',ZV-!/OU+)_U41;K5P-?;Z=U
M=DF41U1B:9VQQMY6OT.8#\B>F]KK]*6[VF\YE;)[1=**XPLMX8^2EJ&RM:[Q
M'G1+20#HCHLB4\F4-R'D`%9.B8+YNDX[3(Z,@=MCSWL%6'/J?$::\11X77W"
MMMYIVF22M:`\2\SM@::WIR\OEYE5Y%HF"93@^+8\RKJ,6^G,F,CB'51_Q>%N
M_9.CL$^?U=_$+,T"1QLAH7;E?)AP`,:Z5W':SZDF@!ZJQ\&*SB_/''#9XG5M
MC(Y7"\`FEY//E<[VM?!FQ[POSQXI>&4-"'61U'%DW.WQX+6\NI1_;WTY1KKH
M-T?>%2LVXEY?EO-#<;FZ&B=T%'2_DH0/<0.KO[Q*IZT/R&B/PV[^I_;R_-<C
M&Z-,[+&9*1&?^EG?^\_U?_4>ZVOT8AZW:\XM,!'KE7;6B$>9]F5O_%[5BC@6
MN+7`@@Z(/DK%PZRRNPO*J:^431+R;9/"3H31'ZS2?+R(/D0"M)O]AX69[6OO
MMFS*GQBLJ3XE51U\8:P2'N1MS0"3U.BX?)`WQH6M:=VWM[KU\AZ9U"::5I,<
MH:=0!-%HJB!9W%4:I8HK=PPP.YYW=9Z:CGAHZ2E8)*NKE&VQ-.]=/,G1Z;'8
M]0KM08EPDQ245^29M%DCXSS,HK='MDA'DXM<[8^;FCWKBX'Y;CUMFR7';],Z
MWVW((3$RI`T(>CVZ.NVQ)W[`A>1X[6R-$I&_K]U+,ZO+-B2OP6.MM42T[V=]
M(-$D"SQ^:]_Y,\%K/4!M9Q!NU750N!YJ"#3>8>;7>&X?L<5U^EXT#.[4[0V;
M8`3_`/-D_P"JC*G`N'-BD-=>>)5)=:-G5M):XVOGF]S=A[@S[_VA>_I2W>TW
MG,K9/:+I17&%EO#'R4M0V5K7>(\Z):2`=$=%?)M`\$`<<'_R5R<0B3JN/(R1
M\@I]ES:'`V'PM^OT7?QW>\<(^&L8>X,=;F%S=]"13PZ.OO/[57_1EJZFGXNV
MZ&%SA'50SQ3`=BT1N>-_WFM6CYS8,>R#A;P_I;WE,./S-M<1I9)XN:*7\A%S
M!QV`W7L]2?/S4!C9P+A)#5WV'**3*L@D@=%1PT>C%'ON26N<!V&R2#K8`5CX
MR,@2$T!7?T'U67&S(W]'DPF-+I'F0`!IJRYU&ZTT.YO:E&\.Z>&E]*F2FIP&
MPQW.X-8T=FCPYNGW=E1>+?Z3\E^TY_XRO3AUE7T%Q,M^57,OF#:I\E4YHVXB
M0.:]VO,^V3I:%GF&X'D-^N.547$VSTU/6N=4NI7\KI@\C9`!>'=3[V[&_-9P
M/&A(;SJ)^E+KN?\`Z?U)DD]Z3$UM@$_$'&QL#YVO/$?Z*.5?:8_BIEFW#+])
M&,?;%)^,Q7W@OD>-5&#7[A[E%Q;:X+D_QJ>K?]1KR&C1/8:+&$;T#U&PNO%\
M5X;XAE=JN-VX@TEXGCK874L-`&B-C^<<LDLG,6M8TZ<=D=!Y]E(L\3PW-(H`
M`[^JI&2,)V9!(QVI[BYM-)L%H&Q`K:M[(I5_TE_TP73_`&4'X35:/2]K*K^5
M5HH/69?5!0^+X/.>3G,CQS:[;T`-JG>D)<*"Z<5+C66RNIJVF?%`&S4\K9&.
MU$T'3FD@Z*G?2EN]IO.96R>T72BN,++>&/DI:ALK6N\1YT2TD`Z(Z+V1PJ:C
MR1]RO,*%WB=,U-^6-U[<'2S\E`>CW14]?Q=L<=2UKV1ODF:T^;V1N<W]A`/W
M*]<1+7PFNF;7:JO_`!`N\5R]9?'/"**1S82T\OAM/A$$-UH:)[+',0OM7C.3
M4%]H@'3T<PD#2=!X[.:?@02/O6N9/:^&G$JN.1V[,:;&+G4@.K*2X-#6E^M;
M!+FC?O()!]P.U''<#"6``F[H_P#Z%9U>%\?4FY+WO;&6:=3!=&R=QI<:/F!R
M-UTT=ZX68[PTR?&[3F-9<S<J:1T$-113`"?PR&Z/A@#;@SJ?<%@JT^_8?PXQ
MW'ZZ6;.XK[>#"X44%`S\D)/(N<TNV!WZEOWK,%3E.<=(<`*\O_TKH]#B@:)9
M(7/=J-DO%6:JQ\+=O6D7U!Q]HN&M3DU!)FMVO5'6MHP(8Z)@+'1\[NIVQW7>
M_,+Y?6N^E+=[3><RMD]HNE%<866\,?)2U#96M=XCSHEI(!T1T4\=X9#)8!XY
M^JS]7Q7Y'4L4-<YHJ3=O(V;WH\K^W3B#B&+XK7X]PUMM;'-<&^'5W.L/Y0MT
M1[/GO1.NC0-DZWU4AZ-,\U-A_$2IIY7Q316^)\;VG1:X1U!!!]X*Q%:[P$N]
MIMN&\0(+C=**CFJ[>QE-'45#8W3.\.H&F!Q!<=N'0>\>]2QYR^=I=L`#[<%4
M]8Z7'C=,E;$"YSG,))W)^-O/L/H`LDGEEGF?--(^65[BY[WN)<XGN23W*VN\
MO?:?10L[:`EGTI<2*QS.G..>4]?^Z8/N6)+6N&.68S<L#J^'.;5+Z*BDD,U!
M7-&Q`\GFT?=[6SOL>9P.E5BN%N:35@A;NO1/,<,K&EPC>UQ`%FA8V'>K!KT6
M<8G7U5LR>V7"B>YE1!51O86GJ3S#I\CVU\5L?&NS4%1Z1F/4LD;#%<O4G53=
M?7W,Z,@_-K&A<F/X9PWQ*[0Y#>N(ULO,%$\304=$UKGRO:=MY@U[CWUTUKWG
M6U0LXSBMR'B*[+H6&G?#/&^CC=U\)L9!8#\=C9^)*MKP8M+^20:YX6(O/4L\
M38H(:UCFEQ!;9=5`6`357Z*Q>D]<:JKXK5E%,]Q@H8(8H&;Z-#HVO)`^)<?V
M#W*:]'C]'W$K[,;^%4+NS&DP'BK+39/#F=!C5T,#8ZVEK^4;+?,<SF[([;&P
M0!V.U.</;;A]CX>9]28Y?9KU/';'&OK/#Y(2?"FY&QCX>UL[/<=?=H9&3DF2
MQ1OOZ%<?(S8V=%9A:'"1N@$:30(<VR35;GC?>U3O1&_21</L>7\:%95?ZRJK
M[U65=;4RU-1),XODE>7.=U\R5I/HNW:UV?/ZZIN]RH[?`ZU21MEJIVQ-+C+$
M0T%Q`WH$Z^!675[@ZNG<T@M,KB"#T/58I#_R[!ZG]E])B1D=8R7D?TLH_P#V
MM:!P^X8"^XW)E>0WVGL%@8XM;/(WF?*0='E!(`&^GF201I:%P?M?"FW<0K=_
M)K*[S<KP1*V*-\)9"\>$XNYMQCH`"1U[@*!Q"Z8KFG!ZEP&^9##CM?;:DS4\
M]1H12C;R-DD#M(X$;!Z`]>R[N&5FX<X3GEOJZO/J2Z7+\J(7P<L=)3@Q/!=)
M*7%IV-M`![D=%L@8UI86@5M9)WO\_P!E\[U2>>>/*CG?('#4&L:WX2VMB3I-
M@]_B'DJ?6-#?26:&@`?RKC/3XU(7IZ34LC^+MP8^1[FQPP-8TNV&CPVG0]W4
MD_>N.LN%`[TA&W1M=3&@_E+'-ZT)6^%X8J`>?GWKEUUWO6D](2X4%TXJ7&LM
ME=35M,^*`-FIY6R,=J)H.G-)!T5GD(\%X_[O\KL8D;O]0QW$<0_K;?U5QP>H
MG'HJY5RS2,,=>Z-I:X@AI-/L?(\QW\RLUX3ROBXG8T^-Q:3<X&DCW.>`?W$J
M[X?=[3!Z-646F>Z4,5QGN'/%2/J&B:1O^+]6L)YB/9=U`\C[E0.&]1!2<0L>
MJJJ>*""&YT[Y)9'AK&-$C222>@`'FDCA<6_8?=,.(B//!;R]WU^`*R^DBQK.
M,=ZY0!S-IR?GX$:@N$TTE/Q.QI\1(<;G`PZ]SGAI_<2I;T@:^AN?%:ZUMMK:
M:MI7L@Y)J>5LC':A8#IS20=$$+I]'.Q2WKBE;YN0FGMH=63.UT'*-,^_G+?V
M%1(U99#?^K]U;'(V#^'VNEVJ(7?]O'[+\^DA$R+C%>>0`<[8'D#WF%FUG:M/
M%R]19!Q(OEU@>'P25)CB>.SF1@,:1\"&@_>JLJ<AP=*XCS*ZG2(GQ8$$;^0Q
MH/Y!$1%2N@B(B(B(B(BX;_\`F*X?JLO\!7<N&_\`YBN'ZK+_``%61?./=9<W
M^6D_M/V7<B(JUJ1$1$1$1$1:Y/EV,YMPK@LF77/Z.R&S^S;JQ\$DC9VZURN+
M&N(V``=^8:[KU"R-%;'*8[`W!6'-P(\O0YQ(<PV".1^8.Q&Q%;HB(JEN1$1$
M1$1$1$1$1$1$13F#?R:_E'"W+?61:7,>)74^^=KN4\I&O<=>]0:*33I-JN:/
MQ8RRR+%6-B/;U6NLL_`6EF%6[*[_`%T3#S>I^KN:Y_\`HEWA-_<1\U5N+F;C
M-K]!-24AH;50PBGH:<ZVU@\SKH">G0=``!UULTM%:_(+FZ0`!Z+FXW260S"9
M\CI'"P-1!J^:``&_GRB(BH761$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$6H<6LKL%]X>X-:K57^L5EKH6Q5D?@O;X3Q%$W6W-`/5KAT)[
M++T162R&1VHK'@X4>%#X,9)%D[^I)/EYHB(JUL1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$6JT-KX'U]LI):K)+Y:*T0,%3$('2,,G*.;EU&[SWYK]Y/F>'6'
M!*S#.'L5=.VXN!K[E5CE=(W^RT:!Z@:[``$]"3M9.BT_B2!36@>H7%_T5CGA
MTLKW@&PUQ%6-QP`37:R41$69=I$1$1$1$1$1$1%KE)EV,8/PLJ+/B=S^DLDO
M`Y:^K9!)&VG;K6FE[6DZ!(&O,N=TZ!9&BMCE,=EO)6'-P(\T-;*3I:;KL:XO
M;<`[UMZHB(JEN1$1$1$1$1$1$1<-_P#S%</U67^`KN7#?_S%</U67^`JR+YQ
M[K+F_P`M)_:?LNY$15K4B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(BX;_`/F*X?JLO\!7<N&__F*X?JLO\!5D7SCW67-_EI/[3]EW
M(B*M:D1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$7#
M?_S%</U67^`KN7#?_P`Q7#]5E_@*LB^<>ZRYO\M)_:?LNY$15K4B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(BX;_^8KA^JR_P%=RX
M;_\`F*X?JLO\!5D7SCW67-_EI/[3]EW(B*M:D1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$7#?_P`Q7#]5E_@*[EPW_P#,5P_59?X"
MK(OG'NLN;_+2?VG[+N1$5:U(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(N&__`)BN'ZK+_`5W+AO_`.8KA^JR_P`!5D7SCW67-_EI
M/[3]EW(B*M:D1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$7#?_`,Q7#]5E_@*[EPW_`/,5P_59?X"K(OG'NLN;_+2?VG[+N1$5:U(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(N&_\`YBN'
MZK+_``%=RX;_`/F*X?JLO\!5D7SCW67-_EI/[3]EW(B*M:D1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$7#?\`\Q7#]5E_@*[EPW_\
MQ7#]5E_@*LB^<>ZRYO\`+2?VG[+N1$5:U(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(
MB(B(B(B(B(B(B(B(B(B(B(B(B(N&_P#YBN'ZK+_`5W+AO_YBN'ZK+_`59%\X
M]UES?Y:3^T_9=R(BK6I$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$
M1$1$1$1$1$1%PW_\Q7#]5E_@*[EPW_\`,5P_59?X"K(OG'NLN;_+2?VG[+N1
M$5:U(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B
M(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(B(N&_
M_F*X?JLO\!7<N&__`)BN'ZK+_`59%\X]UES?Y:3^T_9=R(BK6I$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1
M$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1$1%PW_\`,5P_59?X"NY<
9-_\`S%</U67^`JR+YQ[K+F_RTG]I^R__V0``
`
end
End_JPEG
    my $frame;
    foreach (split "\n", $uu_frame) {
        last if /^end/;
        next if /[a-z]/;
        next unless int((((ord() - 32) & 077) + 2) / 3) == int(length() / 4);
        $frame .= unpack "u", $_;
    }
    open FJH, ">$tmp_dir/Decode_Failure.jpg" or die "Could not open $tmp_dir/Decode_Failure.jpg: $!\n";
    binmode FJH;
    print FJH $frame;
    close FJH;
}
{

package      VDRRECORDING;
require      Exporter;

use strict;
use warnings;


our @ISA       = qw(Exporter);
our @EXPORT    = qw(
        );

our $VERSION   = "0.0.7";                # Version number


sub new {
    my $invocant = shift;
    my $class   = ref($invocant) || $invocant;
    my $self = {
        files               => [],
        file_handles       => [],
        file_sizes           => [],
        marks_file           => "",
        chapter_marks_file=> "",
        summary_file       => "",
        movie_size        => 0,
        movie_size_cut    => 0,
        movie_length      => 0,
        movie_length_cut  => 0,
        index_file           => "",
        IFH               => "",
        marks               => [],
        chapter_marks     => [],
        EOF               => 0,
        filepos           => 0,
        total_input       => 0,
        DIR               => "",              # A dirhandle to the recording's DIR
        total_frames       => 0,
        frame_pos           => 0,          # Inidicates at which frame the last operation started, usefull for relative Operations
        current_pos       => 0,  
        current_frame      => 0,
        split_output      => 0,
        cut_info            => {
                start           => 0,
                start_file      => 1,
                start_offset    => 0,
                chunk_length    => 0,
                bytes_written   => 0,
                total_bytes     => 0,
                current_chunk   => "",
                current_offset  => "",
                
                        }, 
        debug               => 0,
        create_index        => "create_indexvdr.pl",
        debug_log          =>"/tmp/vdrrecording.debug",
        @_,
    };
    return bless $self, $class;
}
sub verify_chapter_marks {
    my $self = shift;
    my @marks = @{ $self->{marks} };
    my @chapter_marks = @{  $self->{chapter_marks} };
    my @new_marks;
    
    $self->VR_dprint("verifying Chapter marks\n");
    
    for (my $i = 0; $i < scalar(@chapter_marks); $i++)  {
        my $cf = $self->mark2frames($chapter_marks[$i]);
        print "checking mark $chapter_marks[$i] at frame $cf \n" if $self->{debug};
        for (my $j = 0; $j < scalar(@marks); $j +=2) {
            my $ok = 0;
            my $f = $self->mark2frames($marks[$j]);
            print "comparing with mark $marks[$j] at frame $f \n" if $self->{debug};
            if ($f <= $cf) {
                print "$f < $cf \n";
                my $ef = $self->mark2frames($marks[$j+1]);
                print "Ok now $ef should be bigger than $cf\n" if $self->{debug};
                if ($ef >= $cf) {
                    print "$ef > $cf \n" if $self->{debug};
                    push @new_marks, $chapter_marks[$i];
                    print "Mark $cf verified...\n" if $self->{debug};
                } 
            } else {
                print "Mark $cf will be removed\n" if $self->{debug};
                last;
            }
        }
    }
    return @new_marks;
}
sub generate_chapters {
   
    my $self = shift;
    my $chapter_interval = shift;
    my $cut = shift || 0;
    
    my @marks = @{ $self->{marks} };
    my @auto_chapters;
    my @frame_marks;
    my $offset = 0;
    my $end = 0;
    my $current_start;
    my $current_end;
    
    
    if (! $cut) {
        for (my $i = 0; $i < $self->{total_frames}; $i += $chapter_interval * 25) {
            push @auto_chapters, $i;             # Also a chapter start at 0...
        }
    } 
    
    else {
    
        for (my $i = 0; $i < scalar(@marks); $i+=2)  {
            my $startf = $self->mark2frames($marks[$i]);
            my $endf;
            if ($marks[$i+1]) {
                $endf = $self->mark2frames($marks[$i+1]);
            } else {
                $endf = $self->{total_frames};
            }
            push @frame_marks, "$startf TO $endf";
            #print "$startf TO $endf\n";
        }
        
        my $counter = 0;
        ($current_start, $current_end) = split / TO /, $frame_marks[$counter];
        $offset = $current_start;
        push @auto_chapters, $offset;           # Always include a chapter start at 0
        #print "starting at $offset to $current_end\n" i;
        while (! $end) {
            if (($offset + $chapter_interval * 25) <= $current_end) {
                $offset += $chapter_interval * 25;
                push @auto_chapters, $offset;
                next;
            } else {
                if (!$frame_marks[$counter+1]) {
                    $end = 1;
                    next;
                }
                my $rest = $current_end - $offset;
                $counter++;
                ($current_start, $current_end) = split / TO /, $frame_marks[$counter];
                $offset = $current_start - $rest;
            }   
        }
    }
    
    foreach(@auto_chapters) {
        my ($real_frame, undef) = $self->find_GOP_offset($_);
        push @{$self->{chapter_marks}}, $self->frames2mark($real_frame);
        $self->VR_dprint("pushed $real_frame as ${$self->{chapter_marks}}[-1]\n") if $self->{debug};
    }
}

sub get_cut_chapter_marks {
    
    my $self = shift;
    
    my @marks = @{ $self->{marks} };
    my @chapter_marks = @{  $self->{chapter_marks} };
    my @new_marks;
    
    
    for (my $i = 0; $i < scalar(@chapter_marks); $i++)  {
        my $cf = $self->mark2frames($chapter_marks[$i]);
        $self->VR_dprint("checking mark $chapter_marks[$i] at frame $cf \n") if $self->{debug};
        my $offset = 0;
        for (my $j = 0; $j < scalar(@marks); $j +=2) {
            my $ok = 0;
            my $f = $self->mark2frames($marks[$j]);
            $self->VR_dprint("Comparing with mark $marks[$j] at frame $f \n") if $self->{debug};
            if ($f <= $cf) {
                $self->VR_dprint("$f < $cf \n") if $self->{debug};
                my $ef = $self->mark2frames($marks[$j+1]);
                $self->VR_dprint("Ok now $ef should be bigger than $cf\n") if $self->{debug};
                if ($ef >= $cf) {
                    $self->VR_dprint("$ef > $cf \n") if $self->{debug};
                    push @new_marks, $offset + $cf - $f;
                    $self->VR_dprint("Mark $cf verified...\n") if $self->{debug};
                    last;
                } 
                $offset += ($ef -$f);
                $self->VR_dprint("offset is now $offset frames\n") if $self->{debug};
                
            } else {
                $self->VR_dprint( "Mark $cf will be removed\n") if $self->{debug};
                last;
            }
        }
    }
    return @new_marks;
}

sub get_cut_marks {
    
    my $self = shift;
    
    my @marks = @{ $self->{marks} };
    
    my @new_marks;
    my $offset = 0;
    
    for (my $j = 0; $j < scalar(@marks); $j +=2) {
        my $f = ($self->mark2frames($marks[$j]));
        $self->VR_dprint("Comparing with mark $marks[$j] at frame $f \n") if $self->{debug};
        my $ef = $self->mark2frames($marks[$j+1]);
        my $chunk_length = $ef - $f;
        push @new_marks, $offset;
        push @new_marks, $offset + $chunk_length - 10;
        $offset += $chunk_length; 
    }
    return @new_marks;
}

sub get_chapter_shots{
 
    my $self   = shift;
    my $subdir = shift;
    my $chapter_length = shift;
    my $help_prog = shift;
    
    my $transcode;
    my $jpeg2yuv;
    
    if ($help_prog =~ /transcode/) {
        $transcode = 1;
    } elsif ($help_prog =~ /jpeg2yuv/) {
        $jpeg2yuv = 1;
    } else {
        die "Need to know which helper program to use...\n";
    }
    
    my @chapter_list;
        
    # First, extract the Video Bits and transform them into JPEGs
    #
    if (! -d "$subdir") {
        mkdir "$subdir" or die "Could not create $subdir: $!\n"; 
    }
    chdir "$subdir/" or die "Could not change to $subdir: $!\n";
    for (my $i = 0; $i < scalar(@{ $self->{chapter_marks} }); $i++) {
        #print "Chunk start mark: ${ $self->{marks} }[$i]\n";
        my $sec = $self->HMS2sec(${ $self->{chapter_marks} }[$i]);
        #print "in sec: $sec\n";
        my $dirname = sprintf("chapter_%04u", $i+1);
        #print "Trying to create $dirname\n";
        if (! -d  "./$dirname") {
            mkdir "./$dirname" or die "Could not create $dirname: $!\n";
        }
        my ($gop, $gopno) = $self->get_gop_at($sec, int($chapter_length * 2.2) +2);
        #print "Got a Video chunk of length " . length($gop) . "\n";
        open FH, ">./$gopno.mpv" or die "can not open file: $!\n";
        binmode FH;
        print FH $gop;
        close FH;
        my ($cmd, $error);
        if ($transcode) {
            $cmd = "transcode -i $gopno.mpv   -z -k -Z 180x144 -F jpg  -x mpeg2,null -o " . $gopno . "_  -y im,null ";
        } elsif ($jpeg2yuv) {
            $cmd = "ffmpeg -i $gopno.mpv -s 150x120 -t $chapter_length ./$dirname/$gopno" . "_%4d.jpg ";
        } else {
            die "You need either transcode or jpeg2yuv for this script to work!\n";
        }
        
        
        $self->VR_dprint("running: $cmd\n");
        $error = system ($cmd . " &> /dev/null");
        
        #unlink "./$gopno.mpv";
        
        if ($error) {
            #cmd_error($cmd);
            $self->VR_dprint("The follwing command returned the error $error\n: $cmd\n");
        }
    }
    chdir ".." or die "Could not change the dir \n";;
    
}

sub read_chunk{
    my $self = shift;
    my ($start, $end) = @_;
    ($start, my $fileno) = ( split/ : /, $start );
    $fileno--;
    $end = ( split/ : /, $end )[0];
    my $bytes_to_read = $end - $start;
    my $IFH = ${$self->{file_handles}}[$fileno];
    
    
    my $success = (sysseek $IFH, $start, 0);
    #print "seeked to pos $success, $start\n";
    if (!($start == $success)) {
        die "could not seek, dieing..\n";
    }
    my $rbuffer;
    my $rbytes = sysread $IFH, $rbuffer, $bytes_to_read, 0;

        if ($rbytes != $bytes_to_read) {
            if (! ${$self->{file_handles}}[$fileno+1]) {
                print "Something weird happend, GOP not in files\n";
                exit;
            }
            else {
                $IFH = ${$self->{file_handles}}[$fileno+1];
                
                my $helpbuffer;
                my $helprbytes = sysread $IFH, $helpbuffer, ($bytes_to_read - length($rbuffer)), 0;
                $rbuffer .= $helpbuffer;
                $rbytes += $helprbytes;
            }
        }
    return \$rbuffer;
};

sub find_GOP_offset{
    my $self = shift;
    my $frame = int (shift);
    #print "got frame $frame\n";

    if ($self->binidx2frame($frame)) {
        return ($frame, $self->binidx2frame($frame));
    } 
    my $return_frame = 0;
    my $i = 1;
    while ((! $return_frame) && ($i < 10)) {
        if ($self->binidx2frame($frame - $i))  {
            #$self->VR_dprint("Frame $i frames before $frame is an I-Frame. Returning " . ($frame - $i) . " " . $self->binidx2frame($frame - $i) . "\n") if $self->{debug};;
            return ($frame - $i , $self->binidx2frame($frame - $i));
        }
        elsif ($self->binidx2frame($frame + $i)) {
            #$self->VR_dprint(print "Frame $i frames behind $frame is an I-Frame. returning " . ($frame + $i) . " " . $self->binidx2frame($frame + $i) . "\n") if $self->{debug};;
            return ($frame + $i , $self->binidx2frame($frame + $i));
        }
        else {
            #$self->VR_dprint(print "no iframe found in $i frame distance\n"),
            $i++;
        }
    }
    die "could not find a reasonably positioned iframe, exiting\n";
};

sub get_gop_at{
    my $self = shift;
    my $sec_to_get = shift;
    my $gops_to_get = shift || 1;
    (my $current_gop, my $return_frameno) = get_gop($self, $sec_to_get, $gops_to_get);
    return ($current_gop, $return_frameno);
};

sub get_next_gop{
    my $self = shift;
    my $diff = shift;
    if (! $self->{current_frame}) {
        $self->{current_frame} = 1;
    }
    my $now = $self->{current_frame} * 0.04;
    (my $current_gop, $self->{current_frame}) = get_gop($self, $now + $diff, 1);
    return ($current_gop, $self->{current_frame});
};

sub get_file_list {
    my $self = shift;
    my $DIR;
    my $indir = $self->{indir};
    opendir $DIR, $indir or die "Can not open $indir $!\n";
    #print "trying to open $indir\n";
    if (! $DIR) {
        die "did not get a handle back\n";
    }
    my @allfiles =   grep { ! /^\./  } readdir $DIR;
    closedir $DIR;
    my @files =   sort (grep { /\d{3}.vdr$/  } @allfiles);

    $self->{summary_file} = $indir . "summary.vdr" if (-e "$indir/summary.vdr");
    $self->{marks_file} = $indir . "marks.vdr" if (-e "$indir/marks.vdr");
    $self->{index_file} = $indir . "index.vdr" if (-e "$indir/index.vdr");
    $self->{chapter_marks_file} = $indir . "chapter_marks.vdr" if (-e "$indir/chapter_marks.vdr");
    
    foreach (@files) {
        $_ = $indir . $_;
        #print "trying to open $_\n";
        my $DUMMYFH;
        open $DUMMYFH, "$_" or die "Can not open $_: $!\n";
        push @{$self->{file_handles}}, $DUMMYFH;
        my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks)
            = stat $DUMMYFH;
        push @{$self->{file_sizes}},$size;
        $self->{movie_size} += $size;
    }
}

sub read_marks_file {
    my $self = shift;
    if (! $self->{marks_file}) {return;}
    open MFH, "$self->{marks_file}" || die "Could not open marks.vdr";
    @{$self->{marks}} = ();
    my @raw_marks = <MFH>;
    foreach (@raw_marks) {
        chomp $_;
        $_ = (split /\s/, $_)[0];
        if ($_ !~ /^\d{1,2}:\d{1,2}:\d{1,2}\.\d{1,2}$/) {
            $self->VR_dprint("Could not read mark $_, skipping\n");
            next;
        }
        my ($h,$m,$s,$f) = split /[:.]/,$_;
        my $frame = ($h * 3600 + $m * 60 + $s)* 25 + $f;
        my $real_frame = (find_GOP_offset($self, $frame))[0];
        my $real_mark = $self->frames2mark($real_frame);
        if ($real_mark ne "-1") {
            push @{$self->{marks}}, $real_mark;
        }
    }
}


sub read_chapter_marks_file {
    my $self = shift;
    if (! $self->{chapter_marks_file}) {return;}
    #print "Trying to get chapter_marks_file: $self->{chapter_marks_f}\n";
    
    open MFH, "$self->{chapter_marks_file}" or die "Could not open chapter_marks.vdr";
    @{$self->{chapter_marks}} = <MFH>;
    foreach (@{$self->{chapter_marks}}) {
        #print "got chapter_mark $_";
        chomp $_;
        my ($h,$m,$s,$f) = split /[:.]/,$_;
        #print "$h,$m,$s,$f\n";
        my $frame = ($h * 3600 + $m * 60 + $s)* 25 + $f;
        #print "$frame\n";
        my $real_frame = (find_GOP_offset($self, $frame))[0];
        
        $_ = $self->frames2mark($real_frame);
        #print "replaced $frame with $real_frame, result is: $_\n";
    }
}


sub read_index_file{
    my $self = shift;
    
    print "*" x40 ."\n";
    print "READING INDEX FILE, please be patient\n";
    print "*" x40 . "\n";
    
    my $index_file = $self->{index_file};
       open FH,"$index_file" or die "Can not open File: $!\n";
    binmode FH;
    my $buffer;
    my $bytesread = sysread (FH, $buffer, 10000000);
    $self->{binindex} = $buffer;
    close FH;
    my $offset = 0;
    my $I_counter = 0;
    
  
    my $li_frame = 0;
    $offset = 8;
    while ($li_frame == 0) {
        my ($c, $Ptype, $number, $reserved) = unpack ("I C C S", substr($buffer, (length($buffer) - $offset), 8));
        if ($Ptype == 1) {
            $li_frame = int ((length($buffer) - $offset) / 8);
            print "last Iframe found as frame $li_frame \n";
        } else {
            $offset += 8;
        }
        if ($offset >= (length($buffer)) ) {
            die "Something went horribly wrong\n";
        }
    }
    
    $self->{total_frames} = int(length($self->{binindex}) / 8);
    $self->{last_Iframe} = $li_frame;
    
    return ;
}


sub get_gop {
    my $self = shift;
    my $sec = shift;
    my $gops_to_get = shift || 1;
    if ($sec > $self->{total_frames} * 0.04) {
        $sec = ($self->{total_frames} * 0.04) -1;
    }
    if ($sec < 0) {
        $sec = 0;
    }
    
    my $frame_offset = $sec * 25;
    
    my ($frameno, $gop_offset) = find_GOP_offset($self, $frame_offset);
    #$self->VR_dprint("Wanted frame $frame_offset, got $frameno at $gop_offset\n") if $self->{debug};
    
    my $counter = $frameno;
    
    while ($gops_to_get) {
        $counter++;
        if ($counter > $self->{total_frames}) {
            $counter = $self->{last_Iframe};
        }
        if (! $self->binidx2frame($counter)) {
               next;
        }
        else {
            $gops_to_get--;
        }
    }
    my $gop_end = $self->binidx2frame($counter);
    if ($gop_offset eq $gop_end) {
        my $fileno = (split " : ",$gop_offset)[1];
        $gop_end = ${$self->{file_sizes}}[$fileno - 1] . " : $fileno";
        #print "changed gop end to $gop_end\n";
    }
    $self->VR_dprint("Now we should read the range $gop_offset to $gop_end\n") if $self->{debug};
    
    my $raw_chunk = ${ $self->read_chunk ($gop_offset, $gop_end) };
    my $offset = 0;
    my $video_stream = "";
    my $plength = 0;
    my $packet_counter = 0;
    
    while ($offset + $plength < length($raw_chunk))    {
        # analyse the needed info from PES Header
        my ($startcode,  $streamcode, $plength, $notused, $header_length) = unpack ( "H6 H2 n1 n1 C1", substr($raw_chunk, $offset, 9));
        
        if ($startcode ne "000001") {
            print "No 0x000001 found at current packet, found $startcode and $streamcode instead in packet\n"; exit
        }
        if (! ($streamcode =~/^e/)) {
            #print "video stream $streamcode found\n";
            #print "not a video stream\n";
            $offset +=  $plength + 6;
            next;
        }
        if (($offset + $plength) > (length($raw_chunk))) {
            die "Did not get a rawchunk back that ends with a packet...\n";
            return \$video_stream;
        }
        
        my $packet = substr($raw_chunk, $offset, ($plength + 6));
        my $payload = (substr($packet, 9 + $header_length));
        $offset +=  $plength + 6;
        $video_stream .= $payload; #$packet;#
        $packet_counter++;
    }

    $self->VR_dprint ("returning " . length($video_stream) . " bytes of video data\n");
    $raw_chunk = "";
    
    my %GOP_info;
    my $seqstart = pack("H8", "000001b3");
    
    
    
    if ($video_stream !~ /$seqstart(.{8})/s) {
        print "No luck here...\n";
        print "length is " . (length($video_stream)) ."\n";
    }
    
    my $bitview = unpack("B64", $1);
    my $hor_size = "0b" . substr($bitview,0,12);
    my $horizontal_size = oct $hor_size; 
    
    my $ver_size = "0b" . substr($bitview,12,12);
    my $vertical_size = oct $ver_size;
    
    my $asp_ratio = substr($bitview,24,4);
    my $aspect_ratio;
    if ($asp_ratio eq "0001"){
        $aspect_ratio = "1:1";
    }
    elsif ($asp_ratio eq "0010") {
        $aspect_ratio = "4:3";
    }
    elsif ($asp_ratio eq "0011") {
        $aspect_ratio = "16:9";
    }
    elsif ($asp_ratio eq "0111") {
        $aspect_ratio = "2.21:1";
    }
    else {
        $aspect_ratio = "$asp_ratio";
    }
    
    
    my $fps = substr($bitview,28,4);
    if ($fps eq "0001")    {
        $fps = 23.967;
    }
    elsif ($fps eq "0010")    {
        $fps = 24;
    }
    elsif ($fps eq "0011")    {
        $fps = 25;
    }
    elsif ($fps eq "0100")    {
        $fps = 29.97;
    }
    elsif ($fps eq "0101")    {
        $fps = 30.97;
    }
    my $bitrate_value = 400 * ( oct ("0b" . substr($bitview,32, 18)));
    $self->VR_dprint("Got the following infos:\n\taspect ratio: $aspect_ratio\n\tsize: $horizontal_size" . "x$vertical_size\n");
    return $video_stream, $frameno;
}

sub get_movie_chunk    {
    my $self =shift;
    my $sec = shift;
    
    if ($sec > $self->{total_frames} * 0.04) {
        $sec = ($self->{total_frames} * 0.04) -1;
    }
    if ($sec < 0) {
        $sec = 0;
    }
    my $gops_to_get = shift;
    my $frame_offset = $sec * 25;
    
    my ($frameno, $gop_offset) = find_GOP_offset($self, $frame_offset);
    #$self->VR_dprint("IN get_movie_chunk: wanted frame $frame_offset, got $frameno at $gop_offset\n") if $self->{debug};
    
    my $counter = $frameno;
    while ($gops_to_get) {
        $counter++;
        if ($counter > $self->{total_frames}) {
            $counter = $self->{last_Iframe};
        }
        if (! $self->binidx2frame($counter)) {
            next;
        }
        else {
            $gops_to_get--;
        }
    }
    my $gop_end = $self->binidx2frame($counter);
    $self->VR_dprint("Now we should read the range $gop_offset to $gop_end\n") if $self->{debug};
    
    my $raw_chunk = ${ $self->read_chunk($gop_offset, $gop_end) };
    my $offset = 0;
    my $video_stream = "";
    my $plength = 0;
    my $packet_counter = 0;
    
    while ($offset + $plength < length($raw_chunk))    {
        # analyse the needed info from PES Header
        my ($startcode,  $streamcode, $plength, $notused, $header_length) = unpack ( "H6 H2 n1 n1 C1", substr($raw_chunk, $offset, 9));
        
        if ($startcode ne "000001") {
            print "No 0x000001 found at current packet, found $startcode and $streamcode instead in packet\n"; exit
        }
        if (($streamcode =~/^b/)) {
            #print "video stream $streamcode found\n";
            #print "not a video stream\n";
            $offset +=  $plength + 6;
            next;
        }
        if (($offset + $plength) > (length($raw_chunk))) {
            die "Did not get a rawchunk back that ends with a packet...\n";
            return \$video_stream;
        }
        
        my $packet = substr($raw_chunk, $offset, ($plength + 6));
        my $payload = (substr($packet, 9 + $header_length));
        $offset +=  $plength + 6;
        $video_stream .= $payload;
        $packet_counter++;
    }

    print "returning " . length($video_stream) . " bytes of vdr data\n";
    #return $video_stream, $frameno;
    return $raw_chunk, $frameno;
}

sub cut_movie {
    my $self = shift;
    my $out_dir = shift || "./";
    my $split = shift || 0;
    my $mainw = shift || 0;
    my $prog_ind = shift || 0;   
    my $file_size = shift || 2000000000;
   
    my $stepsize = 0;
    
    
    $self->compute_movie_cuts();
    if ($mainw && $prog_ind && $self->{movie_size_cut}) {
        $stepsize = 100 / ($self->{movie_size_cut} / 10000000);
        #print "$mainw and $prog_ind and $stepsize and $self->{movie_size_cut}\n";
    }
  
    
    my @chunks = @{$self->{cut_chunks}};
    print "in do_cut we got @chunks and $self->{split_output} for split\n";
    
    my $start;
    my $total_bytes    = 0;
        
    my $cunk_no = scalar(@chunks);
    my $start_file;
    my $end_file;
    my $start_offset;
    my $end_offset;
    my $buffer;
    my $rbuffer;
    my $offset = 0;
    my $EOF = 0;
    my $current_chunk = 0;
    my $current_file = 0;
    my $bytes_written = 0;
    
    
    
    ($start_offset, $start_file, $end_offset, $end_file) = split/ : | TO /, $chunks[$current_chunk];


    $current_file   = $start_file;
    $offset = $start_offset;
    
    my $IFH = ${$self->{file_handles}}[$start_file - 1];
    
    my $success = (sysseek $IFH, $start_offset, 0);
    print "seek to $success in $start_file\n";
    
    if (! $success) {
        print "could not seek, dieing..\n"; die;
    }
    print "opening | $self->{create_index}  -s $file_size -o \"$out_dir\"\n";
    open OFH, "| $self->{create_index}  -s $file_size -o \"$out_dir\"" or die "Could not open output file\n";
    binmode OFH;
    select((select(OFH), $| = 1)[0]);
    while (! $EOF) {
        my $bytes_to_read = 10000000;
        while ($bytes_to_read ) {
            print "we are now at: " . (sysseek($IFH, 0, 1)) ." in file $IFH\n";
            print "Current Chunk : $chunks[$current_chunk] \n";
            if ($current_file == $end_file) {
                if (($end_offset - $offset ) > 10000000) {
                    print "Enough bytes to read left in this file for $bytes_to_read \n";
                    my $rbytes = sysread $IFH, $rbuffer, $bytes_to_read, 0;
                    print "Read $rbytes\n";
                    if ($rbytes == 0) {
                        die "Holy shit, start_offset is $start_offset, offset is $offset, end_offset is $end_offset, diff is " . ($end_offset - $offset) ." and we are at the end of the file...?\n";
                    }
                    $bytes_to_read  -= $rbytes;
                    $buffer         .= $rbuffer;
                    $offset += $rbytes;
                    print "EE offset is now $offset, end_offset is $end_offset, diff is " . ($end_offset - $offset) ."\n";
                    #print "bytes_to_read is now $bytes_to_read\n";
                }
                else {
                    print "not enough bytes to read left in current chunk or this was the last chunk\n";
                    print "Trying to read " . ($end_offset - $offset) ."\n";
                    my $rbytes = sysread $IFH, $rbuffer, ($end_offset - $offset), 0;
                    $bytes_to_read  -= $rbytes;
                    $offset += $rbytes;
                    $buffer         .= $rbuffer;
                   
                    if ($chunks[$current_chunk + 1]) {
                        # Here we go for additional Audio at the end of a chunk
                        # So we try to red the Audio from the next GOP, and append it to the current buffer.
                        # Thereby we should always have enough Audio in the buffer, and can prevent the
                        # insertion of Silent frames
                        #
                    
                        #my $add_audio = get_audio_from_gop($self);
                    
                        #print "length before " . length($buffer) . "\n" if $self->{debug};
                        #$buffer      .= $add_audio if $add_audio;
                        #$buffer      .= $add_audio if $add_audio;
                        #print "length after " . length($buffer) . "\n" if $self->{debug};
                        #
                        #
                        $current_chunk++;
                        ($start_offset, $start_file, $end_offset, $end_file) = split/ : | TO /, $chunks[$current_chunk];
                        $current_file   = $start_file;
                        $offset = $start_offset;
                        $IFH = ${$self->{file_handles}}[$start_file - 1];
                        #print "$IFH\n";
                        $success = (sysseek $IFH, $start_offset, 0);
                        print "After Switch: $start_file}, $offset}, $end_file, $end_offset\n";
                    } else {
                        print "This was the last chunk!\n";
                        $bytes_to_read = 0;
                        $EOF = 1;
                    }
                }
            }
            else { 
                my $rbytes = sysread $IFH, $rbuffer, $bytes_to_read, 0;
                $bytes_to_read  -= $rbytes;
                $offset += $rbytes;
                $buffer .= $rbuffer;
                
                print "NE offset is now $offset, end_offset is $end_offset, diff is " . ($end_offset - $offset) ." in file $current_file\n";
                if ($bytes_to_read) {
                    print "That was not enough in this file, switching to next file $current_file\n";
                    $IFH = ${$self->{file_handles}}[$current_file];
                    $current_file++;
                    #print "$IFH\n";
                    $success = (sysseek $IFH, 0, 0);
                    $offset = 0;
                    #print "After Switch: $start_file, $start_offset, $end_file, $end_offset\n";
                } 
            }
            
            $bytes_written += length($buffer);
            
            print "in VDRrecording read $bytes_written until now\n";
            if ($mainw && $prog_ind) {
                $$prog_ind += $stepsize;
                print "$prog_ind is now prog $stepsize\n";
                $mainw->update; 
            }
           #print TOF $buffer;
            print OFH $buffer;
            
            $buffer = "";
        }
    }
    close OFH;
    my @new_chap_marks =  $self->get_cut_chapter_marks();
    open OFH, ">$out_dir/chapter_marks.vdr" or die "Could not open chapter_marks.vdr: $!\n";
    foreach (@new_chap_marks) {
        print OFH ($self->frames2mark($_) . "\n");
    }
    close OFH;
    my @new_marks =  $self->get_cut_marks();
    open OFH, ">$out_dir/marks.vdr" or die "Could not open marks.vdr: $!\n";
    foreach (@new_marks) {
        print OFH ($self->frames2mark($_) . "\n");
    }
    close OFH;
}
    
    



sub get_movie_piecewise {
    my $self = shift;
    my $bytes_to_read = shift || 10000000;
    
    my @chunks = @{$self->{cut_chunks}};
    #print "in do_cut we got @chunks and $self->{split_output} for split\n";
    
    my $start;
    my $total_bytes    = 0;
        
    my $cunk_no = scalar(@chunks);
    
            
    my $start_file;
    my $end_file;
    my $start_offset;
    my $end_offset;
    my $buffer;
    my $rbuffer;
    
    if (! $self->{bytes_written}) {
       # foreach (@chunks) {
       #     print "Processing $_\n";
       #     my ($start, $end) = split/ TO /, $_;
       #     print "do_cut: $start, $end\n";
        #}
    }
    
    
    if ($self->{EOF}) {
        return "";
    }

    #print "next round of reading\n";
    ($start_offset, $start_file, $end_offset, $end_file) = split/ : | TO /, $chunks[$self->{current_chunk}];

    if ((! $self->{current_chunk}) && (! $self->{bytes_written})) {
        #print "seems to be the very first read...\n";
        $self->{current_file}   = $start_file;
        $self->{current_offset} = $start_offset;
    }
        #print " Now we move to $self->{current_file}, $self->{current_offset}, $end_file, $end_offset\n";
        
        
    my $IFH = ${$self->{file_handles}}[$self->{current_file} - 1];
    #print "$IFH\n";
    my $success = (sysseek $IFH, $self->{current_offset}, 0);
    #print "seeked to pos $success, $start\n";
    if (!($self->{current_offset} == $success)) {
        print "could not seek, dieing..\n"; die;
    }
    
    while ($bytes_to_read && (! $self->{EOF})) {
        if ($self->{current_file} == $end_file) {
            #print "start file is end file: $self->{current_file}, $self->{current_offset}, $end_file, $end_offset\n";
            if (($end_offset - $self->{current_offset}) > $bytes_to_read) {
                #print "Enough bytes to read left in this file\n";
                my $rbytes = sysread $IFH, $rbuffer, $bytes_to_read, 0;
                #print "Read $rbytes\n";
                $bytes_to_read  -= $rbytes;
                $buffer         .= $rbuffer;
                $self->{current_offset} += $rbytes;
                #print "current offset is now $self->{current_offset}\n";
                #print "bytes_to_read is now $bytes_to_read\n";
            }
            else {
                #print "not enough bytes to read left in current file or end of recording\n";
                #print "Trying to read " . ($end_offset - $self->{current_offset}) ."\n";
                my $rbytes = sysread $IFH, $rbuffer, ($end_offset - $self->{current_offset}), 0;
                $bytes_to_read  -= $rbytes;
                $self->{current_offset} += $rbytes;
                $buffer         .= $rbuffer;
                if ($chunks[$self->{current_chunk} + 1]) {
                    # Here we go for additional Audio at the end of a chunk
                    # So we try to red the Audio from the next GOP, and append it to the current buffer.
                    # Thereby we should always have enough Audio in the buffer, and can prevent the
                    # insertion of Silent frames
                    #
                    
                    my $add_audio = get_audio_from_gop($self);
                    print "length before " . length($buffer) . "\n" if $self->{debug};
                    $buffer      .= $add_audio if $add_audio;
                    $buffer      .= $add_audio if $add_audio;
                    print "length after " . length($buffer) . "\n" if $self->{debug};
                    #
                    #
                    $self->{current_chunk}++;
                    ($start_offset, $start_file, $end_offset, $end_file) = split/ : | TO /, $chunks[$self->{current_chunk}];
                    $self->{current_file}   = $start_file;
                    $self->{current_offset} = $start_offset;
                    $IFH = ${$self->{file_handles}}[$start_file - 1];
                    #print "$IFH\n";
                    $success = (sysseek $IFH, $self->{current_offset}, 0);
                    #print "After Switch: $self->{current_file}, $self->{current_offset}, $end_file, $end_offset\n";
                } else {
                    $self->{EOF} = 1;
                }
            }
        }
        else { 
            my $rbytes = sysread $IFH, $rbuffer, $bytes_to_read, 0;
            $bytes_to_read  -= $rbytes;
            $self->{current_offset} += $rbytes;
            $buffer         .= $rbuffer;
            #print "in the current file could read " . length($buffer) . " bytes\n";
            if ($bytes_to_read) {
                #print "That was not enough, switching to next file\n";
                $IFH = ${$self->{file_handles}}[$self->{current_file}];
                $self->{current_file}++;
                #print "$IFH\n";
                $success = (sysseek $IFH, 0, 0);
                $self->{current_offset} = 0;
                #print "After Switch: $start_file, $start_offset, $end_file, $end_offset\n";
            } 
        }
            
    }
    
    if ($self->{EOF}) {
        # Here we go for additional Audio at the end of a chunk
        # So we try to red the Audio from the next GOP, and append it to the current buffer.
        # Thereby we should always have enough Audio in the buffer, and can prevent the
        # insertion of Silent frames
        #
        my $add_audio = get_audio_from_gop($self);
        $buffer .= $add_audio if $add_audio;
    }


    #print "in VDRrecording read " . length($buffer) . " until now\n";
    $self->{bytes_written} += length($buffer);
    #print TOF $buffer;
    return $buffer;
    #close TOF;
    #die "finished all?\n";
}

sub get_movie_piece {
    
    my $self = shift;
    my $start_frame = shift || 1;         # What is the start point (in Frames)?
    my $frames_to_read = shift || 96;     # How man Frames should be read?
    my $cut_flag    = shift || 0;         # Should the cut marks be observed?
    #my $add_audio   = shift || 0;        # Should the routine add Audio at cutting points?
    #my $ret_format  = shift || "PES";    # For the moment ignored, in the future it should be used to provide PES or ES or Audio Only and things like that

    print "got params: self $self startf $start_frame, frames_to_read: $frames_to_read cut_flag $cut_flag\n";
    
    my $cur_chunk   = 0;
    my $end_frame;
    
    
    my @frame_chunks;
    my @read_chunks;
    
    for (my $i = 0; $i < scalar(@{$self->{marks}}); $i += 2) {
        if ($self->mark2frames(${$self->{marks}}[$i + 1])) {
            push @frame_chunks, $self->mark2frames(${$self->{marks}}[$i]) . " TO " . $self->mark2frames(${$self->{marks}}[$i + 1]);
        }
    }
    
    ($start_frame, my $byte_start) = $self->find_GOP_offset($start_frame);


    # Calculate whether the start frame is within a selected region
    if ($cut_flag && (scalar(@frame_chunks > 0))) {
        print "movie has marks and should be cut    \n";
        my $hit = 0;
        my ($chunk_s, $chunk_e);
        while (! $hit) {
            ($chunk_s, $chunk_e) = split / TO /, $frame_chunks[$cur_chunk];
            if ($chunk_e < $start_frame) {
                if ($frame_chunks[$cur_chunk + 1]) {
                    $cur_chunk += 1;
                    next;
                } else {
                    return undef;
                }
            } else {
                if ($start_frame >= $chunk_s) {
                    $hit = 1; 
                } else {
                    
                    $start_frame = $chunk_s;
                    $hit = 1;
                    print "start frame was out of movie region, correcting to $start_frame\n";
                }
            }
        }
        
        my $i = $start_frame;
        my $j = $frames_to_read;
        my $k = $cur_chunk;
        
        push @read_chunks, $self->binidx2frame($start_frame);
        print "START serach for endframe\n";
        while ($j) {
            if ($chunk_e < $i + $j) {
                print "not enough frames in current chunk left\n";
                $j -= ($chunk_e - $i);
                print "need $j frames in addition\n";
                push @read_chunks, $self->binidx2frame($chunk_e);
                $k++;
                print "switching to next chunk $k\n";
                ($chunk_s, $chunk_e) = split / TO /, $frame_chunks[$k];
                push @read_chunks, $self->binidx2frame($chunk_s);
                print "Added chunk start tor rewad list $chunk_s\n";
                $i = $chunk_s;
            } else {
                print "Enough frames in this chunk left...\n";
                $end_frame = $i + $j;
                $j = 0;
                ($end_frame, undef) = $self->find_GOP_offset($end_frame);
                push @read_chunks, $self->binidx2frame($end_frame);
            }
        }
        print "END serach for endframe\n";
    } else { # No Cut Flag!!!
        ($end_frame, undef) = $self->find_GOP_offset($start_frame + $frames_to_read);
        push @read_chunks, $self->binidx2frame($start_frame);
        push @read_chunks, $self->binidx2frame($end_frame);
    }
    print "\ngoing to show the movie from frame $start_frame to $end_frame, and that is in chunk $cur_chunk\n";
    
    print "\n" . (join " DADA ",@read_chunks) . "\n";
    
    my $buffer;
    my $rbuffer;
    my $hbuffer;
    
    
    for (my $i = 0; $i < scalar(@read_chunks); $i += 2) {
        my ($soff, $sfile) = split / : /, $read_chunks[$i];
        my ($eoff, $efile) = split / : /, $read_chunks[$i + 1];
        my $IFH = ${$self->{file_handles}}[$sfile  - 1];
        
        
        
        my $success = (sysseek $IFH, $soff, 0);
        if (!($soff == $success)) {
            die "could not seek, dieing..\n";
        }
        while ($sfile < $efile) {
            my $bytes_to_read = ${$self->{file_sizes}}[$sfile - 1] - $soff;
            my $rbytes = sysread $IFH, $hbuffer, $bytes_to_read, 0;
            $rbuffer .= $hbuffer;
            $sfile++;
            $soff = 0;
        }
        
        my $rbytes = sysread $IFH, $hbuffer, ($eoff - $soff), 0;
        $rbuffer .= $hbuffer;
    }

    print "retunrning a buffer of length " . length($rbuffer) . " and frame number  $end_frame\n";

    return ($rbuffer, $end_frame);
   
    
    
}

sub get_audio_from_gop {
    my $self = shift;
    print "current chunk is $self->{current_chunk}\n" if $self->{debug};
    print "In marks that means ${$self->{marks}}[$self->{current_chunk}*2] and ${$self->{marks}}[$self->{current_chunk}*2+1]\n" if $self->{debug};;
    print "$self->{bytes_written} bytes written for ${$self->{cut_chunks}}[$self->{current_chunk}]\n"if $self->{debug};;
    
    my $search_frame = mark2frames($self, ${$self->{marks}}[$self->{current_chunk}*2+1]);
    
    #Check whether the next chunk is just behind, then we skip additional Audio
    if (${$self->{marks}}[$self->{current_chunk} * 2 + 2]) {
        my $next_frame = mark2frames($self, ${$self->{marks}}[$self->{current_chunk}*2 + 2]);
        if (($next_frame - $search_frame) < 36) {
            return;
        }
    }
 
    print "searching for additional GOP starting at $search_frame\n" if $self->{debug};
    my $counter = 0;
 
    if (! $self->binidx2frame($search_frame)) {
        print "did not find additional GOP at end of this chunk\n";
        return;
    }
    
    my ($frameno, $gop_offset) = find_GOP_offset($self, $search_frame);
    #print "wanted frame $search_frame, got $frameno at $gop_offset\n" if $self->{debug};
    
        
    $self->VR_dprint("Next GOP should start at frame $search_frame\n") if $self->{debug};
       
    my $gops_to_get = 2;
    
    $counter = $search_frame;
    
    while ($gops_to_get) {
        $counter++;
        if ($counter > $self->{total_frames}) {
            $counter = $self->{last_Iframe};
        }
        if (! $self->binidx2frame($counter)) {
            next;
        }
        else {
            $gops_to_get--;
        }
    }
    my $gop_end = $self->binidx2frame($counter);
    
    print "Now we should read the range $gop_offset to $gop_end\n" if $self->{debug};
    
    my $raw_chunk = ${ read_chunk ($self, $gop_offset, $gop_end) };
    my $offset = 0;
    my $audio_buffer = "";
    my $plength = 0;
    my $packet_counter = 0;
    
    while ($offset + $plength < length($raw_chunk))    {
        # analyse the needed info from PES Header
        my ($startcode,  $streamcode, $plength, $notused, $header_length) = unpack ( "H6 H2 n1 n1 C1", substr($raw_chunk, $offset, 9));
        
        if ($startcode ne "000001") {
            print "No 0x000001 found at current packet, found $startcode and $streamcode instead in packet\n"; exit
        }
        if ($streamcode =~/^e/) {
            #print "video stream $streamcode found\n";
            $offset +=  $plength + 6;
            next;
        }
        if (($offset + $plength) > (length($raw_chunk))) {
            die "Did not get a rawchunk back that ends with a packet...\n";
            return \$audio_buffer;
        }
        
        my $packet = substr($raw_chunk, $offset, ($plength + 6));
        my $payload = (substr($packet, 9 + $header_length));
        $offset +=  $plength + 6;
        #print "adding a packet for stream $streamcode\n";
        $audio_buffer .= $packet;
        $packet_counter++;
    }

    print "returning " . length($audio_buffer) . " bytes of audio packet data\n" if $self->{debug};
    return $audio_buffer;
}

sub set_movie_position {
    my $self = shift;
    my $frameno = shift || 1;
    my $return_no = 1;
    my $chunk_no  = 1;
    my @marks = @{$self->{marks}};
    if (! @marks) {
        push @marks,"0:00:00.1";
    }
    
    if ($frameno < $self->mark2frames($marks[0])) {
        $return_no =  $self->mark2frames($marks[0]);
    }
    if ($frameno > $self->mark2frames($marks[-1])) {
        $return_no = $self->mark2frames($marks[0]);
    }
        
    for (my $j = 0; $j < scalar(@marks); $j +=2) {
        my $ok = 0;
        my $sf = $self->mark2frames($marks[$j]);
        print "comparing with mark $marks[$j] at frame $sf \n";
        if ($sf <= $frameno) {
            my $ef = $self->mark2frames($marks[$j+1]);
            if ($ef >= $frameno) {
                $return_no = $frameno;
                $chunk_no = ($j + 2) / 2;
            }
        } 
    }
    
    ($return_no, my $gop_offset) = $self->find_GOP_offset($return_no);
    my ($offset, $current_file) =  split( / : /,$gop_offset);
    
    $self->{current_chunk}  = $chunk_no - 1;
    $self->{current_file}   = $current_file;
    $self->{current_offset} = $offset;
}


sub compute_movie_cuts {
    my $self = shift;
    
    my @chunks;
    my @marks                  = @{$self->{marks}};
    my @file_sizes             = @{$self->{file_sizes}};
    my $total_frames_cut       = 0;
    $self->{movie_size_cut} = 0;
    
    
    
    if (scalar(@marks) < 2) {
        print "no cutting without marks...\n";
        return -1;
    }
    
    # Here we check whether the marks are just remains from an already cut movie
    # Criteria: First and last I-Frame are marked, and the diff between the marks inside are smaller than 20
    
    my $is_cut = 1;
    if (($marks[0] eq "0:00:00.01") && (mark2frames($self, $marks[-1]) == $self->{last_Iframe})) {
        for (my $i = 1; $i < scalar(@marks) - 2; $i += 2) {
            if ((mark2frames($self,$marks[$i+1]) - mark2frames($self,$marks[$i])) < 20) {
                next;
            } else {
                $is_cut = 0; 
                last;
            }
        }
    } else {
        $is_cut = 0;
    }    
    
    if ($is_cut) {
        print "CAUTION: You specified the -cut Option, but the marks look like remains of an already cut movie!\nI will process the full movie!\n";
        my $startframe = mark2frames($self, $marks[0]);
        my $startpos   = $self->binidx2frame($startframe);
        my $endframe   = mark2frames($self, $marks[-1]);
        my $endpos     = $self->binidx2frame($endframe);
        if ((! $startpos) || (! $endpos)) {
            print "Mark does not mark an Iframe, cutting stopped!\n";
            return -1;
        }
        $total_frames_cut += $endframe - $startframe;
        push @chunks, "$startpos TO $endpos";
    } else {
        for (my $i = 0; $i < (scalar(@marks)); $i += 2) {
            if ($marks[$i+1]) {
                my $startframe = mark2frames($self, $marks[$i]);
                my $startpos   = $self->binidx2frame($startframe);
                my $endframe   = mark2frames($self, $marks[$i + 1]);
                my $endpos     = $self->binidx2frame($endframe);
                if ((! $startpos) || (! $endpos)) {
                    print "Mark does not mark an Iframe, cutting stopped!\n";
                    return -1;
                }
                $total_frames_cut += $endframe - $startframe;
                push @chunks, "$startpos TO $endpos";
            }
        }
    }
        
    @{$self->{cut_chunks}} = @chunks;
    foreach (@chunks) {
        my ($start_offset, $start_file, $end_offset, $end_file) = split/ : | TO /, $_;
        if ($start_file == $end_file) {
            $self->{movie_size_cut} += $end_offset - $start_offset;
        } else {
            while ($start_file < $end_file) {
                $self->{movie_size_cut} += $self->{file_sizes}[$start_file -1] - $start_offset;
                $start_offset = 0;
                $start_file++;
            }
            
            $self->{movie_size_cut} += $end_offset;
            
        }
    }
    $self->{movie_length_cut} = HMS2sec($self, frames2mark($self, $total_frames_cut));
    
}


sub frames2mark {
    my $self = shift;
    my $now = shift;
    if (! $now) {
        $now = $self->{current_frame};
    }
    $now++;                     # VDR start numbering the frames with 1, I start with 0
    my $rest = $now  % 25;
    my $full_secs = int ($now  * 0.04);
    
    my $h = int($full_secs/3600);
    my $min = int(($full_secs - (3600 * $h)) / 60);
    my $sec = $full_secs - (3600 * $h + 60 * $min);
    return sprintf "%1u:%02u:%02u.%02u", $h, $min, $sec, $rest;
}

sub mark2frames{
    my $self = shift;
    my $mark = shift;
    my($h, $m, $s, $f) = split /[:.]/, $mark;
    my $frame = (3600 * $h + 60 * $m + $s) * 25 + $f - 1; # VDR start numbering the frames with 1, I start with 0
    return $frame;
}


sub init_recording {
    my $self = shift;
    my $indir = shift;
    if (! $indir) {
        print "Did not receive a dir!\n";
        return;
    }
    $self->{indir} = $indir;
    foreach (@{$self->{file_handles}}) {close $_};
    @{$self->{file_handles}} = ();
    @{$self->{files}} = ();
    @{$self->{marks}} = ();
    @{$self->{file_sizes}} = ();
    @{$self->{cut_chunks}} = ();
    $self->{total_input} = 0;
    $self->{EOF} = 0;
    $self->{filepos} = 0;
    $self->{total_frames} = 0;
    $self->{current_chunk} = 0;
    $self->{bytes_read_for_chunk} = 0;
    $self->{binindex} = 0;
    
    # check the files in the Dir 
    get_file_list($self);
    # THIS SCRIPT NEEDS A VALID index.vdr
    read_index_file($self);
    # If an exisiting marks file is found, it is read
    read_marks_file($self);
    # If an exisiting chapter_marks file is found, it is read
    read_chapter_marks_file($self);

    
    $self->{frameno} = 1;
    $self->{movie_length} = frames2mark($self, $self->{total_frames});

    $self->{current_pos} = frames2mark($self, $self->{frame_pos});
    $self->{mark_no} = scalar(@{$self->{marks}});
    $self->compute_movie_cuts;
}
sub HMS2sec {
    my $self = shift;
    my $HMS = shift;
      
    my ($h, $m, $s, $f) = split /:|\./, $HMS;
    return ($h * 3600 + $m * 60 + $s + $f * 0.04);
}

sub VR_dprint {
    my $self = shift;
   # print "IN VR_dprint \n";
    
    if ((! $self->{debug}) || (! $self->{debug_log})) {
        return
    }
    my $msg = shift;
    print "$msg";
    open DFH, ">>$self->{debug_log}" or die "Can not open debug.log: $!\n";
    binmode DFH;
    print "VDRRECORDING: $msg";
    print DFH "VDRRECORDING: $msg";
    close DFH;
}

sub frames2HMS {
    my $self = shift;
    my $frames = shift;
    
    my $total_sec = int($frames * 0.04 + 0.5);
    
    my ($h, $m, $s);

    $h = int ($total_sec / 3600);
    $m = int(($total_sec - $h *3600) / 60);
    $s = int($total_sec - $h *3600 - $m * 60);
    
    return sprintf("%02u:%02u:%02u", $h, $m, $s);
}

sub binidx2frame {
    my $self = shift;
    my $frame = int (shift );
  
   
    #print "bindex has a length of " . length($self->{binindex}) .",searching for frame $frame\n";
    if ($frame > $self->{last_Iframe}) {
        #print "Got only $self->{total_frames}, you wanted $frame and last IFrame is $self->{last_Iframe}...\n";
        $frame = $self->{last_Iframe};
    }
    my $offset = 8 * $frame;
    #print "going to show frame $frame of $self->{total_frames} at offset $offset\n";
    my ($c, $Ptype, $number, $reserved) = unpack ("I C C S", substr($self->{binindex}, $offset, 8));
        #print "Got $c, $Ptype, $number, $reserved\n";
    if ($Ptype == 1) {
        $self->VR_dprint("returning $c : $number\n") if $self->{debug};
        return "$c : $number" ;
    } else {
        return undef;
    }
} 


}